module  stats_mod

  public  :: calcmean, calcmedian,sort
!  private :: sort

  contains

  real function calcmean(q) result(mean)
    real, intent(in)  :: q(:)
    integer           :: n
    n = size(q); mean=0.
    mean=real(sum(dble(q))/dble(n))
  end function

  
  real function calcmedian(q) result(median)
    real, intent(inout) :: q(:)
    integer             :: n
    n = size(q); median=0.
    call sort(q)
    median = (q(n/2+1)+q(n/2))/2.
  end function

  real function calcquantile(q,k) result(quant)
    real, intent(inout) ::   q(:)
    real                ::     k
    integer             :: n
    n = size(q); quant=0.; k = k
    call sort(q)
    quant = (q(n*k/100.+1)+q(n*k/100.))/2.
  end function
  
  
  subroutine sort(q)
    real                :: q(:)
    integer             :: i,n
    real                :: p(size(q))
    p = 0.
    n = size(q)
    do i=1,n
      p(                  i   )=       minval(q)
      q(int(minval(minloc(q))))=2.*abs(maxval(q))
    end do
    q = p
  end subroutine


  real function calcstdv(q) result(stdv)
    real, intent(in)  :: q(:)
    double precision  :: stdval, mean
    integer           :: n,i
    n = size(q);
    mean = 0.
    stdval = 0.
    mean=real(sum(dble(q))/dble(n))
    do i=1,n
      stdval = stdval + (dble(q(i))-mean)**2.
    end do
    stdv = real(sqrt( stdval/dble(n*(n-1))))
  end function


end module
