program sdrAnisoArrow
  ! Program to take output of a anisotropic program, azimuth and magnitude.
  ! And create vectors to be drawn with 'Graph'
  
  use sep
  implicit none

  
  integer                :: n1,n2,n3
  real                   :: d1,d2,d3
  real                   :: o1,o2,o3
  
  real   ,allocatable    :: datain(:,:,:),head_xy(:,:),angv(:)
  complex,allocatable    :: datout(:,:)
  
  integer                :: ii,jj,kk,it
  integer                :: ntr
  real                   :: c1,c2,unit_hdr,unit_dat

  real                   :: mg,mgn
  
  logical                :: nomag,normag,domask,mask

  real                   :: rmax
  real                   :: angmin,angrmax
  real                   :: value,rad,ang
  integer                :: nang

  real                   :: fact,clip

  integer                :: nkept
  
  real,parameter         :: pi=3.14159265

  call sep_init()
  
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  
  call from_history('d1',d1,0.)
  call from_history('d2',d2,0.)
  call from_history('d3',d3,0.)
  
  call from_param('nomag' , nomag,.false.)
  call from_param('normag',normag, .true.)
  call from_param('mask'  ,domask,.false.)

  call from_param('fact',fact, 1.)
  call from_param('clip',clip,-1.)
  
  call from_param("unit_hdr",unit_hdr,1.)
  call from_param("unit_dat",unit_dat,1.)
  call from_param("value",value,-1.)

  rmax=sqrt( ((n1-1)*d1)**2.+((n2-1)*d2)**2.)
  call from_param("rmax",rmax, rmax)
  call from_param( "angmin",angmin ,-1.)
  call from_param("angrmax",angrmax,25.)
  angmin = angmin/180*pi

  ! Allocate
  allocate(datain(n1,n2, 3))
  allocate(datout( 2,n1*n2))


  ! Read data
  call sreed("in",datain,4*size(datain))

  ! Apply clip
  if (clip.ne.-1.) then
    do jj=1,n2
    do ii=1,n1
      if (datain(ii,jj,2).gt.clip) then
        datain(ii,jj,2)=clip
      end if
    end do
    end do
  end if
  
  ! Determine center
  do jj=1,n2
    c2 = o2 + (jj-1)*d2
  do ii=1,n1
    c1 = o1 + (ii-1)*d1
    
    if (nomag) then
      mg = fact*d1/2.
    else if (normag) then
      if (datain(ii,jj,2).ne.0.) then
        mg = fact*d1/2*datain(ii,jj,2)/maxval(datain(:,:,2))
      else
        mg = 0.
      end if
    else
      mg = fact*d1/2*datain(ii,jj,2)
    end if
    write(0,*) 'mg',mg
    
    datout(1,(jj-1)*n1+ii) = cmplx( c1+mg*cos(datain(ii,jj,3)), c2+mg*sin(datain(ii,jj,3)) )
    datout(2,(jj-1)*n1+ii) = cmplx( c1-mg*cos(datain(ii,jj,3)), c2-mg*sin(datain(ii,jj,3)) )
    
  end do
  end do
  
  
  
  nkept = n1*n2
  if (domask) then
  
  call from_aux("header","n2",ntr,1)
  allocate(head_xy(2,ntr),angv(4))
  call sreed("header",head_xy,4*2*ntr)
  head_xy = head_xy*unit_hdr
  
  
  nkept = 0
  ! Optionally apply mask
  do jj=1,n2
  do ii=1,n1
    
    mask=.true.
    
    ! Apply the first acceptance condition
    do it=1,ntr
      
      rad = sqrt ( (o1+(ii- 1   )*d1-head_xy(1,it))**2. + (o2+(jj-1)*d2-head_xy(2,it))**2. )
   !   write(0,*) "rad,rmax",rad,rmax
      if ( rad.le.rmax) then
        mask = .false.
      end if
      
    end do
    
    ! Apply the second acceptance condition
    if (angmin.ne.-1./180*pi) then
    if (mask) then
    
    nang = 0
    angv = 0.
    
    do it=1,ntr
      rad = sqrt ( (o1+(ii- 1   )*d1-head_xy(1,it))**2. +    (o2+(jj-1)*d2-head_xy(2,it))**2. )
      ang = atan2( (o1+(ii- 1   )*d1-head_xy(1,it))     ,    (o2+(jj-1)*d2-head_xy(2,it))     )
      if (rad.le.angrmax) then
        if ( ang .ge.-1. *pi .and. ang .lt.- .5*pi ) then;  angv(1) = 1;  end if
        if ( ang .ge.- .5*pi .and. ang .lt. 0.     ) then;  angv(2) = 1;  end if
        if ( ang .ge. 0.     .and. ang .lt.  .5*pi ) then;  angv(3) = 1;  end if
        if ( ang .ge.  .5*pi .and. ang .lt.     pi ) then;  angv(4) = 1;  end if
      end if
    end do
    if (sum(angv).eq.4) then
      mask = .false.
    end if
    
    end if
    end if
   
    ! Actual masking
    if (.not.mask) then
      nkept = nkept+1
      datout(:,nkept) = datout(:,(jj-1)*n1+ii)
    end if    
    
  end do
  end do
  
  end if

  write(0,*) "rmax",rmax
  write(0,*) "nkept",nkept


  ! Fnish header
  call to_history('n1',    2)
  call to_history('d1',   1 )
  call to_history('o1',   0.)
  
  call to_history('n2',nkept)
  call to_history('d2',   1 )
  call to_history('o2',   0.)
  
  call to_history('n3',   1 )
  call to_history('d3',   1 )
  call to_history('o3',   0.)
  
  call to_history('n4',   1 )
  call to_history('d4',   1 )
  call to_history('o4',   0.)
  
  call to_history('esize',8)
  call sep_close()


  ! Write Arrows as complex numbers
  call srite("out",datout(:,1:nkept)*unit_dat,8*2*nkept)

  ! Deallocate
  deallocate(datain)
  deallocate(datout)



end program
