program sdrEikonalTomo
  ! Program taking a series oif traveltime surfaces as input,
  ! calculating the locat derivative and performing statistics to obtain
  ! output images

  use sep
  implicit none

  real,allocatable               :: data(:,:),derivm(:,:),deriva(:,:)
  integer                        :: n1,n2,n3
  character*50                   :: label1,label2,label3
  real                           :: d1,d2,d3
  real                           :: o1,o2,o3
  integer                        :: ii,jj,kk
  real                           :: dfact1,dfact2
  
  
  call sep_init()
  
  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )
  
  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  call from_history("d3",d3,1.)
  
  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  call from_history("o3",o3,0.)
  
  call from_history("label1",label1," ")
  call from_history("label2",label2," ")
  call from_history("label3",label3," ")

  call to_history("n4",2)
  call to_history("label4","(magnitude,azimuth)")
  
  call sep_close()

  
  ! Alocate stuff
  allocate (data(n1,n2),derivm(n1,n2),deriva(n1,n2))
  
  ! Determine derivative factors
  dfact1 = 1./(d2*2)
  dfact2 = 1./(d2*2)**2.

  do kk=1,n3
    
    ! Zero values
    data   = 0.
    derivm = 0.
    deriva = 0.
    
    ! Read new traveltime surface
    call sreed("in",data,4*n1*n2)
    
    do jj=2,n2-1
    do ii=2,n1-1
      
      ! Magnitude of gradient
      derivm(ii,jj) = sqrt ( dfact2*(data(ii+1,jj)-data(ii-1,jj))**2. + dfact2*(data(ii,jj+1)-data(ii,jj-1))**2. )
      
      ! Azimuth of gradient
      deriva(ii,jj) = atan2( dfact1*(data(ii+1,jj)-data(ii-1,jj))     , dfact1*(data(ii,jj+1)-data(ii,jj-1))     )
      
    end do
    end do
    
    derivm(1:n1, 1) = derivm(1:n1, 1+1)
    derivm(1:n1,n2) = derivm(1:n1,n2-1)
    derivm( 1,1:n2) = derivm( 1+1,1:n2)
    derivm(n1,1:n2) = derivm(n1-1,1:n2)

    deriva(1:n1, 1) = deriva(1:n1, 1+1)
    deriva(1:n1,n2) = deriva(1:n1,n2-1)
    deriva( 1,1:n2) = deriva( 1+1,1:n2)
    deriva(n1,1:n2) = deriva(n1-1,1:n2)
    
    ! Write out magnitude and azimuth
    call srite_window("out",4,(/n1,n2,n3,2/),(/n1,n2,1,1/),(/0,0,(kk-1),0/),(/1,1,1,1/),4,derivm)
    call srite_window("out",4,(/n1,n2,n3,2/),(/n1,n2,1,1/),(/0,0,(kk-1),1/),(/1,1,1,1/),4,deriva)
    
  end do
  
  ! Deallocate stuff
  deallocate(data,derivm,deriva)
  
  
end program
