program sdrEikonalTomoCStats
  ! Program to analyse statistics and compile
  ! images from the result of Eikonal Tomography
  ! Calculates Average and Standard deviation

  ! Standard out: s or c, set mode: convert 

  use sep
  implicit none

  real,allocatable               :: mdata(:,:,:),mean(:,:),stdv(:,:),header(:,:)
  integer                        :: n1,n2,n3,n1h,n2h
  character*50                   :: label1,label2,label3
  real                           :: d1,d2,d3
  real                           :: o1,o2,o3
  integer                        :: ii,jj,kk
  integer                        :: cntr, sx_key, sy_key
  
  double precision               :: meanval,stdvval
  real                           :: dist,minr,maxr
  logical                        :: convert
  
  call sep_init()
  
  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )

  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  call from_history("d3",d3,1.)
  
  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  call from_history("o3",o3,0.)
  
  call from_history("label1",label1," ")
  call from_history("label2",label2," ")
  call from_history("label3",label3," ")
  
  call from_param("convert",convert,.true.)
  
  call from_param("minr",minr,0.    )  ! Only when header provided
  call from_param("maxr",maxr,15000.)
  
  call to_history("n3",2 )
  call to_history("label3","(mean,stnd-dev)")
  
  
  if (exist_file('header')) then
    call from_aux("header","n1",n1h,1)
    call from_aux("header","n2",n2h,1)
    call from_param( "sx_key" ,sx_key,1)
    call from_param( "sy_key" ,sy_key,2)
  end if
  
  call sep_close()
  
  
  ! Alocate stuff
  allocate (mdata(n1,n2,n3),mean(n1,n2),stdv(n1,n2),header(n1h,n2h))
  
  ! Read new slowness cube
  call sreed("in",mdata,4*n1*n2*n3)
  
  if (exist_file('header')) then
    ! Read source position file
    call sreed("header",header,4*n1h*n2h)
    
    do jj=1,n2
    do ii=1,n1
      
      meanval = 0
      cntr    = 0
      ! Mean of slowness
      do kk=1,n3
        dist = sqrt( (o1+(ii-1)*d1-header(sx_key,kk))**2. + (o2+(jj-1)*d2-header(sy_key,kk))**2. )
      !  write(0,*) dist
        if ( dist.ge.minr .and. dist.le.maxr ) then
          cntr    = cntr    + 1
          meanval = meanval +   dble( mdata(ii,jj,kk) )
        end if
      end do
      mean(ii,jj) = real( meanval / dble(cntr) )
      if (mean(ii,jj).ne.mean(ii,jj)) then; mean(ii,jj)=0.; end if
      
      
      stdvval = 0
      cntr    = 0
      ! Standard deviation of slowness
      do kk=1,n3
        dist = sqrt( (o1+(ii-1)*d1-header(sx_key,kk))**2. + (o2+(jj-1)*d2-header(sy_key,kk))**2. )
      !  write(0,*) dist
        if ( dist.ge.minr .and. dist.le.maxr ) then
          cntr    = cntr    + 1
          stdvval = stdvval + ( dble( mdata(ii,jj,kk) ) - dble( mean(ii,jj) ) )**2.
        end if
      end do
      stdv(ii,jj) = real(sqrt( stdvval/ (dble(cntr)*(dble(cntr)-1)) ))
      if (stdv(ii,jj).ne.stdv(ii,jj)) then; stdv(ii,jj)=0.; end if

      ! Convert slowness to velocity
      if (convert) then
        if(mean(ii,jj).ne.0) then
          mean(ii,jj) = real(1./dble(mean(ii,jj)))
          stdv(ii,jj) = real(dble(stdv(ii,jj))*dble(mean(ii,jj))**2.)
        end if
      end if
      
    end do
    end do
    
  else

    do jj=1,n2
    do ii=1,n1
    
      ! Mean of slowness
      mean(ii,jj) =      real( sum(  dble(mdata(ii,jj,:))) / dble(n3) )
    
      ! Standard deviation of slowness
      stdv(ii,jj) = real(sqrt(sum( (dble(mdata(ii,jj,:))-dble(mean(ii,jj)))**2. )/(dble(n3)*(dble(n3)-1))))
    
      ! Convert slowness to velocity
      if (convert) then
        if(mean(ii,jj).ne.0) then
          mean(ii,jj) = real(1./dble(mean(ii,jj)))
          stdv(ii,jj) = real(dble(stdv(ii,jj))*dble(mean(ii,jj))**2.)
        end if
      end if
     ! write(0,*)  mean(ii,jj), stdv(ii,jj)
    
    end do
    end do

  end if
  
  ! Write out Mean and Ave
  call srite("out",mean,4*n1*n2)
  call srite("out",stdv,4*n1*n2)
  
  ! Deallocate stuff
  deallocate(mdata,mean,stdv)
  
  call exit(0)
  
end program
