program sdrEtomoAnisoCStat
  ! Program to investigate the anisotropic properties obtained through Eikonal Tomography.


  use sep
  implicit none

  
  integer                        :: n1,n2,n3,n4,n1h,n2h
  real                           :: d1,d2,d3
  real                           :: o1,o2,o3
  
  character*50                   :: label1,label2,label3,label4
  
  integer                        :: ii,jj,kk,ia
  
  real,allocatable               :: datain(:,:,:,:),header(:,:),mask(:,:)
  real,allocatable               :: datout(:,:,:,:)
  
  logical                        :: convert,fit
  
  real                           :: dang,oang
  integer                        :: nang,sx_key,sy_key,is1,is2
  
  real                           :: dist,minr,maxr


  double precision,allocatable   :: meanval(:),cntr(:),stdvval(:)

  real,parameter                 :: pi=3.14159265
  
  call sep_init()
  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )
  
  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  call from_history("d3",d3,1.)
  
  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  call from_history("o3",o3,0.)
  
  call from_history("label1",label1,' ')
  call from_history("label2",label2,' ')
  call from_history("label3",label3,' ')
  call from_history("label4",label4,' ')

  call from_param("nang",nang, 360)

  call from_param("convert",convert,.true. )

  oang =   -pi
  dang = 2.*pi/(nang-1)
  
  call to_history("n3",nang )
  call to_history("n4",  2  )
  call to_history("d3",dang )
  call to_history("o3",oang )
  call to_history("label4","Mean,Stddev")
  
  call from_param("minr",minr,0.    )
  call from_param("maxr",maxr,15000.)

  call from_aux("header","n1",n1h,1)
  call from_aux("header","n2",n2h,1)
  call from_param( "sx_key" ,sx_key,1)
  call from_param( "sy_key" ,sy_key,2)
  
  call sep_close()
  
  
  ! Allocate
  allocate(datain(n1,n2,n3  ,2));  datain = 0.
  allocate(datout(n1,n2,nang,2));  datout = 0.
  allocate(header ( n1h , n2h ));  header = 0.
  allocate(stdvval(     nang  ));
  allocate(meanval(     nang  ));
  allocate(  cntr (     nang  ));
  allocate(mask(n1,n2)); mask=0.
  
  ! read data
  call sreed("in",datain,4*size(datain))
  
  ! Read source position file
  call sreed("header",header,4*n1h*n2h)

  ! Read Mask
  call sreed("mask",mask,4*size(mask))
  
  ! Loop over space
  do jj=1,n2
  do ii=1,n1
    
    meanval = 0.
    cntr    = 0.
    
    ! For measurements from each shot
    do kk=1,n3
      
      ! Is station inside the mask?
      is1 = nint( (header(sx_key,kk)-o1)/d1 + 1. )
      is2 = nint( (header(sy_key,kk)-o2)/d2 + 1. )
      if(is1.ge.1..and.is1.le.n1.and.is2.ge.1..and.is2.le.n2) then
        if ( mask(is1,is2).eq.1 ) then
          dist = sqrt( (o1+(ii-1)*d1-header(sx_key,kk))**2. + (o2+(jj-1)*d2-header(sy_key,kk))**2. )
          if ( dist.ge.minr .and. dist.le.maxr ) then
            ! Determine which angle indice
            ia = nint( (datain(ii,jj,kk,2) - oang)/dang ) + 1
            meanval(ia) = meanval(ia) + dble( datain(ii,jj,kk,1) )
               cntr(ia) =    cntr(ia) + 1.
          end if
        end if
      end if

    end do
    
  !  write(0,*) "cntr",ii,jj,cntr

    do ia=1,nang
      if (cntr(ia).gt.1.) then
        meanval(ia) = meanval(ia)/cntr(ia)
      else
        meanval(ia) = 0.
      end if
    end do
    
  !  write(0,*) "meanval",ii,jj,meanval

    stdvval = 0.
    cntr    = 0.
    
    ! Standard deviation of slowness
    do kk=1,n3
      
      ! Is station inside the mask?
      is1 = nint( (header(sx_key,kk)-o1)/d1 + 1. )
      is2 = nint( (header(sy_key,kk)-o2)/d2 + 1. )
      if(is1.ge.1..and.is1.le.n1.and.is2.ge.1..and.is2.le.n2) then
        if ( mask(is1,is2).eq.1 ) then
          
          dist = sqrt( (o1+(ii-1)*d1-header(sx_key,kk))**2. + (o2+(jj-1)*d2-header(sy_key,kk))**2. )
          if ( dist.ge.minr .and. dist.le.maxr ) then
            ! Determine which angle indice
            ia = nint( (datain(ii,jj,kk,2) - oang)/dang ) + 1
            stdvval(ia) = stdvval(ia) + ( dble( datain(ii,jj,kk,1) ) - meanval(ia) )**2.
               cntr(ia) =    cntr(ia) + 1.
          end if
        end if
      end if
      
    end do

    
    do ia=1,nang
      stdvval(ia) = sqrt( stdvval(ia)/ (cntr(ia)*(cntr(ia)-1)) )
      if (stdvval(ia).ne.stdvval(ia)) then; stdvval(ia)=0.; end if
      if (stdvval(ia).eq.   1./0.   ) then; stdvval(ia)=0.; end if
      if (stdvval(ia).eq.  -1./0.   ) then; stdvval(ia)=0.; end if
    end do

  !  write(0,*) "stdvval",ii,jj,stdvval

    
    ! Convert slowness to velocity
    if (convert) then
      do ia=1,nang
        if(meanval(ia).ne.0) then
          datout(ii,jj,ia,1) = real(     1.    /meanval(ia)    )
          datout(ii,jj,ia,2) = real(stdvval(ia)/meanval(ia)**2.)
        end if
      end do
    else
      do ia=1,nang
        if(meanval(ia).ne.0) then
          datout(ii,jj,ia,1) = real(meanval(ia))
          datout(ii,jj,ia,2) = real(stdvval(ia))
        end if
      end do
    end if
    
  end do
  end do
  
  ! Write data
  call srite("out",datout,4*size(datout))
  
  
  ! Deallocate
  deallocate(datain)
  deallocate(datout)
  deallocate(stdvval)
  deallocate(meanval)
  deallocate(cntr  )
  deallocate(mask  )
  
end program
