program sdrEtomoAnisoFit
  ! Program to fit anisotropic velocities by grid search


  use sep
  implicit none

  
  integer                        :: n1,n2,n3,n4,n1h,n2h
  real                           :: d1,d2,d3
  real                           :: o1,o2,o3
  
  character*50                   :: label1,label2,label3,label4
  
  integer                        :: ii,jj,kk
  
  real,allocatable               :: datain(:,:,:,:),mask(:,:)
  real,allocatable               :: datout(:,:,:)
  real,allocatable               :: misfit(:,:),header(:,:)
  
  logical                        :: convert,fit
  
  real                           :: dv,ov
  real                           :: da,oa
  integer                        :: ia,na,iv,nv
  real                           :: ang,vel,vel0
  integer                        :: sx_key,sy_key,is1,is2
  
  real                           :: dist,minr,maxr,value
  real                           :: norm,funcval,cntr

  real,parameter                 :: pi=3.14159265
  
  call sep_init()

  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )
  
  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  call from_history("d3",d3,1.)
  
  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  call from_history("o3",o3,0.)
  
  call from_history("label1",label1,' ')
  call from_history("label2",label2,' ')
  call from_history("label3",label3,' ')
  
  call from_param("nv",nv,  75)
  call from_param("ov",ov,  0.)
  call from_param("dv",dv,  1.)
  
  call from_param("na",na,   90  )
  call from_param("oa",oa,   0.  )
  call from_param("da",da, pi/na )
  da = da/180.*pi
  oa = oa/180.*pi
  
  call to_history("n1",n1 )
  call to_history("n2",n2 )
  
  call to_history("n3", 3 )
  call to_history("d3",  1. )
  call to_history("o3",  0. )
  
  call to_history("n4",  1  )
  call to_history("d4",  1. )
  call to_history("o4",  0. )
  
  call from_param("norm",norm,2.)
  
  call from_param("minr",minr,0.    )
  call from_param("maxr",maxr,15000.)
  
  call from_aux("header","n1",n1h,1)
  call from_aux("header","n2",n2h,1)
  call from_param( "sx_key" ,sx_key,1)
  call from_param( "sy_key" ,sy_key,2)
  
  call sep_close()
  
  
  ! Allocate
  allocate(datain(n1,n2,n3  ,2));  datain = 0.
  allocate(datout(n1,n2,     3));  datout = 0.
  allocate(misfit(nv,na       ));  misfit = 0.
  allocate(header ( n1h , n2h ));  header = 0.
  allocate(mask(n1,n2)); mask=0.
  
  ! read data
  call sreed("in",datain,4*size(datain))
  
  ! Read source position file
  call sreed("header",header,4*n1h*n2h)

  ! Read Mask
  call sreed("mask",mask,4*size(mask))

  ! Convert to Velocity
  do kk=1,n3
  do jj=1,n2
  do ii=1,n1
    if (datain(ii,jj,kk,1).ne.0.) then
      datain(ii,jj,kk,1) = 1./datain(ii,jj,kk,1)
    end if
  end do
  end do
  end do
  
  ! Loop over space
  do jj=1,n2
  do ii=1,n1
    write(0,*) "(ii,jj)",ii,jj,"/",n1,n2
    
    ! Determine average velocity
    vel0 = 0.
    cntr = 0
    do kk=1,n3
        if (datain(ii,jj,kk,1).ne.0.) then
          ! Is station inside the mask?
          is1 = nint( (header(sx_key,kk)-o1)/d1 + 1. )
          is2 = nint( (header(sy_key,kk)-o2)/d2 + 1. )
          if(is1.ge.1..and.is1.le.n1.and.is2.ge.1..and.is2.le.n2) then
            if ( mask(is1,is2).eq.1 ) then

              dist = sqrt( (o1+(ii-1)*d1-header(sx_key,kk))**2. + (o2+(jj-1)*d2-header(sy_key,kk))**2. )
              if ( dist.ge.minr .and. dist.le.maxr ) then
                cntr    = cntr + 1
                vel0 = vel0 + datain(ii,jj,kk,1)
              end if

            end if
          end if
        end if
    end do
    vel0 = vel0/cntr
    
    misfit = 0.
    do ia=1,na
    do iv=1,nv

      ang = oa + (ia-1)*da
      vel = ov + (iv-1)*dv
      
      cntr = 0
      ! Calculate total misfit
      do kk=1,n3
        if (datain(ii,jj,kk,1).ne.0.) then
          ! Is station inside the mask?
          is1 = nint( (header(sx_key,kk)-o1)/d1 + 1. )
          is2 = nint( (header(sy_key,kk)-o2)/d2 + 1. )
          if(is1.ge.1..and.is1.le.n1.and.is2.ge.1..and.is2.le.n2) then
            if ( mask(is1,is2).eq.1 ) then

              dist = sqrt( (o1+(ii-1)*d1-header(sx_key,kk))**2. + (o2+(jj-1)*d2-header(sy_key,kk))**2. )
              if ( dist.ge.minr .and. dist.le.maxr ) then

                cntr    = cntr + 1
                funcval = vel0 + vel*cos(2*(datain(ii,jj,kk,2)-ang))
                misfit(iv,ia) = misfit(iv,ia) + abs( funcval - datain(ii,jj,kk,1) )**norm

              end if

            end if
          end if
        end if
      end do
      
      ! Normalize misfit
      misfit(iv,ia) = misfit(iv,ia) / cntr
      
    end do
    end do
    
    ! Pick minimum
    datout(ii,jj,1  ) = vel0
    datout(ii,jj,2:3) = minloc(misfit)
    
  end do
  end do

  ! Convert to phyical units  
  datout(:,:,2) = ov + (datout(:,:,2)-1)*dv
  datout(:,:,3) = oa + (datout(:,:,3)-1)*da
  
  ! Write data
  call srite("out",datout,4*size(datout))
  
  ! Deallocate
  deallocate(datain)
  deallocate(datout)
  deallocate(misfit)
  deallocate(header)
  deallocate( mask )
  
end program
