program sdrMaskValhall
  ! Program to apply a mask to Data and set to predetermined datavalue
  
  use sep
  implicit none
  
  integer             :: n1,n2,n3
  real                :: d1,d2
  real                :: o1,o2
  
  real,allocatable    :: data(:,:),head_xy(:,:),angv(:)
  
  real                :: value,unit
  real                :: eclip,bclip

  real                :: rmax
  real                :: angmin,angrmax

  logical             :: mask,getmask

  real                :: rad,ang

  integer             :: ii,jj,it,kk
  integer             :: ntr,nang
  
  real,parameter      :: pi=3.14159265

  call sep_init()
  
  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )
  
  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  
  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  
  call from_aux("header","n1",ntr,1)
  
  write(0,*) "ntr",ntr
  
  call from_param("unit",unit,1.)
  call from_param("value",value,-1.)
  call from_param("getmask",getmask,.false.)

  rmax=sqrt( ((n1-1)*d1)**2.+((n2-1)*d2)**2.)
  call from_param("rmax",rmax, rmax)

  call from_param( "angmin",angmin ,-1.)
  call from_param("angrmax",angrmax,25.)
  angmin = angmin/180*pi

  call from_param("bclip",bclip,-pi*9.99)  ! Some crazy default
  call from_param("eclip",eclip,-pi*9.99)
  
  call sep_close()
  
  ! Allocate
  allocate(data(n1,n2))
  allocate(head_xy(2,ntr),angv(4))
  
  
  ! Read data
  call sreed("header",head_xy,4*2*ntr)
  head_xy = head_xy*unit
  do kk=1,n3
  call sreed("in",data,4*n1*n2)
  if (getmask) then; data = 1.; end if


  ! Do artificial clip
  if (bclip.ne.-pi*9.99 .and. eclip.ne.-pi*9.99) then
  do ii=1,n1
  do jj=1,n2
    if (data(ii,jj).lt.bclip) then; data(ii,jj) = bclip; end if
    if (data(ii,jj).gt.eclip) then; data(ii,jj) = eclip; end if
  end do
  end do
  else if (bclip.ne.-pi*9.99) then
  do ii=1,n1
  do jj=1,n2
    if (data(ii,jj).lt.bclip) then; data(ii,jj) = bclip; end if
  end do
  end do
  else if (eclip.ne.-pi*9.99) then
  do ii=1,n1
  do jj=1,n2
    if (data(ii,jj).gt.eclip) then; data(ii,jj) = eclip; end if
  end do
  end do
  end if
  
  
  
  ! Apply mask
  do jj=1,n2
  do ii=1,n1
    
    mask=.true.
    
    ! Apply the first acceptance condition
   ! write(0,*) "Do first test"
    do it=1,ntr
      
      rad = sqrt ( (o1+(ii- 1   )*d1-head_xy(1,it))**2. + (o2+(jj-1)*d2-head_xy(2,it))**2. )
      
      if ( rad.le.rmax) then
        mask = .false.
      end if
      
    end do
    
    ! Apply the second acceptance condition
!    if (angmin.ne.-1./180*pi) then
  !  write(0,*) "Do second test"
    if (mask) then
    
    nang = 0
    angv = 0.
    
    do it=1,ntr
      rad = sqrt ( (o1+(ii- 1   )*d1-head_xy(1,it))**2. +    (o2+(jj-1)*d2-head_xy(2,it))**2. )
      ang = atan2( (o1+(ii- 1   )*d1-head_xy(1,it))     ,    (o2+(jj-1)*d2-head_xy(2,it))     )
      if (rad.le.angrmax) then
        if ( ang .ge.-1. *pi .and. ang .lt.- .5*pi ) then;  angv(1) = 1;  end if
        if ( ang .ge.- .5*pi .and. ang .lt. 0.     ) then;  angv(2) = 1;  end if
        if ( ang .ge. 0.     .and. ang .lt.  .5*pi ) then;  angv(3) = 1;  end if
        if ( ang .ge.  .5*pi .and. ang .lt.     pi ) then;  angv(4) = 1;  end if
      end if
    end do
    
    
    
    if (sum(angv).eq.4) then
      mask = .false.
    end if
    
!    end if
    end if
    
    ! Actual masking
    if (mask) then
      data(ii,jj) = value
    end if    
    
  end do
  end do
  
  ! Write data
  call srite("out",data,4*n1*n2)
  
  end do

  ! Deallocate
  deallocate(data)
  deallocate(head_xy,angv)
  call exit()
  
end program
