program sdrRotate
  ! Program to rotate a perfectly nice rectangular grid by an angle,
  ! Use linear interpolation. Pulling algorithm.
  
  
  use sep
  implicit none
  
  real,allocatable :: datain(:,:),dataout(:,:)
  
  integer          :: n1,n2,n3
  real             :: d1,d2,d3
  real             :: o1,o2,o3
  integer          :: n1out,n2out
  integer          :: c1,c2
  integer          :: i1,i2
  integer          :: c1out,c2out
  
  real             :: rot,shift1,shift2
  real             :: ang,rad
  
  integer          :: ii,jj,kk

  real,parameter   :: pi=3.14159265
  
  
  call sep_init()
  
  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )
  
  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  call from_history("d3",d3,1.)
  
  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  call from_history("o3",o3,0.)
  
  call from_param("rotate",rot,0.)
  call from_param("shift1",shift1,0.)
  call from_param("shift2",shift2,0.)
  call from_param("n1out",n1out,n1)
  call from_param("n2out",n2out,n2)

  call to_history("n1",n1out )
  call to_history("n2",n2out )

  call to_history("o1",o1+(n1-n1out)/2*d1 )
  call to_history("o2",o2+(n2-n2out)/2*d2 )

  call to_history("n1",n1out )
  call to_history("n2",n2out )

  call from_history("n3",n3,1 )

  rot    = rot*pi/180
  shift1 = shift1/d1
  shift2 = shift2/d2
  
  call sep_close()
  
  c1    = n1/2
  c2    = n2/2
  c1out = n1out/2
  c2out = n2out/2
  
  ! Allocate
  allocate(datain(n1,n2),dataout(n1out,n2out))
  
  do kk=1,n3
  
  datain  = 0.
  dataout = 0.
  
  call sreed("in" ,datain ,4*n1*n2)
  
  
  do jj=1,n2out
  do ii=1,n1out
    
    rad = sqrt ( real(ii-c1out)**2. + real(jj-c2out)**2. )
    ang = atan2( real(ii-c1out)     , real(jj-c2out)     )
    
    ang = ang+rot
     
    i1 = nint(c1+rad*sin(ang)-shift1)
    i2 = nint(c2+rad*cos(ang)-shift2)

    if (i1.ge.1 .and. i1.le.n1) then
    if (i2.ge.1 .and. i2.le.n2) then
      dataout(ii,jj) = datain( i1 , i2 )
    end if
    end if
    
  end do
  end do
  
  call srite("out",dataout,4*n1out*n2out)

  end do
  
  ! Deallocate
  deallocate(datain,dataout)
  call exit()
  
end program
