program EikonalTomo


  use sep
  use regurize2d_splines_mod
  implicit none
  
  ! general
  integer          :: nt,nr,ns
  
  integer          :: n1,n2,n3
  real             :: d1,d2,d3
  real             :: o1,o2,o3
  character*50     :: label1,label2,label3
  character*50     :: label4,label5,label6
  
  integer          :: it,ix,iy,ir,is,cntr
  
  ! For tomography grid
  integer          :: nx,ny
  real             :: dx,dy,ox,oy
  
  ! Memory variables
  real,allocatable,dimension(:  ) :: datain,datpos,datneg,tdat,treg,headerin, pckept,t
  real,allocatable,dimension(:,:) :: hkept,headeout,rpos
  real,allocatable,dimension(:,:,:)  :: trpcks
  real,allocatable,dimension(:,:,:)  :: tsurfs
  real,allocatable,dimension(:,:,:)  :: derivm
  real,allocatable,dimension(:,:,:)  :: deriva
  real,allocatable,dimension(:,:,:)  :: eitomo

  
  real             :: value(10)
  
  logical          :: slowness
  
  ! Controll values for travel time picking
  real             :: v,maxp
  real             :: tau0,dtau,rad,snr,ang,Msnr
  integer          :: wine,winb,en,bp,nn,np,ne
  integer          :: t0,dt,cnt
  
  real             :: sn_pos, sn_neg
  real             :: sig_in,sig_out
  real             :: dfact1,dfact2
  real             :: meanval,stdvval,dist,minr,maxr

  ! Header-key #'s
  integer          :: nh1
  integer          :: sx_key,sy_key,rx_key,ry_key,dd_key


  write(0,*) '-----------------------------------'
  write(0,*) '       EIKONAL TOMOGRAPHY          '
  write(0,*) '-----------------------------------'
  ! Header IO
  call sep_init()
  
  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )

  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  call from_history("d3",d3,1.)

  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  call from_history("o3",o3,0.)

  call from_history("label1",label1,' ')
  call from_history("label2",label2,' ')
  call from_history("label3",label3,' ')

  ! Determine number of receivers
  nr = n2;
  ns = n3;
  call from_param("nr",nr,n2)
  call from_param("ns",ns,n3)
  write(0,*) ' determined nr,ns:',nr,ns


  ! Velocity estimate for Envelope-picker
  call from_param('v'   ,v   ,1000. ) ! Guessed V
  call from_param('dtau',dtau, 1.   ) ! Allowed deviation from v*dx
  call from_param('snr' ,Msnr, 1.   ) ! Minimum snr
  dt = nint(dtau/d1)
  
  ! Get header and header-key #'s 
  if (exist_file('header')) then
    call from_aux("header","n1",nh1,1)
    call from_param( "sx_key" ,sx_key,1)
    call from_param( "sy_key" ,sy_key,2)
    call from_param( "rx_key" ,rx_key,4)
    call from_param( "ry_key" ,ry_key,5)
    call from_param("dead_key",dd_key,7)
  end if

  ! Get output grid from param
  call from_param("nx",nx)
  call from_param("ny",ny)
  call from_param("dx",dx)
  call from_param("dy",dy)
  call from_param("ox",ox)
  call from_param("oy",oy)

  ! Minimum and maximum offsets for statistical characterization
  call from_param("minr",minr,0.    )  ! Only when header provided
  call from_param("maxr",maxr,15000.)


  ! Create output History
  call to_history("n1",nx)
  call to_history("n2",ny)
  call to_history("n3", 4)  

  call to_history("d1",dx)
  call to_history("d2",dy)
  call to_history("d3",1.)  

  call to_history("o1",ox)
  call to_history("o2",oy)
  call to_history("o3",1.)
    
  call to_history("label1","x position")
  call to_history("label1","y position")
  call to_history("label1","(mean,stddev,#picks,#estimates)")

  call to_history("hff",-1)
  call to_history("gff",-1)

  call sep_close()

  ! Construct output grid
  allocate(headeout(nx*ny,2)); headeout=0
  do iy=1,ny
  do ix=1,nx
    headeout((iy-1)*nx+ix,:) = (/ ox+(ix-1)*dx , oy+(iy-1)*dy /)
  end do
  end do



  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Travel time picking section !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(0,*) '-----------------------------------'
  write(0,*) '     Start Travel Time Picking     '

  ! Determine some indices in positive and negative time windows
  ! Construct t-axis
  allocate(t(n1)); t=0.
  do it=1,n1;  t(it)=o1+(it-1)*d1;  end do
  do it=1,n1;  if (t(it).lt.0.) then; en = it; end if; end do; bp = en+1
  ! Number of positive and negative elements
  np = n1-en
  nn =  1+en
  ! Number of elements
  ne = maxval((/np,nn/))
  
  ! Allocate some memory
  allocate( datain(n1) ); datain=0.
  allocate( datneg(ne) ); datneg=0.
  allocate( datpos(ne) ); datpos=0.
  allocate( headerin(nh1) ); headerin=0.
  allocate( trpcks(nr,ns,12) ); trpcks=0.
  

  ! Loop over sources and receivers
  do is=1,ns
  do ir=1,nr
  
  datain= 0.
  datneg = 0.
  datpos = 0.

  ! Read headers and data for this trace
  call sreed("header",headerin,4*nh1)
  call sreed("in"    ,datain  ,4*n1 )
  
  ! Sort data into positve and negative windows
  do it= 1,en+1;  datneg(it   ) = datain(bp-it+1);  end do
  do it=bp,n1  ;  datpos(it-en) = datain(   it  );  end do
  datpos(ne)=datpos(ne-1) ! Just suplement one sample
  
  ! Guessed traveltime
  rad = sqrt ( (headerin(rx_key)-headerin(sx_key))**2. + (headerin(ry_key)-headerin(sy_key))**2. )
  ang = atan2( (headerin(ry_key)-headerin(sy_key))     , (headerin(rx_key)-headerin(sx_key))     )
  if (ang.gt. 4.) then;  write(0,*) 'detected'; ang=0;  end if
  if (ang.lt.-4.) then;  write(0,*) 'detected'; ang=0;  end if
  t0 = nint( rad/real(v*d1) + 1. )  ! Index of t0
  
  ! Determine indices of window of interest
  winb = minval((/maxval((/t0-dt, 1/)) , ne/))
  wine = maxval((/minval((/t0+dt,ne/)) ,  1/))
  
  ! Estimate S/N ratio in negative time window
  sig_in  = maxval(datneg(winb:wine))
  sig_out = 0.
  if (winb.gt. 1) then
    sig_out = sig_out + sum(abs( datneg(   1:winb) )) / (winb-   1+1)
  end if
  if (wine.lt.ne) then
    sig_out = sig_out + sum(abs( datneg(wine:  ne) )) / (ne  -wine+1)
  end if
  sn_neg = sig_in/sig_out
  
  ! Estimate S/N ratio in positive time window
  sig_in  = maxval(datpos(winb:wine))
  sig_out = 0.
  if (winb.gt. 1) then
    sig_out = sig_out + sum(abs( datpos(   1:winb) )) / (winb-   1+1)
  end if
  if (wine.lt.ne) then
    sig_out = sig_out + sum(abs( datpos(wine:  ne) )) / (ne  -wine+1)
  end if
  sn_pos = sig_in/sig_out
  
  ! Use maxium snr fro + or - window
  if (sn_pos.ge.sn_neg) then
    ! Use positive window
    maxp  = maxval(winb-1+maxloc(datpos(winb:wine)))
    snr   = sn_pos
  else
    ! Use negative window
    maxp  = maxval(winb-1+maxloc(datneg(winb:wine)))
    snr   = sn_neg
  end if
  
  ! Create picked array
  trpcks(ir,is,1)  = (real(maxp)-1.)*d1    ! Travel time pick
  trpcks(ir,is,2)  = snr                   ! Signal over noise ratio
  trpcks(ir,is,3)  = headerin(rx_key)      ! Receiver x-position
  trpcks(ir,is,4)  = headerin(ry_key)      ! Receiver y-position
  trpcks(ir,is,5)  = headerin(sx_key)      ! Source x-position
  trpcks(ir,is,6)  = headerin(sy_key)      ! Source y-position
  trpcks(ir,is,7)  = rad                   ! Source-receiver distance
  trpcks(ir,is,8)  = ang                   ! Source-receiver azimuth
  trpcks(ir,is,9)  = ir                    ! Receiver nr
  trpcks(ir,is,10) = is                    ! Source nr
  trpcks(ir,is,11) = (real(winb)-1.)*d1    ! Lower limit of picking window
  trpcks(ir,is,12) = (real(wine)-1.)*d1    ! Upper limit of picking window
  
  end do
  end do
  
  deallocate( datain )
  deallocate( datneg )
  deallocate( datpos )
  deallocate( headerin )
  deallocate( t )
  
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Keep some travel time picks and regularize !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(0,*) '-----------------------------------'
  write(0,*) ' Start Travel Time Regularization  '; write(0,*) ' '
  allocate(eitomo(nx,ny,4)); eitomo=0.
  allocate(pckept(nr)); pckept=0.
  allocate(hkept(nr,2)); hkept=0.
  allocate(rpos (nr,2)); rpos=0.
  allocate(tdat (nr)); tdat=0.
  allocate(treg(nx*ny)); treg=0.
  allocate(tsurfs(nx,ny,is)); tsurfs=0.
  
  
  do ir=1,nr ! Get receiver coordinates from first shot
    rpos(ir,:) = (/ trpcks(ir,1,3) , trpcks(ir,1,4) /)
  end do

  ! Loop over virtual source positions
  do is=1,ns
    write(0,*) 'Regularize travel time surface', is, '/', ns
    
    cntr    = 0
    do ir=1,nr
      ! Keep if exceeding a SNR
      if (trpcks(ir,is,2).gt.Msnr) then
        cntr          = cntr + 1
        tdat(cntr   ) = trpcks(ir,is,1)
        hkept(cntr,:) = (/ trpcks(ir,is,3) , trpcks(ir,is,4) /)
        pckept(ir)    = pckept(ir)+1
      end if
    end do
    
    ! Interpolate the number of kept picks
    call regurize2d_isplines_init(hkept(1:cntr,:),headeout)
  
    ! Regularization
    call regurize2d_isplines     (tdat (1:cntr  ),treg)
    
    ! Insert into surfaces array
    do iy=1,ny
    do ix=1,nx
      tsurfs(ix,iy,is) = treg((iy-1)*nx+ix)
    end do
    end do

  end do

  write(0,*) 'Regularize number of kept picks'
  ! Initialize regularizator
  call regurize2d_isplines_init(rpos,headeout)

  ! Regularization
  call regurize2d_isplines(pckept,treg)

  ! Insert into surfaces array
  do iy=1,ny
  do ix=1,nx
      eitomo(ix,iy,3) = treg((iy-1)*nx+ix)
  end do
  end do
  
  
  ! Close some and deallocate 
  call regurize2d_splines_close()  
  deallocate(pckept)
  deallocate(hkept)
  deallocate(tdat)
  deallocate(treg)
  
  
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Determine magnitude and azimuth of derivative !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(0,*) '-----------------------------------'
  write(0,*) '         Compute Gradients         '
  ! Alocate stuff
  allocate(derivm(nx,ny,ns)); derivm=0.
  allocate(deriva(nx,ny,ns)); deriva=0.
  
  ! Determine derivative factors
  dfact1 = 1./(dx*2)
  dfact2 = 1./(dx*2)**2.


  do is=1,ns
    
    do iy=2,ny-1
    do ix=2,nx-1
      
      ! Magnitude of gradient
      derivm(ix,iy,is) = sqrt ( dfact2*(tsurfs(ix+1,iy,is)-tsurfs(ix-1,iy,is))**2. + dfact2*(tsurfs(ix,iy+1,is)-tsurfs(ix,iy-1,is))**2. )
      
    !  ! Azimuth of gradient
    !  deriva(ix,iy,is) = atan2( dfact1*(tsurfs(ix+1,iy,is)-tsurfs(ix-1,iy,is))     , dfact1*(tsurfs(ix,iy+1,is)-tsurfs(ix,iy-1,is))     )
      
    end do
    end do
    
    ! Fill the edges by extension
    derivm(1:nx, 1,is) = derivm(1:nx, 1+1,is)
    derivm(1:nx,ny,is) = derivm(1:nx,ny-1,is)
    derivm( 1,1:ny,is) = derivm( 1+1,1:ny,is)
    derivm(nx,1:ny,is) = derivm(nx-1,1:ny,is)

    !deriva(1:nx, 1,is) = deriva(1:nx, 1+1,is)
    !deriva(1:nx,ny,is) = deriva(1:nx,nx-1,is)
    !deriva( 1,1:ny,is) = deriva( 1+1,1:ny,is)
    !deriva(nx,1:ny,is) = deriva(nx-1,1:ny,is)
        
  end do
  
  deallocate(tsurfs)
  
  
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Determine magnitude and azimuth of derivative !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(0,*) '-----------------------------------'
  write(0,*) '     Characterize Gradients        '

  ! Loop over tomography grid
  do iy=1,ny
  do ix=1,nx
    
    meanval = 0
    cntr    = 0
    ! Mean of slowness
    do is=1,ns
      dist = sqrt( (headeout((iy-1)*nx+ix,1)-rpos(is,1))**2. + (headeout((iy-1)*nx+ix,2)-rpos(is,2))**2. )
      !  write(0,*) dist
      if ( dist.ge.minr .and. dist.le.maxr ) then
        cntr    = cntr    +    1
        meanval = meanval +   dble( derivm(ix,iy,is) )
      end if
    end do
    eitomo(ix,iy,1) = real( meanval / dble(cntr) )
    if (eitomo(ix,iy,1).ne.eitomo(ix,iy,1)) then; eitomo(ix,iy,1)=0.; end if   ! fix for NaN
    
    stdvval = 0
    cntr    = 0
    ! Standard deviation of slowness
    do is=1,ns
      dist = sqrt( (headeout((iy-1)*nx+ix,1)-rpos(is,1))**2. + (headeout((iy-1)*nx+ix,2)-rpos(is,2))**2. )
      if ( dist.ge.minr .and. dist.le.maxr ) then
        cntr    = cntr    + 1
        stdvval = stdvval + ( dble( derivm(ix,iy,is) ) - dble( eitomo(ix,iy,1) ) )**2.
      end if
    end do
    eitomo(ix,iy,2) = real(sqrt( stdvval/ (dble(cntr)*(dble(cntr)-1)) ))
    if (eitomo(ix,iy,2).ne.eitomo(ix,iy,2)) then; eitomo(ix,iy,2)=0.; end if   ! fix for NaN
    
    ! Store the total number of estimates for this location
    eitomo(ix,iy,4) = cntr
    
  end do
  end do

  ! Convert slowness to velocity
  if (.not.slowness) then
    do iy=1,ny
    do ix=1,nx
      if(eitomo(ix,iy,1).ne.0) then
        eitomo(ix,iy,1) = real(1./dble(eitomo(ix,iy,1)) )
        eitomo(ix,iy,2) = real(   dble(eitomo(ix,iy,2))*dble(eitomo(ix,iy,1))**2. )
      end if
    end do
    end do
  end if
  
  ! Actually write out results
  call srite("out",eitomo,4*size(eitomo))

  write(0,*) '-----------------------------------'
  ! Deallocate what's left undeallocated
  deallocate(eitomo)
  deallocate(trpcks)
  deallocate(headeout)
  deallocate(rpos)
  call exit(0)

end program
