program EikonalTomo_omp


  use sep
  use regurize2d_splines_mod_omp
  use masker_mod
  use stats_mod
  implicit none
  
  ! general
  integer          :: nr,ns
  
  integer          :: n1,n2,n3
  real             :: d1,d2,d3
  real             :: o1,o2,o3
  character*50     :: label1,label2,label3
  character*50     :: label4,label5,label6
  
  ! Counters
  integer          :: it,ix,iy,ir,is,cntr,cntr_tot
  
  ! For tomography grid
  integer          :: nx,ny
  real             :: dx,dy,ox,oy
  
  ! Memory variables
  real,allocatable,dimension(:    )  :: ndat,nreg
  real,allocatable,dimension(:,:  )  :: tdat,treg,headeout
  real,allocatable,dimension(:,:,:)  :: hdat,hreg,datain
  
  real,allocatable,dimension(:,:,:)  :: derivm
  real,allocatable,dimension(:,:,:)  :: deriva
  real,allocatable,dimension(:,:,:)  :: eitomo
  real,allocatable,dimension(:,:,:)  :: tsurfs

  real,allocatable,dimension(:    )  :: estimates

  integer                            :: stat_space,estimates_nx,estimates_ny
  real                               :: quant

  ! Auxiliary variables
  real             :: dfact1,dfact2
  real             :: meanval,stdvval,dist,minr,maxr,snr
  logical          :: velocity



  write(0,*) '-----------------------------------'
  write(0,*) '       EIKONAL TOMOGRAPHY          '
  write(0,*) '-----------------------------------'
  ! Header IO
  call sep_init()
  
  call from_history("n1",n1,1 )   ! Has to be 12
  call from_history("n2",nr,1 )   ! Assumed to be nr
  call from_history("n3",ns,1 )   ! Assumed to be ns

  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  call from_history("d3",d3,1.)

  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  call from_history("o3",o3,0.)
  
  call from_history("label1",label1,' ')
  call from_history("label2",label2,' ')
  call from_history("label3",label3,' ')
  
  ! Determine number of receivers
  if (n1.ne.12) then; write(0,*) 'Wrong pick input'; call exit(0); end if  ! Assumed to be nr
  write(0,*) ' determined nr,ns:',nr,ns
  
  ! Get output grid from param
  call from_param("nx",nx)
  call from_param("ny",ny)
  call from_param("dx",dx)
  call from_param("dy",dy)
  call from_param("ox",ox)
  call from_param("oy",oy)
  
  ! Minimum and maximum offsets for statistical characterization
  call from_param("snr" ,snr ,1.    )
  call from_param("minr",minr,0.    )  ! Only when header provided
  call from_param("maxr",maxr,15000.)
  call from_param("velocity",velocity,.true.)

  ! Create output History
  call to_history("n1",nx)
  call to_history("n2",ny)
  call to_history("n3", 7)  
  
  call to_history("d1",dx)
  call to_history("d2",dy)
  call to_history("d3",1.)  
  
  call to_history("o1",ox)
  call to_history("o2",oy)
  call to_history("o3",1.)
  
  call to_history("label1","x position")
  call to_history("label2","y position")
  call to_history("label3","(mean,stddev,#picks,#estimates)")
  
  call to_history("hff",-1)
  call to_history("gff",-1)
  
  if (exist_file("rpicks")) then
    call to_history("n1",nr,"rpicks")
    call to_history("n2",1 ,"rpicks")
    call to_history("n3",1 ,"rpicks")
    call to_history("d1",1.,"rpicks")
    call to_history("d2",1.,"rpicks")
    call to_history("d3",1.,"rpicks")
    call to_history("o1",1.,"rpicks")
    call to_history("o2",0.,"rpicks")
    call to_history("o3",0.,"rpicks")
    call to_history("label1",'receiver #',"rpicks")
    call to_history("label2",' '         ,"rpicks")
    call to_history("label3",' '         ,"rpicks")
  end if
  
  if (exist_file("tsurf")) then
    call to_history("n1",nx,"tsurf")
    call to_history("n2",ny,"tsurf")
    call to_history("n3",ns,"tsurf")  
  
    call to_history("d1",dx,"tsurf")
    call to_history("d2",dy,"tsurf")
    call to_history("d3",1.,"tsurf")  
  
    call to_history("o1",ox,"tsurf")
    call to_history("o2",oy,"tsurf")
    call to_history("o3",1.,"tsurf")

    call to_history("label1","x position","tsurf")
    call to_history("label2","y position","tsurf")
    call to_history("label3","shot #"    ,"tsurf")
  end if
  
  call from_param("stat_space"  ,stat_space  ,1   )  ! integer: 1) slowness, 2) log(slowness), 3) log(velocity), 4) velocity
  call from_param("quant"       ,     quant  ,  0.)  ! outer-quantiles to be excluded from statistical characterization
  if (quant.gt..5) then; write(0,*) 'quant too large'; call exit(0); end if
  call from_param("estimates_nx",estimates_nx,nx/2)
  call from_param("estimates_ny",estimates_ny,ny/2)

  if (exist_file("estimates")) then
    call to_history("n2",1 ,"estimates")
    call to_history("n3",1 ,"estimates")
  
    call to_history("d1",1.,"estimates")
    call to_history("d2",1.,"estimates")
    call to_history("d3",1.,"estimates")
  
    call to_history("o1",0.,"estimates")
    call to_history("o2",0.,"estimates")
    call to_history("o3",0.,"estimates")

    call to_history("label1","estimates")
    if (stat_space.eq.1)  call to_history("label2","slowness","estimates")
    if (stat_space.eq.2)  call to_history("label2","log(slowness)","estimates")
    if (stat_space.eq.3)  call to_history("label2","log(velocity)","estimates")
    if (stat_space.eq.4)  call to_history("label2","velocity","estimates")
    call to_history("label3"," ","estimates")
  end if

  
  ! Construct output grid
  allocate(headeout(nx*ny,2)); headeout=0
  do iy=1,ny
  do ix=1,nx
    headeout((iy-1)*nx+ix,:) = (/ ox+(ix-1)*dx , oy+(iy-1)*dy /)
  end do
  end do
  
  ! Allocate some space
  allocate( datain(n1,nr,ns) )

  allocate(tdat (nr,     ns+1) ); tdat = 0.
  allocate(hdat (nr,2,   ns+1) ); hdat = 0.
  allocate(ndat (        ns+1) ); ndat = 0.

  allocate(treg (nx*ny,  ns+1) ); treg = 0.
  allocate(hreg (nx*ny,2,ns+1) ); hreg = 0. 
  allocate(nreg (        ns+1) ); nreg = nx*ny
  
  ! Read picked data
  call sreed("in",datain,4*size(datain))

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Keep some travel time picks and regularize !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  write(0,*) '-----------------------------------'
  write(0,*) '   Determine which picks to keep   '
  
  ! Select which picks to reguralize and build arrays for reg_mod input
  do is=1,ns
    cntr = 0
    do ir=1,nr
      ! Keep if exceeding a SNR
      if (datain(2,ir,is).gt.snr) then
        cntr              = cntr + 1
        hdat(cntr,:,is  ) = (/ datain(3, ir,is) , datain(4,ir,is) /)
        tdat(cntr,  is  ) =    datain(1, ir,is)
        tdat(ir  ,  ns+1) =      tdat(ir, ns+1) + 1.
      end if
    end do
    ndat(    is) = cntr
    hreg(:,:,is) = headeout
  end do
  do ir=1,nr
    hdat(ir,:,ns+1) = (/ datain(3, ir,1) , datain(4,ir,1) /)
  end do
  ndat(    ns+1) = nr
  hreg(:,:,ns+1) = headeout

  ! Write nr of kept picks to file
  if (exist_file("rpicks")) then
    call srite("rpicks",tdat(:,ns+1),4*size(tdat(:,ns+1)) )
  end if

  write(0,*) '-----------------------------------'
  write(0,*) ' Start Travel Time Regularization  '; write(0,*) ' '

  ! Actual Regularization
  call regurize2d_splines_omp_init (nr,nx*ny)
  call regurize2d_splines_omp(hdat,ndat,hreg,nreg,tdat,treg)
  call regurize2d_splines_omp_close()
  
  write(0,*) 'Unwrap regularized data'
  allocate(tsurfs  (nx,ny,ns) ); tsurfs = 0.
  allocate(eitomo  (nx,ny, 7) ); eitomo = 0.
  
  ! Insert regularized picks into surfaces array
  do is=1,ns
  do iy=1,ny
  do ix=1,nx
    tsurfs(ix,iy,is) = treg((iy-1)*nx+ix,is)
  end do
  end do
  end do
  ! Insert regularized nr of kept picks into surfaces array
  do iy=1,ny
  do ix=1,nx
    eitomo(ix,iy,3) = treg((iy-1)*nx+ix,ns+1)
  end do
  end do
  
  
  ! Close some and deallocate 
  deallocate(tdat);
  deallocate(treg);
  deallocate(hreg);
  deallocate(nreg);
  
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Determine magnitude and azimuth of derivative !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(0,*) '-----------------------------------'
  write(0,*) '         Compute Gradients         '
  ! Alocate stuff
  allocate(derivm(nx,ny,ns)); derivm=0.
  allocate(deriva(nx,ny,ns)); deriva=0.
  
  ! Determine derivative factors
  dfact1 = 1./(dx*2)
  dfact2 = 1./(dx*2)**2.


  do is=1,ns
    
    do iy=2,ny-1
    do ix=2,nx-1
      
      ! Magnitude of gradient
      derivm(ix,iy,is) = sqrt ( dfact2*(tsurfs(ix+1,iy,is)-tsurfs(ix-1,iy,is))**2. + dfact2*(tsurfs(ix,iy+1,is)-tsurfs(ix,iy-1,is))**2. )
      
    !  ! Azimuth of gradient
    !  deriva(ix,iy,is) = atan2( dfact1*(tsurfs(ix+1,iy,is)-tsurfs(ix-1,iy,is))     , dfact1*(tsurfs(ix,iy+1,is)-tsurfs(ix,iy-1,is))     )
      
    end do
    end do
    
    ! Fill the edges by extension
    derivm(1:nx, 1,is) = derivm(1:nx, 1+1,is)
    derivm(1:nx,ny,is) = derivm(1:nx,ny-1,is)
    derivm( 1,1:ny,is) = derivm( 1+1,1:ny,is)
    derivm(nx,1:ny,is) = derivm(nx-1,1:ny,is)

    !deriva(1:nx, 1,is) = deriva(1:nx, 1+1,is)
    !deriva(1:nx,ny,is) = deriva(1:nx,nx-1,is)
    !deriva( 1,1:ny,is) = deriva( 1+1,1:ny,is)
    !deriva(nx,1:ny,is) = deriva(nx-1,1:ny,is)
        
  end do
  
  ! Optionally write out regularized travel time surfaces after masking
  if (exist_file("tsurf")) then
    write(0,*) '-----------------------------------'
    write(0,*) '    Mask Travel time Surfaces      '
  
    call mask_init(nx,ny,dx,dy,ox,oy,125.,1500.,-1.)
    do is=1,ns
      call do_mask(tsurfs(:,:,is),hdat(1:ndat(is),:,is))
    end do

    call srite("tsurf",tsurfs,4*size(tsurfs))
  end if
  deallocate(tsurfs)
  

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Determine magnitude and azimuth of derivative !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(0,*) '-----------------------------------'
  write(0,*) '  Mask Gradients for Array Extend  '
  
  call mask_init(nx,ny,dx,dy,ox,oy,125.,1500.,-1.)
  do is=1,ns
    call do_mask(derivm(:,:,is),hdat(1:ndat(is),:,is))
  end do
  deallocate(hdat);
  deallocate(ndat);
  
  
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Determine magnitude and azimuth of derivative !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(0,*) '-----------------------------------'
  write(0,*) '     Characterize Gradients        '
  if (stat_space.eq.1)  write(0,*) "in slowness space"
  if (stat_space.eq.2)  write(0,*) "in log(slowness) space"
  if (stat_space.eq.3)  write(0,*) "in log(velocity) space"
  if (stat_space.eq.4)  write(0,*) "in velocity space"
  allocate(estimates(ns))

  cntr_tot=0
  ! Loop over tomography grid
  do iy=1,ny
  do ix=1,nx
 
 !   if (ix.eq.estimates_nx   .and. iy.eq.estimates_ny) then
 !     write(0,*) "derivm"
 !     write(0,*)  derivm(ix,iy,:)
  !  end if

    cntr    = 0
    ! Mean of slowness
    do is=1,ns
      if (derivm(ix,iy,is).gt.0) then
      dist = sqrt( (headeout((iy-1)*nx+ix,1)-datain(5,1,is))**2. + (headeout((iy-1)*nx+ix,2)-datain(6,1,is))**2. )
        if ( dist.ge.minr .and. dist.le.maxr ) then
          
          cntr    = cntr    +    1
          if (stat_space.eq.1)  estimates(cntr) =          derivm(ix,iy,is)
          if (stat_space.eq.2)  estimates(cntr) = log10(   derivm(ix,iy,is))
          if (stat_space.eq.3) then;
            estimates(cntr) = log10(1./derivm(ix,iy,is))
 !   if (ix.eq.estimates_nx   .and. iy.eq.estimates_ny) then
 !           write(0,*) "Converted"
  !  end if
          end if
          if (stat_space.eq.4)  estimates(cntr) =       1./derivm(ix,iy,is)
          
        end if
      end if
    end do

  !  if (ix.eq.estimates_nx   .and. iy.eq.estimates_ny) then
  !    write(0,*) "estimates"
   !   write(0,*)  estimates(1:cntr)
   ! end if

    if (quant.gt.0.) then
      ! Sort and exclude quantiles    
      call sort(estimates(1:cntr))
      estimates(1:(cntr-quant*cntr)-(1+quant*cntr)+1) = estimates((1+quant*cntr):(cntr-quant*cntr));
      cntr=       (cntr-quant*cntr)-(1+quant*cntr)+1
    end if

    if (cntr.ge.1) then
     ! Do Statistics
      eitomo(ix,iy,1) = calcmean    (estimates(1:cntr)    )
      eitomo(ix,iy,2) = calcstdv    (estimates(1:cntr)    )
      eitomo(ix,iy,4) = cntr
      eitomo(ix,iy,5) = calcmedian  (estimates(1:cntr)    )
      eitomo(ix,iy,6) = calcquantile(estimates(1:cntr), 5.)
      eitomo(ix,iy,7) = calcquantile(estimates(1:cntr),95.)
    else
      eitomo(ix,iy,1) = 0.
      eitomo(ix,iy,2) = 0.
      eitomo(ix,iy,4) = cntr
      eitomo(ix,iy,5) = 0.
      eitomo(ix,iy,6) = 0.
      eitomo(ix,iy,7) = 0.
    end if

  !  if (ix.eq.estimates_nx   .and. iy.eq.estimates_ny) then
  !    write(0,*) "eitomo"
  !    write(0,*)  eitomo(ix,iy,:)
  !  end if

    if (ix.eq.estimates_nx-1 .and. iy.eq.estimates_ny-1 .and. exist_file("estimates")) then
      cntr_tot = cntr_tot + cntr
      write(0,*) "Estimates written to file for (nx,ny)",estimates_nx-1,estimates_ny-1
      call srite("estimates",estimates(1:cntr),4*cntr)
    end if
    if (ix.eq.estimates_nx   .and. iy.eq.estimates_ny-1 .and. exist_file("estimates")) then
      cntr_tot = cntr_tot + cntr
      write(0,*) "Estimates written to file for (nx,ny)",estimates_nx,estimates_ny-1
      call srite("estimates",estimates(1:cntr),4*cntr)
    end if
    if (ix.eq.estimates_nx+1 .and. iy.eq.estimates_ny-1 .and. exist_file("estimates")) then
      cntr_tot = cntr_tot + cntr
      write(0,*) "Estimates written to file for (nx,ny)",estimates_nx+1,estimates_ny-1
      call srite("estimates",estimates(1:cntr),4*cntr)
    end if

    if (ix.eq.estimates_nx-1 .and. iy.eq.estimates_ny   .and. exist_file("estimates")) then
      cntr_tot = cntr_tot + cntr
      write(0,*) "Estimates written to file for (nx,ny)",estimates_nx-1,estimates_ny
      call srite("estimates",estimates(1:cntr),4*cntr)
    end if
    if (ix.eq.estimates_nx   .and. iy.eq.estimates_ny   .and. exist_file("estimates")) then
      cntr_tot = cntr_tot + cntr
      write(0,*) "Estimates written to file for (nx,ny)",estimates_nx,estimates_ny
      call srite("estimates",estimates(1:cntr),4*cntr)
    end if
    if (ix.eq.estimates_nx+1 .and. iy.eq.estimates_ny   .and. exist_file("estimates")) then
      cntr_tot = cntr_tot + cntr
      write(0,*) "Estimates written to file for (nx,ny)",estimates_nx+1,estimates_ny
      call srite("estimates",estimates(1:cntr),4*cntr)
    end if

    if (ix.eq.estimates_nx-1 .and. iy.eq.estimates_ny+1 .and. exist_file("estimates")) then
      cntr_tot = cntr_tot + cntr
      write(0,*) "Estimates written to file for (nx,ny)",estimates_nx-1,estimates_ny+1
      call srite("estimates",estimates(1:cntr),4*cntr)
    end if
    if (ix.eq.estimates_nx   .and. iy.eq.estimates_ny+1 .and. exist_file("estimates")) then
      cntr_tot = cntr_tot + cntr
      write(0,*) "Estimates written to file for (nx,ny)",estimates_nx,  estimates_ny+1
      call srite("estimates",estimates(1:cntr),4*cntr)
    end if
    if (ix.eq.estimates_nx+1 .and. iy.eq.estimates_ny+1 .and. exist_file("estimates")) then
      cntr_tot = cntr_tot + cntr
      write(0,*) "Estimates written to file for (nx,ny)",estimates_nx+1,estimates_ny+1
      call srite("estimates",estimates(1:cntr),4*cntr)
    end if

  end do
  end do
  deallocate(estimates)
  
  ! Convert slowness to velocity
  if (velocity) then
 !   write(0,*) "Yessss"
  do iy=1,ny
  do ix=1,nx
      if (stat_space.eq.1) then
        if(eitomo(ix,iy,1).ne.0) then
          eitomo(ix,iy,1) = real(1./dble(eitomo(ix,iy,1)) )
          eitomo(ix,iy,2) = real(   dble(eitomo(ix,iy,2))*dble(eitomo(ix,iy,1))**2. )
        end if
      end if
      
      if (stat_space.eq.2) then
        if(eitomo(ix,iy,1).ne.0) then
          eitomo(ix,iy,2) = (10.**dble(eitomo(ix,iy,1)+eitomo(ix,iy,2))-10.**dble(eitomo(ix,iy,1)-eitomo(ix,iy,2)))/2.
          eitomo(ix,iy,1) =  10.**dble(eitomo(ix,iy,1))
          eitomo(ix,iy,1) = real(1./dble(eitomo(ix,iy,1)) )
          eitomo(ix,iy,2) = real(   dble(eitomo(ix,iy,2))*dble(eitomo(ix,iy,1))**2. )
        end if
      end if
      
      if (stat_space.eq.3) then
        if(eitomo(ix,iy,1).ne.0) then
          eitomo(ix,iy,2) = (10.**dble(eitomo(ix,iy,1)+eitomo(ix,iy,2))-10.**dble(eitomo(ix,iy,1)-eitomo(ix,iy,2)))/2.
          eitomo(ix,iy,1) =  10.**dble(eitomo(ix,iy,1))
        end if
      end if

  !  if (ix.eq.estimates_nx   .and. iy.eq.estimates_ny) then
  !    write(0,*) "eitomo"
  !    write(0,*)  eitomo(ix,iy,:)
  !  end if

    end do
    end do
  end if
  
  ! Actually write out results
  call srite("out",eitomo,4*size(eitomo))
  
  write(0,*) '-----------------------------------'
  ! Deallocate what's left undeallocated
  if (exist_file("estimates"))  call to_history("n1",cntr_tot,"estimates")
  call sep_close()
  deallocate(eitomo)
  deallocate(headeout)
  deallocate(derivm)
  deallocate(deriva)
  deallocate(datain)
  call exit(0)
  
end program
