program SNRcalc
  ! Progrmam to pick Maximum in traces for first arrival picking.
  ! First combines positive and negative time windows (if S/N>sn).

  use sep
  implicit none
  
  integer          :: n1,n2,n3,n4,n5,n6,n1h
  real             :: d1,d2,d3,d4,d5,d6
  real             :: o1,o2,o3,o4,o5,o6
  integer          :: ii,jj,kk,ll,mm,oo
  character*50     :: label1,label2,label3
  character*50     :: label4,label5,label6
  
  real,allocatable :: data(:),data_pos(:),data_neg(:)
  
  real,allocatable :: t(:)
  
  real             :: value(12)
  
  logical          :: indices

  
  integer          :: maxp

  real,allocatable  :: hdrs(:) !(course receiver coordinates)
  real             :: v ! Controll values for velocity zone
  real             :: tau0,dtau,rad,ang,snr,unit
  integer          :: wine,winb,en,bp,nn,np,ne
  integer          :: t0,dt,cnt
  
  real             :: sn_pos, sn_neg
  real             :: sig_in,sig_out

  integer          :: sx_key,sy_key,rx_key,ry_key,dd_key
  

  write(0,*) '-----------------------------------'
  write(0,*) '       Calculate SNR Ratio         '
  write(0,*) '-----------------------------------'
  call sep_init()
  
  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )
  call from_history("n4",n4,1 )
  call from_history("n5",n5,1 )
  call from_history("n6",n6,1 )

  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  call from_history("d3",d3,1.)
  call from_history("d4",d4,1.)
  call from_history("d5",d5,1.)
  call from_history("d6",d6,1.)

  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  call from_history("o3",o3,0.)
  call from_history("o4",o4,0.)
  call from_history("o5",o5,0.)
  call from_history("o6",o6,0.)
  
  call from_history("label1",label1,' ')
  call from_history("label2",label2,' ')
  call from_history("label3",label3,' ')
  call from_history("label4",label4,' ')
  call from_history("label5",label5,' ')
  call from_history("label6",label6,' ')

  call from_param('indices',indices,.false.)   ! Get indixes instead of axis values

  call to_history("n1",12);
  call to_history("n2",n2)
  call to_history("n3",n3)
  call to_history("n5",n4)
  call to_history("n6",n5)
  call to_history("n7",n6)
  call to_history("hff",-1)


  call from_param('v',v,1000.)   ! Get indixes instead of axis values
  call from_param('unit',unit,1.)   ! Get indixes instead of axis values
  call from_param('dtau',dtau, 1.)   ! Get indixes instead of axis values
  dt = nint(dtau/d1)
  
  
  if (exist_file('header')) then
    call from_aux("header","n1",n1h,1)
    call from_param( "sx_key" ,sx_key,1)
    call from_param( "sy_key" ,sy_key,2)
    call from_param( "rx_key" ,rx_key,4)
    call from_param( "ry_key" ,ry_key,5)
    call from_param("dead_key",dd_key,7)
  end if
  
  write(0,*) ' determined number of traces:',n2*n3*n4*n5*n6
 
  call sep_close()
  
  ! Determine some indices in posiive and negative time windows
  ! Construct t-axis
  allocate(t(n1))
  do ii=1,n1;  t(ii)=o1+(ii-1)*d1;  end do
  do ii=1,n1;  if (t(ii).lt.0.) then; en = ii; end if; end do; bp = en+1
  ! Number of positive and negative elements
  np = n1-en
  nn =  1+en
  ! Number of elements
  ne = maxval((/np,nn/))
  
  
  allocate( data    (n1) )
  allocate( data_neg(ne) )
  allocate( data_pos(ne) )
  allocate( hdrs    (n1h) )
  
  do oo=1,n6
  do mm=1,n5
  do ll=1,n4
  do kk=1,n3
  do jj=1,n2
  
  data = 0.
  data_neg = 0.
  data_pos = 0.

  ! Read headers and data for this trace
  call sreed("header",hdrs,4*n1h)
  call sreed("in"    ,data,4*n1 )
  
  ! Sort data into positve and negative windows
  do ii= 1,en+1;  data_neg(ii   ) = data(bp-ii+1);  end do
  do ii=bp,n1  ;  data_pos(ii-en) = data(   ii  );  end do
  data_pos(ne)=data_pos(ne-1) ! Just suplement one sample
  
  ! Guessed traveltime
  rad = sqrt( (hdrs(rx_key)-hdrs(sx_key))**2. + (hdrs(ry_key)-hdrs(sy_key))**2. )*unit
  ang = atan2( (hdrs(ry_key)-hdrs(sy_key)),(hdrs(rx_key)-hdrs(sx_key)) )
  if (ang.gt. 4.) then;  write(0,*) 'detected'; ang=0;  end if
  if (ang.lt.-4.) then;  write(0,*) 'detected'; ang=0;  end if
  t0 = nint( rad/real(v*d1) + 1. )  ! Index of t0
  
  ! Determine indices of window of interest
  winb = minval((/maxval((/t0-dt, 1/)) , ne/))
  wine = maxval((/minval((/t0+dt,ne/)) ,  1/))
  
  ! Estimate S/N ratio in negative time window
  sig_in  = sum(abs( data_neg(winb:wine) )) / (wine-winb+1) !maxval(datneg(winb:wine))
  sig_out = 0.
  if (winb.gt. 1) then
    sig_out = sig_out + sum(abs( data_neg(   1:winb) )) / (winb-   1+1)
  end if
  if (wine.lt.ne) then
    sig_out = sig_out + sum(abs( data_neg(wine:  ne) )) / (ne  -wine+1)
  end if
  sn_neg = sig_in/sig_out
  
  ! Estimate S/N ratio in positive time window
  sig_in  = sum(abs( data_pos(winb:wine) )) / (wine-winb+1) ! maxval(datpos(winb:wine))
  sig_out = 0.
  if (winb.gt. 1) then
    sig_out = sig_out + sum(abs( data_pos(   1:winb) )) / (winb-   1+1)
  end if
  if (wine.lt.ne) then
    sig_out = sig_out + sum(abs( data_pos(wine:  ne) )) / (ne  -wine+1)
  end if
  sn_pos = sig_in/sig_out
  
  ! Use maxium snr fro + or - window
  if (sn_pos.ge.sn_neg) then
    ! Use positive window
    data(1:ne) = data_pos
    ! Pick maximum
    maxp  = maxval(winb-1+maxloc(data(winb:wine)))
    snr   = sn_pos
  else
    ! Use positive window
    data(1:ne) = data_neg
    ! Pick maximum
    maxp  = maxval(winb-1+maxloc(data(winb:wine)))
    snr   = sn_neg
  end if
  
  
  if (indices) then;  value(1) =  real(maxp)
  else;               value(1) = (real(maxp)-1.)*d1;  end if
  value(2)  = snr
  value(3)  = hdrs(rx_key)*unit
  value(4)  = hdrs(ry_key)*unit
  value(5)  = hdrs(sx_key)*unit
  value(6)  = hdrs(sy_key)*unit
  value(7)  = rad
  value(8)  = ang
  value(9)  = jj
  value(10) = kk
  value(11) = (real(winb)-1.)*d1    ! Lower limit of picking window
  value(12) = (real(wine)-1.)*d1    ! Upper limit of picking window
  
  ! Write the SNR
  call srite("out",value,4*12)
  
  
  end do
  end do
  end do
  end do
  end do
  
  write(0,*) '-----------------------------------'
  deallocate( data , data_neg , data_pos , hdrs , t)
  call exit(0)

end program
