program sdrFFTmd
! Tape a datafile

  use sep                      ! Main SEP module
  use fftw_mod
  implicit none
  
  real, allocatable               :: data_in (:,:,:)
  complex,allocatable             :: data_out(:,:,:)
  integer                         :: n(3),nw(3)
  real                            :: d(3),o(3),dw(3)
  integer                         :: dir
  logical                         :: adj
  character*50                    :: labels(3)
  integer                         :: unit(3),ii
  real, parameter                 :: pi=3.14159265

  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
    
  ! Read input parameters for taper, there is lots of options :-)
  call from_param ("dir" ,dir,   1    )
  call from_param ("adj" ,adj,.false. )
  call from_param ("unit" ,unit,(/1,1,1/) )


  if (adj) then;
    call from_history("n1", nw(1) )
    call from_history("n2", nw(2) )
    call from_history("n3", nw(3) )
    call from_history("d1", dw(1) )
    call from_history("d2", dw(2) )
    call from_history("d3", dw(3) )
    call from_history("o1",  o(1) )
    call from_history("o2",  o(2) )
    call from_history("o3",  o(3) )
    call from_history("label1", labels(1) )
    call from_history("label2", labels(2) )
    call from_history("label3", labels(3) )
    do ii=1,3
      if (unit(ii).eq.2) then; dw(ii) = dw(ii)/2./pi; end if
    end do
    call fft_r2c_units_t(adj,n(1),d(1),nw=nw(1),df=dw(1))
    call fft_c2c_units_x(adj,n(2),d(2),nk=nw(2),dk=dw(2))
    if (nw(3).gt.1) then
      call fft_c2c_units_x(adj,n(3),d(3),nk=nw(3),dk=dw(3))
    else
      n(3) = nw(3)
    end if
    call to_history("n1", n(1) )
    call to_history("n2", n(2) )
    call to_history("n3", n (3) )
    call to_history("d1", d(1) )
    call to_history("d2", d(2) )
    call to_history("d3", d(3) )
    call to_history("o1", o(1) )
    call to_history("o2", o(2) )
    call to_history("o3", o(3) )
    call to_history("esize", 4 )
    call to_history("label1", labels(1) )
    call to_history("label2", labels(2) )
    call to_history("label3", labels(3) )
  else;
    call from_history("n1", n(1) )
    call from_history("n2", n(2) )
    call from_history("n3", n(3) )
    call from_history("d1", d(1) )
    call from_history("d2", d(2) )
    call from_history("d3", d(3) )
    call from_history("o1", o(1) )
    call from_history("o2", o(2) )
    call from_history("o3", o(3) )
    call from_history("label1", labels(1) )
    call from_history("label2", labels(2) )
    call from_history("label3", labels(3) )
    call fft_r2c_units_t(adj,n(1),d(1),nw=nw(1),df=dw(1))
    call fft_c2c_units_x(adj,n(2),d(2),nk=nw(2),dk=dw(2))
    if (n(3).gt.1) then
      call fft_c2c_units_x(adj,n(3),d(3),nk=nw(3),dk=dw(3))
    else
      nw(3) = n(3)
    end if
    !write(0,*) "dw",dw
    do ii=1,3
      if (unit(ii).eq.2) then; dw(ii) = dw(ii)*2.*pi; end if
    end do
    call to_history("n1", nw(1) )
    call to_history("n2", nw(2) )
    call to_history("n3", nw(3) )
    call to_history("o1",    0. )
    call to_history("o2", -nw(2)/2*dw(2) )
    call to_history("o3", -nw(2)/2*dw(3) )
    call to_history("d1", dw(1) )
    call to_history("d2", dw(2) )
    call to_history("d3", dw(3) )
    call to_history("esize", 8  )
    call to_history("label1", labels(1) )
    call to_history("label2", labels(2) )
    call to_history("label3", labels(3) )
  end if
  
  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
  
  ! Allocate the arrays datain & dataout, and Initalize the Module
  allocate ( data_in(n(1),n(2),n(3)),data_out(nw(1) ,nw(2),nw(3)) )
  ! Initialize the FFTW functions
  call fftw3d_r2c_lop_init(n1_in=n(1),n2_in=n(2),n3_in=n(3))
  
  ! Read in data from input file
  if (adj) then;
    call sep_read  (data_out)
  else;
    call sep_read  (data_in )
  end if
  
  ! Execute MD FFT
  if (n(3).gt.1) then
    call fftw3d_r2c_lop_3d(adj,.false.,data_in       ,data_out       )
  else
    call fftw3d_r2c_lop_2d(adj,.false.,data_in(:,:,1),data_out(:,:,1))
  end if
  
  ! Write data to output file
  if (adj) then;
    call sep_write (data_in )
  else;
    call sep_write (data_out)
  end if

  ! Deallocate memory
  call fftw3d_r2c_lop_close()
  deallocate     (data_in,data_out)
  
  ! Force clean exit
  call exit (0)
  
end program

