program sdrBandpass2dAng
! Tape a datafile
  ! taper types: 
  ! taper types: 
  ! 1) 'zero'            --  needed: 'nothing' 
  ! 2) 'rect'            --  needed: t1,t2 
  ! 3) 'linear_up'       --  needed: t1,t2
  ! 4) 'linear_down'     --  needed: t1,t2
  ! 5) 'pyramid'         --  needed: t1,t2
  ! 6) 'pyramid_band'    --  needed: t1,t2,t3,t4
  ! 7) 'hann'            --  needed: t1,t2
  ! 8) 'hann_half'       --  needed: t1,t2
  ! 9) 'hann_band'       --  needed: t2,t2,t3,t4
  ! 10) 'cosine'         --  needed: t1,t2
  ! 11) 'gaussian'       --  needed: t1,t2,sigma,spread
  ! 12) 'gaussian_half'  --  needed: t1,t2,sigma,spread
  ! 13) 'gaussian_block' --  needed: t1,t2,t3,t4,sigma,spread
  !
  ! All tapers have the form of a band or lowcut,
  ! Notch tapers are constructed with inv=1 option.
  ! Sum result onto old data using    add=1 option.
  
  ! Multiple calls take input and overwrite if add=1
  
  use sep                      ! Main SEP module
  use fftw_mod
  use fftshift_mod
  use tapers_mod
  use tape_mod
  use mdtapers_mod
  use mdtape_mod
  use padd_mod
  implicit none

  ! Axes and counters  
  integer                         :: n(2),nt(2),nw(2),n3,n4,n5,b(2),e(2)
  real                            :: d(2), o(2),dw(2)
  integer                         :: ii,jj,kk,ll,mm

  ! Memory variables
  real, allocatable               :: data_in(:,:)
  real, allocatable               :: data_pd(:,:)
  real, allocatable               :: data_sf(:,:)
  complex, allocatable            :: data_ft(:,:)

  ! for FFTdetails
  integer                         :: nfft(2)
  integer                         :: unit(2)
  logical                         :: shift,symt,true_t,true_x

  ! For Taper
  integer                         :: type(2),inv(2)
  integer                         :: f1_ind(2),f2_ind(2),f3_ind(2),f4_ind(2)
  real                            :: f1    (2),f2    (2),f3    (2),f4    (2)
  logical                         :: inv_2(2)
  real                            :: sigma(2)
  logical                         :: taper
  
  ! For MDTaper
  real                            :: p1_ang(1),p2_ang(1),p3_ang(1),p4_ang(1)
  real                            :: p1(1),p2(1),p3(1),p4(1),c(2)
  integer                         :: c_ind(2),p_type(1)
  logical                         :: mdtaper
  logical                         :: inv_ang
  real                            :: p_sigma(1)

  ! Misc
  real, parameter                 :: pi=3.14159265
  logical                         :: padd,verb


  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  call from_history("n1", n(1) )
  call from_history("n2", n(2) )
  call from_history("n3", n3   )
  call from_history("n4", n4   )
  call from_history("n5", n5   )
  call from_history("d1", d(1) )
  call from_history("d2", d(2) )
  call from_history("o1", o(1) )
  call from_history("o2", o(2) )

  call from_param  ("verbose",verb,.false.)
 
  ! Read input parameters for taper, there is lots of options :-)
  call from_param ("type" ,type , (/ 0 , 0 /) )
  call from_param ("inv"  ,inv  , (/ 0 , 0 /) )
  call from_param ("sigma",sigma, (/.1 ,.1 /) )
  
  ! FFTW dettails
  call from_param("shift"  , shift ,.true. )
  call from_param("nfft"   , nfft  , (/n(1),n(2)/) )     ! Determine over what length to calculate the fft (can best be powers of two), does symmetric zero padding
  call from_param("true_t" , true_t,   .false.     )     ! Puts data at true indices in data-array, otherwise, it will assume all data has irrelevant absolute t,x,y coordinates
  call from_param("true_x" , true_x,   .false.     )
  call from_param("unit"   , unit  , (/  1 ,  1 /) )     ! Kind of units to put on the axis: 1:frequency, 2:radial frequency    (also for spatial)
  call from_param("symt"   , symt  , .false.   )         ! To be implemented (to get t-symmetric output [-nt:nt])

  ! Angular tapers
  call from_param ("p_type" , p_type , (/0/) )
  call from_param ("inv_ang", inv_ang, .false.)
  call from_param ("p_sigma", p_sigma, (/.1/) )


  ! Determine b(ginning) and e(nd) indices for data_in
  ! Check if nfft is long enough for true_t and true_x
  if (true_t) then
    call taper_indice(o(1)              ,b(1),d_in=d(1),o_in=0.)
    call taper_indice(o(1)+(n(1)-1)*d(1),e(1),d_in=d(1),o_in=0.)

    nt  (1) =  2*max( abs(e(1)),abs(b(1)),abs(b(1)-2),abs(e(1)-2) )
    nfft(1) = max(nt(1),nfft(1))   ! make sure nfft is large enough

    ! Determine final indices for data_in
    call taper_indice(o(1)              ,b(1),d_in=d(1),o_in=-nt(1)/2*d(1))
    call taper_indice(o(1)+(n(1)-1)*d(1),e(1),d_in=d(1),o_in=-nt(1)/2*d(1))
    
  else
     b(1) =   1
     e(1) = n(1)
    nt(1) = n(1)
    nfft(1) = max(nt(1),nfft(1))   ! make sure nfft is large enough
  end if
  
  if (true_x) then
    call taper_indice(o(2)              ,b(2),d_in=d(2),o_in=0.)
    call taper_indice(o(2)+(n(2)-1)*d(2),e(2),d_in=d(2),o_in=0.)
    
    nt  (2) =  2*max( abs(e(2)),abs(b(2)),abs(b(2)-2),abs(e(2)-2) )
    nfft(2) = max(nt(2),nfft(2))   ! make sure nfft is large enough
    
    ! Determine final indices for data_in
    call taper_indice(o(2)              ,b(2),d_in=d(2),o_in=-nt(2)/2*d(2))
    call taper_indice(o(2)+(n(2)-1)*d(2),e(2),d_in=d(2),o_in=-nt(2)/2*d(2))

  else
     b(2) =   1
     e(2) = n(2)
    nt(2) = n(2)
    nfft(2) = max(nt(2),nfft(2))   ! make sure nfft is large enough
  end if

  ! Padd logic
  if (nfft(1).gt.nt(1) .or. nfft(2).gt.nt(2)) then; padd=.true.; else; padd=.false.; end if;

  ! Determine FFT units
  call fft_r2c_units_t(.false.,nfft(1),d(1),nw=nw(1),df=dw(1))
  call fft_c2c_units_x(.false.,nfft(2),d(2),nk=nw(2),dk=dw(2))

  do ii=1,2
    if (unit(ii).eq.2) then; dw(ii) = 2.*pi*dw(ii); end if
  end do
  
  call from_param ("f1",f1, (/0.,0./)  )
  call from_param ("f2",f2, (/ dw(1)*real(nw(1)-2),dw(2)*real(nw(2)/2-2) /) )
  call from_param ("f3",f3, (/ dw(1)*real(nw(1)-2),dw(2)*real(nw(2)/2-2) /) )
  call from_param ("f4",f4, (/ dw(1)*real(nw(1)-1),dw(2)*real(nw(2)/2-1) /) )
  
  !write(0,*) "f1"
  !write(0,*)  f1
  !write(0,*) "f2"
  !write(0,*)  f2
  !write(0,*) "f3"
  !write(0,*)  f3
  !write(0,*) "f4"
  !write(0,*)  f4


  call from_param ("p1",p1_ang, (/ 0. /) )
  call from_param ("p2",p2_ang, (/tan(.49999998*pi)/) )
  call from_param ("p3",p3_ang, (/tan(.49999998*pi)/) )
  call from_param ("p4",p4_ang, (/tan(.49999998*pi)/) )
  call from_param ("c1",c(1), 0. )
  call from_param ("c2",c(2), 0. )

  ! Convert angle in velocity to radial angle
  p1_ang = atan(p1_ang(1)); p2_ang = atan(p2_ang(1))
  p3_ang = atan(p3_ang(1)); p4_ang = atan(p4_ang(1))
  
 ! write(0,*) "p1_ang,p2_ang,p3_ang,p4_ang: ",p1_ang,p2_ang,p3_ang,p4_ang

  ! Make sure that no taper specification gives a 1 in that direction
  do ii=1,2;
    if (type(ii).eq.0) then; inv  (ii)=1; type(ii)=1; end if
  end do

  do ii=1,2
    if ( inv(ii).eq.0) then; inv_2(ii)=.false.; else; inv_2(ii)=.true.; end if
  end do

  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
  
  ! Determine tie indices for f-domain tapers
  call taper_indice(f1(1),f1_ind(1),d_in=dw(1),o_in=0.)
  call taper_indice(f2(1),f2_ind(1),d_in=dw(1),o_in=0.)
  call taper_indice(f3(1),f3_ind(1),d_in=dw(1),o_in=0.)
  call taper_indice(f4(1),f4_ind(1),d_in=dw(1),o_in=0.)
  call taper_indice(f1(2),f1_ind(2),d_in=dw(2),o_in=-nw(2)/2*dw(2))
  call taper_indice(f2(2),f2_ind(2),d_in=dw(2),o_in=-nw(2)/2*dw(2))
  call taper_indice(f3(2),f3_ind(2),d_in=dw(2),o_in=-nw(2)/2*dw(2))
  call taper_indice(f4(2),f4_ind(2),d_in=dw(2),o_in=-nw(2)/2*dw(2))

  call mdtaper_indice( c(1), c_ind(1),d_in=dw(1),o_in=0.)
  call mdtaper_indice( c(2), c_ind(2),d_in=dw(2),o_in=-nw(2)/2*dw(2))



  ! Initialize tape functions
  call tape_init       (nw(1),nw(2),1)
  
  ! Initialize taper functions in tapers_mod
  call tapers_init     (nw(1),nw(2),1)
  call tapers_interface(2,dir_in=(/ 1 , 2/),t1_in=f1_ind,t2_in=f2_ind,t3_in=f3_ind,t4_in=f4_ind,type_in=type,sigma_in=sigma,scale_in=(/1.,1./),inv_in=inv_2)
  call tapers_insert_tape()
  call tapers_close      ()

  call tape_shift( (/.false.,shift,.false./) )

  if (type(1).gt.2.or.type(2).gt.2) then;  taper=.true.;  else;  taper=.false.;  end if
  

  ! Initialize taper functions in tapers_mod
  call mdtape_init       (nw(1),nw(2),1)
  call mdtapers_init     (n_in=(/nw(1),nw(2),1/),c_in=(/c_ind(1),c_ind(2),1/),d_in=(/dw(1),dw(2),1/),o_in=(/ 0., -nw(2)/2*dw(2),0./) )
  call mdtapers_interface(1,p1_in=p1_ang,p2_in=p2_ang,p3_in=p3_ang,p4_in=p4_ang, &
                          t1_in=(/1,1,1/),t2_in=(/nw(1),nw(2),1/),type_in=p_type,sigma_in=p_sigma,scale_in=(/1./),inv_in=(/inv_ang/))
  call mdtapers_insert_mdtape()
  !call mdtapers_close()
  call mdtape_shift( (/.false.,shift,.false./) )

  if (p_type(1).gt.2) then;  mdtaper=.true.;  end if

  ! Initialize FFTW
  call fftw2d_12_r2c_lop_init(n1_in=nfft(1),n2_in=nfft(2))

  ! Allocate the arrays datain and dataout
  allocate( data_in(nt(1), nt(2)), data_sf(nfft(1),nfft(2)) , data_ft(nw(1),nw(2)) )
  if (padd) then; allocate( data_pd(nfft(1),nfft(2)) ); end if
  

  ! Loup over 3rd, 4th and 5th axis
  do mm=1,n5; do ll=1,n4; do kk=1,n3
    
    ! Verbose which Cube
    write(0,*) "Plane",(mm-1)*n5*n4+(ll-1)*n4+kk,"of",n4*n5*n3
    
    ! Read Data
    data_in = 0.
    call sreed( "in" , data_in(b(1):e(1),b(2):e(2)), 4*n(1)*n(2) )

    ! Compute (Padd) (Shift) and FFT
    if (padd) then
      if (verb) then; write(0,*) "Padded"; end if
      call padd_lop        (.false.,.false.,data_in,data_pd)
      
      if (shift) then
        if (verb) then; write(0,*) "Shifted"; end if
        ! Compute (Shift) and FFT
        call fftshift2d_lop(.false.,.false.,data_pd,data_sf)
        call fftw2d_r2c_lop(.false.,.false.,data_sf,data_ft)
      else
        if (verb) then; write(0,*) "Not Shifted"; end if
        ! Compute Shift, FFT and 
        call fftw2d_r2c_lop(.false.,.false.,data_pd,data_ft)
      end if
      
    else;
      
      if (shift) then
        if (verb) then; write(0,*) "Shifted1"; end if
        call fftshift2d_lop(.false.,.false.,data_in,data_sf)
        call fftw2d_r2c_lop(.false.,.false.,data_sf,data_ft)
      else
        if (verb) then; write(0,*) "Not Shifted"; end if
        call fftw2d_r2c_lop(.false.,.false.,data_in,data_ft)
      end if
      
    end if

    
    ! Taper
    if (  taper) then;  call   tape_op(data_ft);  end if
    if (mdtaper) then;  call mdtape_op(data_ft);  end if


    ! Compute (Padd) (Shift) and FFT
    if (padd) then
      
      if (shift) then
        if (verb) then; write(0,*) "Inverse Shifted"; end if
        call fftw2d_r2c_lop(.true. ,.false.,data_sf,data_ft)
        call fftshift2d_lop(.true. ,.false.,data_pd,data_sf)
      else
        if (verb) then; write(0,*) "Not Inverse Shifted"; end if
        call fftw2d_r2c_lop(.true. ,.false.,data_pd,data_ft)
      end if
      if (verb) then; write(0,*) "Padded"; end if
      call padd_lop        (.true. ,.false.,data_in,data_pd)
      
    else;
      
      if (shift) then
        if (verb) then; write(0,*) "Inverse Shifted"; end if
        call fftw2d_r2c_lop(.true. ,.false.,data_sf,data_ft)
        call fftshift2d_lop(.true. ,.false.,data_in,data_sf)
      else
        if (verb) then; write(0,*) "Not Shifted"; end if
        call fftw2d_r2c_lop(.true. ,.false.,data_in,data_ft)
      end if
      
    end if

    !call mdtape_gettaper( data_in(1:nw(1),1:nw(2)) )

    ! Write Data
    !call srite( "out", data_in(1:nw(1),1:nw(2)), 4*size(data_in(1:nw(1),1:nw(2))) )
    call srite( "out", data_in(b(1):e(1),b(2):e(2)), 4*n(1)*n(2) )

    
  end do;     end do;     end do

    
  ! Deallocate memory
  deallocate( data_in , data_sf , data_ft )
  if (padd) then; deallocate( data_pd ); end if
  call fftw2d_12_r2c_lop_close()
  call tape_close() 
  call mdtape_close()  

  ! Force clean exit
  call exit (0)
  
end program
