program sdrBandpassmd
! Tape a datafile
  ! taper types: 
  ! taper types: 
  ! 1) 'zero'            --  needed: 'nothing' 
  ! 2) 'rect'            --  needed: t1,t2 
  ! 3) 'linear_up'       --  needed: t1,t2
  ! 4) 'linear_down'     --  needed: t1,t2
  ! 5) 'pyramid'         --  needed: t1,t2
  ! 6) 'pyramid_band'    --  needed: t1,t2,t3,t4
  ! 7) 'hann'            --  needed: t1,t2
  ! 8) 'hann_half'       --  needed: t1,t2
  ! 9) 'hann_band'       --  needed: t2,t2,t3,t4
  ! 10) 'cosine'         --  needed: t1,t2
  ! 11) 'gaussian'       --  needed: t1,t2,sigma,spread
  ! 12) 'gaussian_half'  --  needed: t1,t2,sigma,spread
  ! 13) 'gaussian_block' --  needed: t1,t2,t3,t4,sigma,spread
  !
  ! All tapers have the form of a band or lowcut,
  ! Notch tapers are constructed with inv=1 option.
  ! Sum result onto old data using    add=1 option.
  
  ! Multiple calls take input and overwrite if add=1
  
  use sep                      ! Main SEP module
  use fftw_mod
  use fftshift_mod
  use tapers_mod
  use mdtape_mod
  implicit none
  
  real, allocatable               :: data_in (:,:,:), data_shift(:,:,:)
  complex, allocatable            :: data_fft(:,:,:)
  integer                         :: n(3), nw(3)
  integer                         :: ii, kk
  integer                         :: type(2),unit(3),inv(3)
  real                            :: d(3),o(3),dw(3)
  real                            :: f1(3),f2(3),f3(3),f4(3)
  integer                         :: f1_ind(3),f2_ind(3),f3_ind(3),f4_ind(3)
  logical                         :: add,inv_3(3)
  real                            :: sigma(3),scale(3)
  
  real, parameter                 :: pi=3.14159265

  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  call from_history("n1", n(1) )
  call from_history("n2", n(2) )
  call from_history("n3", n(3) )
  call from_history("d1", d(1) )
  call from_history("d2", d(2) )
  call from_history("d3", d(3) )
  call from_history("o1", o(1) )
  call from_history("o2", o(2) )
  call from_history("o3", o(3) )
 
  ! Read input parameters for taper, there is lots of options :-)
  call from_param ("type" ,type , (/ 0 , 0 , 0 /) )
  call from_param ("inv"  ,inv  , (/ 0 , 0 , 0 /) )
  call from_param ("sigma",sigma, (/.1 ,.1 ,.1 /) )
  call from_param ("unit" ,unit , (/ 1 , 1 , 1 /) )  ! 1: f, 2: w
  
  call fft_r2c_units_t(.false.,n(1),d(1),nw=nw(1),df=dw(1))
  call fft_c2c_units_x(.false.,n(2),d(2),nk=nw(2),dk=dw(2))
  call fft_c2c_units_x(.false.,n(3),d(3),nk=nw(3),dk=dw(3))
  do ii=1,3
    if (unit(ii).eq.2) then; dw(ii) = 2.*pi*dw(ii); end if
  end do
  
  call from_param ("f1",f1, (/0.,0.,0./)  )
  call from_param ("f2",f2, (/ dw(1)*real(nw(1)-2),dw(2)*real(nw(2)/2-2),dw(3)*real(nw(3)/2-2) /) )
  call from_param ("f3",f3, (/ dw(1)*real(nw(1)-2),dw(2)*real(nw(2)/2-2),dw(3)*real(nw(3)/2-2) /) )
  call from_param ("f4",f4, (/ dw(1)*real(nw(1)-1),dw(2)*real(nw(2)/2-1),dw(3)*real(nw(3)/2-1) /) )
  


  ! Make sure that no taper specification gives a 1 in that direction
  do ii=1,3;
    if (type(ii)==0) then; inv(ii)=1; type(ii)=1; end if
  end do


  do ii=1,3
    if (inv(ii).eq.0) then; inv_3(ii)=.false.; else; inv_3(ii)=.true.; end if
  end do
  
  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles

  
  ! Determine tie indices in 3D
  call taper_indice(f1(1),f1_ind(1),d_in=dw(1),o_in=0.)
  call taper_indice(f2(1),f2_ind(1),d_in=dw(1),o_in=0.)
  call taper_indice(f3(1),f3_ind(1),d_in=dw(1),o_in=0.)
  call taper_indice(f4(1),f4_ind(1),d_in=dw(1),o_in=0.)

  call taper_indice(f1(2),f1_ind(2),d_in=dw(2),o_in=-nw(2)/2*dw(2))
  call taper_indice(f2(2),f2_ind(2),d_in=dw(2),o_in=-nw(2)/2*dw(2))
  call taper_indice(f3(2),f3_ind(2),d_in=dw(2),o_in=-nw(2)/2*dw(2))
  call taper_indice(f4(2),f4_ind(2),d_in=dw(2),o_in=-nw(2)/2*dw(2))

  call taper_indice(f1(3),f1_ind(3),d_in=dw(3),o_in=-nw(3)/2*dw(3))
  call taper_indice(f2(3),f2_ind(3),d_in=dw(3),o_in=-nw(3)/2*dw(3))
  call taper_indice(f3(3),f3_ind(3),d_in=dw(3),o_in=-nw(3)/2*dw(3))
  call taper_indice(f4(3),f4_ind(3),d_in=dw(3),o_in=-nw(3)/2*dw(3))

  !write(0,*) f1_ind
  !write(0,*) f2_ind
  !write(0,*) f3_ind
  !write(0,*) f4_ind



  ! Initialize tape functions
  call tape_init       (nw(1),nw(2),nw(3))
  
  ! Initialize taper functions in tapers_mod
  call tapers_init     (nw(1),nw(2),nw(3))
  call tapers_interface(3,dir_in=(/ 1 , 2, 3/),t1_in=f1_ind,t2_in=f2_ind,t3_in=f3_ind,t4_in=f4_ind,type_in=type,sigma_in=sigma,scale_in=(/ 1., 1., 1. /),inv_in=inv_3)
  call tapers_insert_tape()
  call tapers_close      ()
  call tape_shift( (/.false.,.true.,.true./) )
  
  ! Initialize FFTW
  call fftw3d_r2c_lop_init(n(1),n(2),n(3))
  
  ! Allocate the arrays datain and dataout
  allocate ( data_in(n(1),n(2),n(3)),data_shift(n(1),n(2),n(3)),data_fft(nw(1),nw(2),nw(3)) )

  ! Read in data from input file
  call sreed("in",data_in,4*size(data_in))
    
  ! FFTshift, FFT. Taper, IFFT. IFFTshift
  call fftshift3d_lop(.false.,.false.,data_in ,data_shift)
  call fftw3d_r2c_lop(.false.,.false.,data_shift,data_fft)
  call tape_op       (                           data_fft)
  call fftw3d_r2c_lop( .true.,.false.,data_shift,data_fft)
  call fftshift3d_lop( .true.,.false.,data_in ,data_shift)
  
  ! Write data to output file
  call srite( "out", data_in, 4*size(data_in) )

  ! Deallocate memory
  deallocate ( data_in,data_shift,data_fft )
  call tape_close()
  call fftw3d_r2c_lop_close()  

  ! Force clean exit
  call exit (0)
  
end program

