program sac2mtspec

!
! Thi sis a test program in order to read sac files and 
! compute the eigenspectra components or a 24 hour time
! series. 
! 

!********************************************************************
 
    use spectra
    use paths

   implicit none

   integer, parameter        :: ny_max = 4000000

   real(4), dimension(ny_max) :: y

   character (len=300) :: dir,fname, sta_list
   character (len=300),dimension(300)::list

   integer :: nlen, nerr, i, ios, ilen,ista
   real(4) :: stime, dt

   character (len=8)  :: sta                    !STATION NAME CHARACTER
   character (len=8)  :: chan, net              !CHANNEL & NETWORK OF STATION
   integer            :: iyr,idy,ihr,imn        !EARTHQUAKE TIME
   integer            :: isec,imsec             !START SECONDS OF QUAKE & SERIES
   real(4)            :: lon,lat,elev           !STATION LOCATION

   real(4)            :: sec

!  Doing coherencies

   character (len=8)  :: sta1, sta2

!  Unix things

   character (len=100) :: folder, sacfind, fpwd
   character (len=100), dimension(1000) :: sfiles
   integer                              :: nfiles, larg, istat,nsta
   logical::lex

!  Timing

   integer :: count1, count2, count_rate, count1b, count2b
   real(4) :: dtime 
 
!********************************************************************

   call system_clock(count1, count_rate)


   sacfind = trim(psac)
   call list_files(sacfind,list,nsta)

   do ista = 1,nsta
      dir = trim(sacfind)//adjustl(list(ista))
      call list_files(adjustl(trim(dir)),sfiles,nfiles)
    do i=1,nfiles
      fname=adjustl(trim(dir))//'/'//sfiles(i)
      write(6,*) fname
       if (index(sfiles(i),'.sac') == 0) cycle 
      call rsac1(fname,y,nlen,stime,dt,ny_max,nerr)
      write(6,*) nlen,stime,dt,ny_max,nerr
      if (nerr>0) then
         write(6,'(a)') 'Problem reading the file, nerr = ', nerr
         cycle
      endif

      if (abs(stime) > 30.*60) then    ! 30 minutes
         write(6,'(a,f16.4,a)') &
              'Initial time is TOO far from t=0 ', stime, ' sec'
         cycle
      endif

      if (real(nlen)*dt <= 23.*60.*60.) then   ! 23 hours
         write(6,'(a,f16.4,a)') &
              'Time series TOO short = ', real(nlen)*dt, ' sec'
         cycle
      endif

      call getfhv('stla'  ,lat    ,nerr)     !GET HEADER STATION LATITUDE
      call getfhv('stlo'  ,lon    ,nerr)     !GET HEADER STATION LONGITUDE
      call getfhv('stel'  ,elev   ,nerr)     !GET HEADER STATION ELEVATION
      call getnhv('nzyear',iyr    ,nerr)     !GET HEADER START YEAR
      call getnhv('nzjday',idy    ,nerr)     !GET HEADER START JULIAN DAY
      call getnhv('nzhour',ihr    ,nerr)     !GET HEADER START HOUR
      call getnhv('nzmin' ,imn    ,nerr)     !GET HEADER START MINUTE
      call getnhv('nzsec' ,isec   ,nerr)     !GET HEADER START SECOND
      call getnhv('nzmsec',imsec  ,nerr)     !GET HEADER START MILISECOND
      call getkhv('kstnm' ,sta    ,nerr)     !GET HEADER STATION NAME
      call getkhv('knetwk',net    ,nerr)     !GET HEADER STATION NETWORK 
      call getkhv('kcmpnm',chan   ,nerr)     !GET CHANNEL NAME (e.g. 'BHZ')

      sec = real(isec) + real(imsec)/1000.
 
      net = adjustl(net(1:2))

      if(index('ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789',sta(4:4)) == 0) then
         write(sta2,'(a)') sta(1:3)
      else
         write(sta2,'(a)') sta(1:4)
      endif

      call save_mtspec(net,sta2,chan,lat,lon,elev,iyr,idy,ihr,imn,sec,  &
                    dt,stime,nlen, y(1:nlen))
      enddo
   enddo

   call system_clock(count2, count_rate)
   dtime = real(count2-count1)/real(count_rate)
   print *, 'dt = ',dtime


end program sac2mtspec

