subroutine save_mtspec(net,sta,chan,lat,lon,elev,iyr,idy,ihr,imn,sec, &
                       dt,stime,nlen,y)

!********************************************************************

   use spectra
   use gf5
   use stats
   use param1

   implicit none

   character (*)  :: sta                    !STATION NAME CHARACTER
   character (*)  :: chan, net                   !CHANNEL & NETWORK OF STATION
   real(4)            :: lon,lat,elev           !STATION LOCATION
   integer            :: iyr,idy,ihr,imn        !EARTHQUAKE TIME
   real(4)            :: sec
   real(4)            :: dt, df, stime
   integer            :: nlen

   real(4), dimension(nlen), intent(in) :: y
   
   character (len=100)  :: fname

!   complex(4), dimension(nwin,kspec) :: yk
   complex(4), dimension(:,:), allocatable :: yk
   real(4),    dimension(nfft)       :: psd

!  Variance and STD

   real(4) :: var,std

!  Temporal parameters

   integer :: i,icnt, ilen, k, nwin
   character (len=8) :: sta2
   type(gf_mtspec), dimension(ncnt) :: test_list

!  Unix commands and folders

   character (len=20)  :: fft_dir
   character (len=200) :: tyr, tdy
   character (len=200) :: tyr_fft, tdy_fft

!  Sampling and shifting

   integer :: i1sample, iwin, imsample, issample
   real(4) :: tphase

!********************************************************************

!  Define window length based on sampling rate

   nwin = int(60.*60./dt)
   allocate(yk(nwin,kspec))

!  Calculate the STD of the entire signal for threshold

   call xvar(y,var)
   std = sqrt(var)

!  Make sure the folder to put FFTs is there. 

   fft_dir = trim(path_ft)//'gff'
   fft_dir = adjustl(fft_dir)
   call mkdir(fft_dir)

!  Sampling is not perfect, shifting

   i1sample = 0
   tphase   = 0.0
   imsample = 0
   issample = 0

!  Check for hour and minute differences

   if (ihr == 0) then
      imsample = int(real(imn)/dt) + int((sec)/dt)
      imn = 0
      sec = 0.0
   elseif (ihr == 23) then
      imsample = -int(real(60 - imn)/dt*60.) + int((sec)/dt)
      idy = idy + 1
      ihr = 0
      imn = 0
      sec = 0.0
   else
      write(6,*) 'Initial time is too far from 0:00, sta ', sta
      deallocate(yk)
      return
   endif

   if (abs(stime) > dt) then    ! Correct few seconds delay
      issample = int(stime/dt)
      tphase = stime - real(issample)*dt
   endif

   i1sample = issample + imsample

!   print *, 'ihr, imn, sec, stime, i1sample, tphase, nwin'
!   print *, ihr, imn, sec, stime, i1sample, tphase, nwin

   icnt = 0
   do i = 1,nlen,(nwin+1)/2

      icnt = icnt + 1
      if (icnt > ncnt) exit

      iwin = i-i1sample

      if (iwin < 1) then    ! Start time of window is early
         test_list(icnt)%head%iflag = 0
         cycle
      elseif (iwin+nwin > nlen) then   ! Reached end of time series
         test_list(icnt)%head%iflag = 0
         cycle
      endif

!  Find large spikes and earthquakes

      if (maxval(y(iwin:iwin+nwin))>=10.*std) THEN
         test_list(icnt)%head%iflag = 0
         cycle
      endif

      call mtspec (nwin,dt,y(iwin:iwin+nwin),tbp,kspec,df,yk)
      if (all(yk==0.)) then 
         test_list(icnt)%head%iflag = 0
         cycle
      endif

      do k = 1,kspec
         yk(1:nfft,k) = yk(1:nfft,k) - sum(real(yk(1:nfft,k)))/real(nfft)
      enddo

      psd = sum(abs(yk(1:nfft,:))**2, dim=2) 

      test_list(icnt)%head%iflag = 1
      test_list(icnt)%yk         = yk(1:nfft,:)
      test_list(icnt)%psd        = psd 
      test_list(icnt)%head%sta   = sta
      test_list(icnt)%head%chan  = chan
      test_list(icnt)%head%net   = net
      test_list(icnt)%head%lat   = lat
      test_list(icnt)%head%lon   = lon
      test_list(icnt)%head%elev  = elev
      test_list(icnt)%head%iyr   = iyr 
      test_list(icnt)%head%idy   = idy
      test_list(icnt)%head%imn   = imn
      test_list(icnt)%head%sec   = sec
      test_list(icnt)%head%df    = df
      test_list(icnt)%head%dt    = dt

   enddo

!  Generate file name to save and make sure folders are present

   call ft_name('gff',net,sta,chan,fft_dir,iyr,idy,fname)

!  Save the appropiate parameters
 
   open (12, file=fname, form='unformatted')
   do i = 1,ncnt
      write (12) test_list(i)
   enddo
   close (12)

   deallocate(yk)
   return

end subroutine save_mtspec






