program sdrMTSI2d_self1d_w_omp

  use sep
  use mtspectra_mod_omp
  use sep_timers_mod

  implicit none

  complex,allocatable  :: data_in(:,:,:),data_out(:,:)
  integer              :: n(2),n3,n4,n5,nt
  real                 :: d(2),o(2),d3,d4,o3,o4
  integer              :: ns,js,fs,ind
  integer              :: ii,jj,kk,ll,mm
  integer              :: type                    ! 1: cross-spectrum, 2: coherence, 3: deconvolution (A/B), 4: deconvolution (A\B)
  logical              :: verb

  ! For timers
  logical           :: logic
  integer           :: tm

  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)

  call start_timer_num(tm)
  call sep_init()
  call from_history("n1", n(1) ,1 )
  call from_history("n2", nt   ,1 )
  call from_history("n3", n(2) ,1 )
  call from_history("n4", n3   ,1 )
  call from_history("n5", n4   ,1 )
  call from_history("d1", d(1) ,1.)
  call from_history("d3", d(2) ,1.)
  call from_history("d4", d3   ,1.)
  call from_history("d5", d4   ,1.)
  call from_history("o1", o(1) ,0.)
  call from_history("o3", o(2) ,0.)
  call from_history("o4", o(3) ,0.)
  call from_history("o4", o3   ,0.)
  call from_history("o5", o4   ,0.)

  call from_param  ("ns",ns,n(2))
  call from_param  ("js",js,  1 )
  call from_param  ("fs",fs,  0 )

  ! Misc
  call from_param("verbose",verb ,.false.)
  call from_param("type"   ,type ,   1   )
  
  ! Update History File  
  call to_history  ("n2",         n(2) )
  call to_history  ("d2",         d(2) )
  call to_history  ("n3",      ns      )
  call to_history  ("d3",      js*d(2) )
  call to_history  ("o3", o(2)+fs*d(2) )
  call to_history  ("n4", n3 )
  call to_history  ("n5", n4 )
  call to_history  ("d4", d3 )
  call to_history  ("d5", d4 )
  call to_history  ("o4", o3 )
  call to_history  ("o5", o4 )
  
  call sep_close()
  
  allocate  ( data_in(n(1),nt,n(2)) , data_out(n(1),n(2)) )
  call mtspectra_init()
  
  ! Loup over 3rd, 4th and 5th axis
  do mm=1,n4; do ll=1,n3;
    
    ! Verbose which Cube
    write(0,*) "Plane",(mm-1)*n4+ll,"of",n4*n3
    
    ! Read data
    call sreed("in" ,data_in ,8*size(data_in))
    
    do jj=1,ns
      ! Calculate index
      ind = 1+fs+(jj-1)*js
      write(0,*) "Interfering",ind,'/',ns,"to",n(2)
      
      ! Interferometry
      select case(type)
        case(1);  call mtspectra_crsp(data_in,data_in(:, :, ind),data_out);
        case(2);  call mtspectra_cohr(data_in,data_in(:, :, ind),data_out);
        case(3);  call mtspectra_decv(data_in,data_in(:, :, ind),data_out);
        case(4);  call mtspectra_decv(data_in(:, :, ind),data_in,data_out);
      end select      
      
      ! Write data
      call srite("out",data_out,8*size(data_out))
      
    end do
    
  end do;   end do
  
  deallocate( data_in , data_out )
  call mtspectra_close()
  call stop_timer_num (tm)
  call print_timers()
  
end program
