program sdrSI2d_self1d_w

  use sep
  use spectra_mod
  use sep_timers_mod

  implicit none

  complex,allocatable  :: data_in(:,:),data_out(:,:)
  integer              :: n(2),n3,n4,n5
  real                 :: d(2),o(2),d3,d4,d5,o3,o4,o5
  integer              :: ns,js,fs,ind
  integer              :: ii,jj,kk,ll,mm
  integer              :: type                    ! 1: cross-spectrum, 2: coherence, 3: deconvolution (A/B), 4: deconvolution (A\B)
  logical              :: verb

  ! For timers
  logical           :: logic
  integer           :: tm

  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)

  call start_timer_num(tm)
  call sep_init()
  call from_history("n1", n(1) )
  call from_history("n2", n(2) )
  call from_history("n3", n3   )
  call from_history("n4", n4   )
  call from_history("n5", n5   )
  call from_history("d1", d(1) )
  call from_history("d2", d(2) )
  call from_history("o1", o(1) )
  call from_history("o2", o(2) )
  call from_history("o3", o(3) )

  call from_param  ("ns",ns,n(2))
  call from_param  ("js",js,  1 )
  call from_param  ("fs",fs,  0 )

  ! Misc
  call from_param("verbose",verb ,.false.)
  call from_param("type"   ,type ,   1   )
    
  ! Update History File  
  call to_history  ("n3",      ns      )
  call to_history  ("d3",      js*d(2) )
  call to_history  ("o3", o(2)+fs*d(2) )
  call to_history  ("n4", n3 )
  call to_history  ("n5", n4 )
  call to_history  ("n6", n5 )
  call to_history  ("d4", d3 )
  call to_history  ("d5", d4 )
  call to_history  ("d6", d5 )
  call to_history  ("o4", o3 )
  call to_history  ("o5", o4 )
  call to_history  ("o6", o5 )
  
  call sep_close()
  
  allocate  ( data_in(n(1),n(2)) , data_out(n(1),n(2)) )
  call spectra_init()
  
  ! Loup over 3rd, 4th and 5th axis
  do mm=1,n5; do ll=1,n4; do kk=1,n3;
    
    ! Verbose which Cube
    write(0,*) "Plane",(mm-1)*n5*n4+(ll-1)*n4+kk,"of",n5*n4*n3
    
    ! Read data
    call sreed("in" ,data_in ,8*size(data_in))
    
    do jj=1,ns
      ! Calculate index
      ind = 1+fs+(jj-1)*js
      write(0,*) "Interfering",ind,'/',ns,"to",n(2)
      
      ! Interferometry
      select case(type)
        case(1);  call spectra_crsp(data_in,data_in(:, ind),data_out);
        case(2);  call spectra_cohr(data_in,data_in(:, ind),data_out);
        case(3);  call spectra_decv(data_in,data_in(:, ind),data_out);
        case(4);  call spectra_decv(data_in(:, ind),data_in,data_out);
      end select      

      ! Write data
      call srite("out",data_out,8*size(data_out))
      
    end do
    
  end do;   end do;   end do
  
  deallocate( data_in , data_out )
  call stop_timer_num (tm)
  call print_timers()
  
end program
