program sdrSI2d_self2d_t_omp
  
  use sep                      ! Main SEP module
  use spectra_mod_omp
  use padd_mod
  use fftw_mod
  use fftshift_mod
  use sep_timers_mod
  use average_mod_omp
  use fftshift_io_mod

  implicit none
  
  ! Input memory variable
  real   , allocatable            :: data_iA(:,:), data_iB(:,:)
  complex, allocatable            :: data_fA(:,:), data_fB(:,:)
  real   , allocatable            :: data_pd(:,:), data_sf(:,:)
  real   , allocatable            :: data_siT(:,:), data_out(:)
  complex, allocatable            :: data_siF(:,:)
  
  ! History input parameters
  integer                         :: n(3), n4, n5,ntau,nt,nfft,nw
  real                            :: d(3)
  real                            :: o(3)
  
  ! Counting Variables
  integer                         :: ii,jj,kk
  integer                         :: ind_s,ind_r

  ! Interferometry source grid
  integer                         :: ns,nr
  integer                         :: js,jr
  integer                         :: fs,fr
  
  ! Misc
  logical                         :: conj,sym_t,true_t,verb,padd
  integer                         :: type

  ! For timers
  logical           :: logic
  integer           :: tm

  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  ! Read acis Details
  call from_history("n1", n(1) )
  call from_history("n2", n(2) )
  call from_history("n3", n(3) )
  call from_history("n4", n4   )
  call from_history("n5", n5   )
  call from_history("d1", d(1) )
  call from_history("d2", d(2) )
  call from_history("d3", d(3) )
  call from_history("o1", o(1) )
  call from_history("o2", o(2) )
  call from_history("o3", o(3) )
    
  ! FFTW dettails
  call from_param("nfft"   , nfft  ,  n(1)  )     ! Determine over what length to calculate the fft (can best be powers of two), does symmetric zero padding
  call from_param("true_t" , true_t, .true. )     ! Puts data at true indices in data-array, otherwise, it will assume all data has irrelevant absolute t,x,y coordinates
  call from_param("sym_t"  , sym_t , .true. )     ! To be implemented (to get t-symmetric output [-nt:nt])
  
  ! Misc
  call from_param("verbose",verb ,.false.)
  call from_param("ntau"   ,ntau , n(1)  )            ! Amount of time lags to print out
  call from_param("type"   ,type ,   1   )
  call from_param("conj"   ,conj ,.false.)
  
  ! read from Parameters
  call from_param  ( "nr" ,nr, n(2) )
  call from_param  ( "ns" ,ns, 1  )
  call from_param  ( "jr" ,jr, 1  )
  call from_param  ( "js" ,js, 1  )
  call from_param  ( "fs" ,fs, 0  )
  call from_param  ( "fr" ,fr, 0  )
  
  
  ! internal or external circular correlation (sym_t)
  if (true_t        ) then; nt=2*n(1)  ; else; nt=n(1)     ; end if
  nfft = max(nt,nfft)   ! make sure nfft is large enough
  if (nfft.gt.nt    ) then; padd=.true.; else; padd=.false.; end if; ! Padd logic
  
  ! Update History File
  if (sym_t) then;  call to_history  ("n1", 2*ntau    );
  else;             call to_history  ("n1",   ntau    );  end if
  call to_history  ("d1",       d(1) )
  if (sym_t) then;  call to_history  ("o1", -ntau*d(1));
  else;             call to_history  ("o1",        0. );  end if
  call to_history( "n2" , nr )
  call to_history( "n3" , ns )
  
  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
  
  ! Initialize Padd routine
  call padd_mod_init(center_in=.true.,p1_in=nfft,p2_in=n(2));
  
  ! Initialize FFTW routine
  call fftw1d_1_r2c_lop_init(nt_in=nfft); nw = nfft/2+1
  call spectra_init()
  call average_init()

  ! Allocate Memory
  allocate( data_iA (nt  ,n(2)), data_iB (nt,n(2)) )
  allocate( data_fA (nw  ,n(2)), data_fB (nw,n(2)) )
  allocate( data_siT(nfft,n(2)), data_siF(nw,n(2)) )
  allocate( data_sf (nfft,n(2)), data_out( nfft )  )
  if ( padd ) then;  allocate( data_pd(nfft,n(2)) ); end if


  ! Loop for all Source - Receiver combinations
  do kk=1,ns
    write(0,*) 'Source:', kk,'/',ns
    
    ind_s     = 1 + fs + js*(kk-1)
    
    call   sreed_window("in", 3, (/ n(1),n(2),n(3) /), (/ n(1),1,n(3) /), (/ 0,ind_s-1,0 /), (/ 1, 1, 1 /), 4, data_iB(:,:) )
    
    ! Compute (Padd) (Shift) and FFT
    if (padd) then
      if (verb) then; write(0,*) "Padded"; end if      
      ! Compute (Shift) and FFT
      call padd_lop        (.false.,.false.,data_iB,data_pd)
      call fftshift1d_1_lop(.false.,.false.,data_pd,data_sf);
    else;
      ! Compute (Shift) and FFT
      call fftshift1d_1_lop(.false.,.false.,data_iB,data_sf);
    end if
    call fftw1d_1_r2c_lop(.false.,.false.,data_sf,data_fA);
    
    do jj=1,nr
      write(0,*) 'Receiver:', jj,'/',nr
      
      ! Calculate indices
      ind_r     = 1 + fr + jr*(jj-1)
      
      ! Read plane of data
      call sreed_window("in", 3, (/ n(1),n(2),n(3) /), (/ n(1),1,n(3) /), (/ 0,ind_r-1,0 /), (/ 1, 1, 1 /), 4, data_iA(:,:) )
      
      ! Compute (Padd) (Shift) and FFT
      if (padd) then
        if (verb) then; write(0,*) "Padded"; end if      
        ! Compute (Shift) and FFT
        call padd_lop        (.false.,.false.,data_iA,data_pd)
        call fftshift1d_1_lop(.false.,.false.,data_pd,data_sf);
      else;
        ! Compute (Shift) and FFT
        call fftshift1d_1_lop(.false.,.false.,data_iA,data_sf);
      end if
      call fftw1d_1_r2c_lop(.false.,.false.,data_sf,data_fB);
      
      ! Call Plane by Plane Interferometry
      if ( conj ) then
        select case(type)
          case(1);  call spectra_crsp( data_fA, data_fB , data_siF)
          case(2);  call spectra_cohr( data_fA, data_fB , data_siF)
          case(3);  call spectra_decv( data_fA, data_fB , data_siF)
          case(4);  call spectra_decv( data_fB, data_fA , data_siF)
        end select
      else
        select case(type)
          case(1);  call spectra_crsp( data_fB, data_fA , data_siF)
          case(2);  call spectra_cohr( data_fB, data_fA , data_siF)
          case(3);  call spectra_decv( data_fB, data_fA , data_siF)
          case(4);  call spectra_decv( data_fA, data_fB , data_siF)
        end select
      end if
      

      ! Compute Inverse (Padd) (Shift) and FFT
      call fftw1d_1_r2c_lop(.true.,.false.,data_siT,data_siF);

      ! Mute the AC's
      data_siT(:,ind_r) = 0.
      data_siT(:,ind_s) = 0.

      ! Stack and write trace by trace
      call average_2_op(data_siT,data_out)

      ! Write Data
      if (sym_t) then;  call fftshift1d_srite("out", data_out,2*ntau,4);
      else;             call            srite("out", data_out,  ntau*4);  end if
      
    end do
  end do
  
  ! Deallocate and close stuff
  deallocate( data_iA , data_iB  )
  deallocate( data_fA , data_fB  )
  deallocate( data_sf , data_out )
  deallocate( data_siT, data_siF )
  if ( padd ) then;  deallocate( data_pd ); end if
  call fftw1d_1_r2c_lop_close();
  call stop_timer_num (tm)
  call print_timers()
  
  call exit(0)
  
end program
