program sdrSI2d_self2d_stack_w
  
  use sep                      ! Main SEP module
  use spectra_mod
  use average_mod
  
  implicit none
  
  ! Input memory variable
  complex, allocatable            :: datain_A(:,:), datain_B(:,:)
  complex, allocatable            ::  data_si(:,:), data_out(:)
  
  ! History input parameters
  integer                         :: n1, n2, n3
  real                            :: d1, d2, d3
  real                            :: o1, o2, o3
  
  ! Counting Variables
  integer                         :: ii,jj,kk
  integer                         :: ind_s,ind_r

  ! Interferometry source grid
  integer                         :: ns,nr
  integer                         :: js,jr
  integer                         :: fs,fr
  
  ! Misc
  logical                         :: conj
  integer                         :: type

  call sep_init    ( )
  
  ! Read in from input .H file
  call from_history( "n1", n1 )     
  call from_history( "n2", n2 )
  call from_history( "n3", n3 )
  call from_history( "d1", d1 )
  call from_history( "d2", d2 )
  call from_history( "d3", d3 )
  call from_history( "o1", o1 )
  call from_history( "o2", o2 )
  call from_history( "o3", o3 )
  
  ! read from Parameters
  call from_param  ( "nr" ,nr, n2  )
  call from_param  ( "ns" ,ns, 1  )
  call from_param  ( "jr" ,jr, 1  )
  call from_param  ( "js" ,js, 1  )
  call from_param  ( "fs" ,fs, 0  )
  call from_param  ( "fr" ,fr, 0  )
  call from_param  ( "conj",conj,.false.)
  call from_param  ( "type",type,   1   )
  
  call to_history( "n1" , n1 )
  call to_history( "n2" , nr )
  call to_history( "n3" , ns )
  
  call sep_close()
  
  ! Allocate Memory
  allocate( datain_A(n1,n3), datain_B(n1,n3), data_si(n1,n3), data_out(n1) )
  
  ! Loop for all Source - Receiver combinations
  do kk=1,ns
    write(0,*) 'Source:', kk,'/',ns
    
    ind_s     = 1 + fs + js*(kk-1)
    
    call   sreed_window("in", 3, (/ n1,n2,n3 /), (/ n1,1,n3 /), (/ 0,ind_s-1,0 /), (/ 1, 1, 1 /), 8, datain_B(:,:) )
    
    do jj=1,nr
      write(0,*) 'Receiver:', jj,'/',nr
      
      ! Calculate indices
      ind_r     = 1 + fr + jr*(jj-1)
      
      ! Read plane of data
      call sreed_window("in", 3, (/ n1,n2,n3 /), (/ n1,1,n3 /), (/ 0,ind_r-1,0 /), (/ 1, 1, 1 /), 8, datain_A(:,:) )
      
      ! Call Plane by Plane correlation
      if ( conj ) then
        select case(type)
          case(1);  call spectra_crsp( datain_A, datain_B , data_si)
          case(2);  call spectra_cohr( datain_A, datain_B , data_si);
          case(3);  call spectra_decv( datain_A, datain_B , data_si);
          case(4);  call spectra_decv( datain_B, datain_A , data_si);
        end select
      else
        select case(type)
          case(1);  call spectra_crsp( datain_B, datain_A , data_si)
          case(2);  call spectra_cohr( datain_B, datain_A , data_si);
          case(3);  call spectra_decv( datain_B, datain_A , data_si);
          case(4);  call spectra_decv( datain_A, datain_B , data_si);
        end select
      end if
      
      ! Mute the AC's
      data_si(:,ind_r) = 0.
      data_si(:,ind_s) = 0.
      
      ! Stack and write trace by trace
      call average_2_op(data_si,data_out)
      call srite( "out", data_out, 8*n1 )
      
    end do
  end do
 
  ! Deallocate
  deallocate( datain_A, datain_B, data_si, data_out)
  
end program
