program sdrSI2d_self2d_stack_wt
  
  use sep                      ! Main SEP module
  use spectra_mod
  use average_mod
  use fftw_mod
  use fftshift_io_mod
  use sep_timers_mod
  
  implicit none
  
  ! Input memory variable
  complex, allocatable            :: datain_A(:,:), datain_B(:,:)
  real   , allocatable            :: data_siT(:,:), data_out(:)
  complex, allocatable            :: data_siF(:,:)
  
  ! History input parameters
  integer                         :: n1, n2, n3, ntau
  real                            :: d1, d2, d3
  real                            :: o1, o2, o3
  
  ! Counting Variables
  integer                         :: ii,jj,kk
  integer                         :: ind_s,ind_r

  ! Interferometry source grid
  integer                         :: ns,nr
  integer                         :: js,jr
  integer                         :: fs,fr
  
  ! Misc
  logical                         :: conj,sym_t
  integer                         :: type

  ! For timers
  logical           :: logic
  integer           :: tm

  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  call sep_init    ( )
  
  ! Read in from input .H file
  call from_history( "n1", n1 )     
  call from_history( "n2", n2 )
  call from_history( "n3", n3 )
  call from_history( "d1", d1 )
  call from_history( "d2", d2 )
  call from_history( "d3", d3 )
  call from_history( "o1", o1 )
  call from_history( "o2", o2 )
  call from_history( "o3", o3 )
  
  ! read from Parameters
  call from_param  ( "nr" ,nr, n2  )
  call from_param  ( "ns" ,ns, 1  )
  call from_param  ( "jr" ,jr, 1  )
  call from_param  ( "js" ,js, 1  )
  call from_param  ( "fs" ,fs, 0  )
  call from_param  ( "fr" ,fr, 0  )
  call from_param  ( "conj",conj,.false.)
  call from_param  ( "type",type,   1   )
  call from_param  ("sym_t", sym_t, .true. )
  call from_param("ntau"   ,ntau , n1   )            ! Amount of time lags to print out

  ! Update History File
  if (sym_t) then;  call to_history  ("n1", 2*ntau    );
  else;             call to_history  ("n1",   ntau    );  end if
  call to_history  ("d1",       d1   )
  if (sym_t) then;  call to_history  ("o1", -ntau*d1  );
  else;             call to_history  ("o1",        0. );  end if
  call to_history( "n2" , nr )
  call to_history( "n3" , ns )
  call to_history( "esize",4 )
  
  call sep_close()
  
  ! Allocate Memory
  allocate( datain_A(n1,n3), datain_B(n1,n3), data_siF(n1,n3), data_siT(2*(n1-1),n3), data_out(2*(n1-1)) )
  call fftw1d_1_r2c_lop_init(nt_in=2*(n1-1));
  call spectra_init()
  call average_init()

  ! Loop for all Source - Receiver combinations
  do kk=1,ns
    write(0,*) 'Source:', kk,'/',ns
    
    ind_s     = 1 + fs + js*(kk-1)
    
    call   sreed_window("in", 3, (/ n1,n2,n3 /), (/ n1,1,n3 /), (/ 0,ind_s-1,0 /), (/ 1, 1, 1 /), 8, datain_B(:,:) )
    
    do jj=1,nr
      write(0,*) 'Receiver:', jj,'/',nr
      
      ! Calculate indices
      ind_r     = 1 + fr + jr*(jj-1)
      
      ! Read plane of data
      call sreed_window("in", 3, (/ n1,n2,n3 /), (/ n1,1,n3 /), (/ 0,ind_r-1,0 /), (/ 1, 1, 1 /), 8, datain_A(:,:) )
      
      ! Call Plane by Plane correlation
      if ( conj ) then
        select case(type)
          case(1);  call spectra_crsp( datain_A, datain_B , data_siF)
          case(2);  call spectra_cohr( datain_A, datain_B , data_siF)
          case(3);  call spectra_decv( datain_A, datain_B , data_siF)
          case(4);  call spectra_decv( datain_B, datain_A , data_siF)
        end select
      else
        select case(type)
          case(1);  call spectra_crsp( datain_B, datain_A , data_siF)
          case(2);  call spectra_cohr( datain_B, datain_A , data_siF)
          case(3);  call spectra_decv( datain_B, datain_A , data_siF)
          case(4);  call spectra_decv( datain_A, datain_B , data_siF)
        end select
      end if
      
      ! Compute Inverse (Padd) (Shift) and FFT
      call fftw1d_1_r2c_lop(.true.,.false.,data_siT,data_siF);

      ! Mute the AC's
      data_siT(:,ind_r) = 0.
      data_siT(:,ind_s) = 0.

      ! Stack and write trace by trace
      call average_2_op(data_siT,data_out)

      ! Write Data
      if (sym_t) then;  call fftshift1d_srite("out", data_out,2*ntau,4);
      else;             call            srite("out", data_out,  ntau*4);  end if

    end do
  end do
 
  ! Deallocate
  deallocate( datain_A, datain_B, data_siT, data_siF, data_out)
  
  call stop_timer_num (tm)
  call print_timers()
  
  call exit(0)
  
end program
