program sdrSI3d_self1d_w_omp

  use sep
  use spectra_mod_omp
  use sep_timers_mod

  implicit none

  complex,allocatable  :: data_in(:,:,:),data_out(:,:,:)
  integer              :: n1,n2,n3,n4
  real                 :: d1,d2,d3,d4
  real                 :: o1,o2,o3,o4
  integer              :: ns2,js2,fs2
  integer              :: ns3,js3,fs3
  integer              :: jj,kk,ll,ind2,ind3
  integer              :: type                    ! 1: cross-spectrum, 2: coherence, 3: deconvolution (A/B), 4: deconvolution (A\B)

  ! For timers
  logical           :: logic
  integer           :: tm

  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  call sep_init()
  call from_history("n1",n1      , 1 )
  call from_history("n2",n2      , 1 )
  call from_history("n3",n3      , 1 )
  call from_history("n4",n4      , 1 )
  call from_history("d1",d1      , 1.)
  call from_history("d2",d2      , 1.)
  call from_history("d3",d3      , 1.)
  call from_history("d4",d4      , 1.)
  call from_history("o1",o1      , 0.)
  call from_history("o2",o2      , 0.)
  call from_history("o3",o3      , 0.)
  call from_history("o4",o4      , 0.)
  call from_param  ("ns2",ns2    ,n2 )
  call from_param  ("ns3",ns3    ,n3 )
  call from_param  ("js2",js3    , 1 )
  call from_param  ("js3",js3    , 1 )
  call from_param  ("fs2",fs2    , 0 )
  call from_param  ("fs3",fs3    , 0 )
  call to_history  ("n4",ns2         )
  call to_history  ("n5",ns2         )
  call to_history  ("d4",   js2*d2   )
  call to_history  ("d5",   js3*d3   )
  call to_history  ("o4",o2+fs2*d2   )
  call to_history  ("o5",o3+fs3*d3   )
  call to_history  ("n6",n4          )
  call to_history  ("d6",d4          )
  call to_history  ("o6",o4          )
  call from_param  ("type",type,  1  )
  call sep_close()
  
  allocate  ( data_in(n1,n2,n3) , data_out(n1,n2,n3) )

  do ll=1,n4
    
    ! Read data
    call sreed("in" ,data_in ,8*size(data_in))
    
    do kk=1,ns3
      ! Calculate index
      ind3 = 1+fs3+(kk-1)*js3

      do jj=1,ns2
        ! Calculate index
        ind2 = 1+fs2+(jj-1)*js2
      
        ! Interferometry
        select case(type)
          case(1);  call spectra_crsp(data_in,data_in(:, ind2,ind3),data_out);
          case(2);  call spectra_cohr(data_in,data_in(:, ind2,ind3),data_out);
          case(3);  call spectra_decv(data_in,data_in(:, ind2,ind3),data_out);
          case(4);  call spectra_decv(data_in(:, ind2,ind3),data_in,data_out);
        end select      

        ! Write data
        call srite("out",data_out,8*size(data_out))
      
      end do
    end do
    
  end do
  
  deallocate( data_in , data_out )
  call stop_timer_num (tm)
  call print_timers()

end program
