module gridsearch_mod

implicit none

integer, private               :: GSdim, GSorder
integer, private               :: GSsampels
real, allocatable, private     :: model(:,:), misfit(:)
real, private                  :: GSnorm 

contains


subroutine grid_search_init(sampels,datasize,norm)
  real, intent(in)              :: norm
  integer, intent(in)           :: sampels,datasize
  
  GSsampels = sampels
  GSnorm    = norm

  allocate( model(datasize,GSsampels), misfit(GSsampels) )
  
end subroutine



subroutine grid_search(fop,data,count,modelout)

  interface
  real function fop(x,m)
    integer      :: x,m
    end function 
  end interface
  
  real                 :: data(:), modelout(:)
  integer              :: ix,id
  integer              :: count,ind(1)

  ! Loop over dataspace (1d) and model space (1d), calculate misfit
  do ix = 1,GSsampels
    do id = 1,size(data)
      model(id,ix) = fop( id, ix )
    end do
    misfit(ix) = sum(abs(model(:,ix)-data)**GSnorm,dim=1)
  end do

 
  ! Get index of maximum value
  ind = minloc(misfit)
  count = ind(1)

  ! Make sure Grid search converged within model space
  if ( count == 1 .or. count == GSsampels ) then
    modelout = 0.
  else
    modelout = model(:,count)
  end if
  
end subroutine



subroutine grid_search_close

  deallocate( model, misfit )

end subroutine


end module

