program sdrDispersionCurve
  
  use sep
  implicit none
  
  ! Axis variables
  real          :: d1,dc,d2,d3
  real          :: o1,oc,o2,o3
  integer       :: n1,nc,n2,n3
  character*50  :: label1,label2,label3,label4
  
  ! Axis counters
  integer       :: ii,jj,kk

  ! Optimization variables
  integer       :: ig
  integer       :: niter
  
  ! Line step parameters
  integer       :: nstep,is
  real          :: dstep
  
  ! Memory Variables
  real,allocatable  :: DATACUBE (:,:,:,:)
  real,allocatable  :: DISPCURVE(  :,:,:)

  ! Some more
  integer       :: indice(1)
  real          :: curfit,newfit
  
  call sep_init()

  ! Read from History
  call from_history("n1",n1,1 )
  call from_history("n2",nc,1 )
  call from_history("n3",n2,1 )
  call from_history("n4",n3,1 )
  call from_history("d1",d1,1.)
  call from_history("d2",dc,1.)
  call from_history("d3",d2,1.)
  call from_history("d4",d3,1.)
  call from_history("o1",o1,0.)
  call from_history("o2",oc,0.)
  call from_history("o3",o2,0.)
  call from_history("o4",o3,0.)
  call from_history("label1",label1," ")
  call from_history("label3",label2," ")
  call from_history("label4",label3," ")
  
  ! Read some Parameters
  call from_param("niter",niter,0)
  
  ! Write to History
  call to_history("n1",n1)
  call to_history("n2",n2)
  call to_history("n3",n3)
  call to_history("n4",1 )
  call to_history("d1",d1)
  call to_history("d2",d2)
  call to_history("d3",d3)
  call to_history("d4",1.)
  call to_history("o1",o1)
  call to_history("o2",o2)
  call to_history("o3",o3)
  call to_history("o4",0.)
  call to_history("label1",label1)
  call to_history("label2",label2)
  call to_history("label3",label3)
  call to_history("label4", " ")
  
  call sep_close()
  
  
  ! Allocate Memory
  allocate( DATACUBE (n1,nc,n2,n3) )
  allocate( DISPCURVE(n1,   n2,n3) )
  
  ! Read Data
  call sreed("in",DATACUBE,4*size(DATACUBE))
  
  ! Pick Maximum
  do kk=1,n3
  do jj=1,n2
  do ii=1,n1
    indice = maxloc( DATACUBE(ii,:,jj,kk) )
    DISPCURVE(ii,jj,kk) = oc+dc*(indice(1)-1)
  end do
  end do
  end do
  
  ! Line Search Parameters
  nstep=10
  dstep= 1
  
  ! Apply constraints update on a local-line search for each model parameter and relax globally (by itteration)
  do ig=1,niter
    
    ! For each paramaters in model space
    do ii=1,n1
    do jj=1,n2
    do kk=1,n3
    
    ! Initialize current fit
    curfit = 0.

    ! Apply positive walks
    do is=1,nstep
      ! Calculate the 'proposed' misfit
      newfit = 0.
      if (newfit.lt.curfit) then  ! Accept or not (and break)
        ! Update model and current fit
        DISPCURVE(ii,jj,kk) = DISPCURVE(ii,jj,kk)+ds
        curfit              = newfit
      else
        break
      end if
    end do
    
    ! Apply negative walks
    do is=1,nstep
      ! Calculate the 'proposed' misfit
      newfit = 0.
      if (newfit.lt.curfit) then  ! Accept or not (and break)
        ! Update model and current fit
        DISPCURVE(ii,jj,kk) = DISPCURVE(ii,jj,kk)-ds
        curfit              = newfit
      else
        break
      end if
    end do
    
    end do
    end do
    end do

  end do
  
  ! Read Data
  call srite("out",DISPCURVE,4*size(DISPCURVE))
  
  ! Deallocate Memory
  deallocate(DATACUBE,DISPCURVE)
  
end program
