program sdrDispersionCurve
  ! eps1: to penalize df
  
  use sep
  implicit none
  
  ! Axis variables
  real          :: d1,dc,d2,d3
  real          :: o1,oc,o2,o3
  integer       :: n1,nc,n2,n3
  character*50  :: label1,label2,label3,label4
  
  ! Axis counters
  integer       :: ii,jj,kk
  
  ! Optimization variables
  integer       :: ig
  integer       :: niter
  
  ! Line step parameters
  integer       :: nstep,is
  real          :: dstep
  
  ! Epsilons
  real          :: eps1

  ! Memory Variables
  real,allocatable    :: DATACUBE (:,:)
  real,allocatable    :: DISPCURVE(:)
  integer,allocatable :: DISPCURVEIND(:)
  real,allocatable    :: FWEIGHT(:)

  ! Some more
  integer       :: indice(1),bnd1
  real          :: curfit,newfit,dertiv
  
  call sep_init()

  ! Read from History
  call from_history("n1",n1,1 )
  call from_history("n2",nc,1 )
  call from_history("n3",n2,1 )
  call from_history("n4",n3,1 )
  call from_history("d1",d1,1.)
  call from_history("d2",dc,1.)
  call from_history("d3",d2,1.)
  call from_history("d4",d3,1.)
  call from_history("o1",o1,0.)
  call from_history("o2",oc,0.)
  call from_history("o3",o2,0.)
  call from_history("o4",o3,0.)
  call from_history("label1",label1," ")
  call from_history("label3",label2," ")
  call from_history("label4",label3," ")
  
  ! Read some Parameters
  call from_param("niter",niter, 0)
  call from_param("nstep",nstep, 0)
  call from_param("dstep",dstep,dc)

  call from_param("eps1" ,eps1 ,.1)
  
  ! Write to History
  call to_history("n1",n1)
  call to_history("n2",n2)
  call to_history("n3",n3)
  call to_history("n4",1 )
  call to_history("d1",d1)
  call to_history("d2",d2)
  call to_history("d3",d3)
  call to_history("d4",1.)
  call to_history("o1",o1)
  call to_history("o2",o2)
  call to_history("o3",o3)
  call to_history("o4",0.)
  call to_history("label1",label1)
  call to_history("label2",label2)
  call to_history("label3",label3)
  call to_history("label4", " ")
  

  call sep_close()
  

  ! Allocate Memory
  allocate( DATACUBE (n1,nc) )
  allocate( DISPCURVE(n1),DISPCURVEIND(n1) )
  DATACUBE     = 0.
  DISPCURVEIND = 0.
  DISPCURVE    = 0.
  
  ! Loop over 3rd and 4th
  do kk=1,n3
  do jj=1,n2

  ! Read Data
  call sreed("in",DATACUBE,4*n1*nc)
  
  ! Pick Maximum
  do ii=1,n1
    indice = minloc( DATACUBE(ii,:) )
    DISPCURVEIND(ii) = indice(1)
    DISPCURVE   (ii) = oc+dc*(DISPCURVEIND(ii)-1)
  end do

  ! Modify Misfit plane by frequency dependent weighting
  if (exist_file ("fweight")) then
    allocate(   FWEIGHT(n1) )
    call sreed("fweight",FWEIGHT,4*n1)
    do ii=1,n1
      DATACUBE(ii,:) = FWEIGHT(ii)*DATACUBE(ii,:)
    end do
    deallocate( FWEIGHT )
  end if


  ! Apply constraints update on a local-line search for each model parameter and relax globally (by itteration)
  do ig=1,niter
    
    ! Apply positive walks
    do is=1,nstep
     
      if (DISPCURVEIND(1)+1 .le. nc) then
        ! Initialize current fit
        curfit = DATACUBE(1,DISPCURVEIND(1)  ) + eps1*abs( DISPCURVE(2) - DISPCURVE(1)      )/d1
        ! Calculate the 'proposed' misfit
        newfit = DATACUBE(1,DISPCURVEIND(1)+1) + eps1*abs( DISPCURVE(2) - DISPCURVE(1) - dc )/d1
        if (newfit.lt.curfit) then  ! Accept or not (and break)
          ! Update model
          DISPCURVEIND(1) = DISPCURVEIND(1) +  1
          DISPCURVE   (1) = DISPCURVE   (1) + dc
        else
          exit
        end if
      end if
     
    end do
    
    ! Apply negative walks
    do is=1,nstep
     
      if (DISPCURVEIND(1)-1 .ge. 1) then
        ! Initialize current fit
        curfit = DATACUBE(1,DISPCURVEIND(1)  ) + eps1*abs( DISPCURVE(2) - DISPCURVE(1)      )/d1
        ! Calculate the 'proposed' misfit
        newfit = DATACUBE(1,DISPCURVEIND(1)-1) + eps1*abs( DISPCURVE(2) - DISPCURVE(1) + dc )/d1
        if (newfit.lt.curfit) then  ! Accept or not (and break)
          ! Update model
          DISPCURVEIND(1) = DISPCURVEIND(1) -  1
          DISPCURVE   (1) = DISPCURVE   (1) - dc
        else
          exit
        end if
      end if
       
    end do

    ! For each paramaters in model space
    do ii=2,n1
    
      ! Apply positive walks
      do is=1,nstep
       
        if (DISPCURVEIND(ii)+1 .le. nc) then
          ! Initialize current fit
          curfit = DATACUBE(ii,DISPCURVEIND(ii)  ) + eps1*abs( DISPCURVE(ii)      - DISPCURVE(ii-1) )/d1
          ! Calculate the 'proposed' misfit
          newfit = DATACUBE(ii,DISPCURVEIND(ii)+1) + eps1*abs( DISPCURVE(ii) + dc - DISPCURVE(ii-1) )/d1
          if (newfit.lt.curfit) then  ! Accept or not (and break)
            ! Update model
            DISPCURVEIND(ii) = DISPCURVEIND(ii) +  1
            DISPCURVE   (ii) = DISPCURVE   (ii) + dc
          else
            exit
          end if
        end if
       
      end do
      
      ! Apply positive walks
      do is=1,nstep
       
        if (DISPCURVEIND(ii)-1 .ge. 1) then
          ! Initialize current fit
          curfit = DATACUBE(ii,DISPCURVEIND(ii)  ) + eps1*abs( DISPCURVE(ii)      - DISPCURVE(ii-1) )/d1
          ! Calculate the 'proposed' misfit
          newfit = DATACUBE(ii,DISPCURVEIND(ii)-1) + eps1*abs( DISPCURVE(ii) - dc - DISPCURVE(ii-1) )/d1
          if (newfit.lt.curfit) then  ! Accept or not (and break)
            ! Update model
            DISPCURVEIND(ii) = DISPCURVEIND(ii) -  1
            DISPCURVE   (ii) = DISPCURVE   (ii) - dc
          else
            exit
          end if
        end if
       
      end do
    
    end do

    
   ! ! Construct new Dispersion Curve
   ! do ii=1,n1
   !   DISPCURVE   (ii) = oc+dc*(DISPCURVEIND(ii)-1)
   ! end do

  end do

  ! Read Data
  call srite("out",DISPCURVE,4*size(DISPCURVE))
  
  end do
  end do
  
  ! Deallocate Memory
  deallocate( DATACUBE )
  deallocate( DISPCURVE , DISPCURVEIND )

end program
