program sdrSPAC_SCALING_FIT_omp

  use sep
  implicit none
  
  real, parameter   :: pi = 3.14159265
  real, allocatable :: DATA(:,:), MISFITS(:,:), MISFIT(:,:),FOP(:),f(:),r(:),C(:),SCALING(:),SCALED(:),RWEIGHT(:)
  integer,allocatable :: r_ind(:)
  integer           :: ii,jj,kk,ll,mm,im,ji,is,indices(2)
  real              :: cnorm,snorm
  
  real              :: fmin,fmax,rmin,rmax,cmin,cmax,jmin,jmax,smin,smax,sbnds(2)
  real              :: C_w,frequency
  
  real              :: d1,d2,d3,d4,dc,dj,o1,o2,o3,o4,ds,ds_target,ds_target_m
  integer           :: n1,n2,n3,n4,nc,nj,f1,f2,f3,f4,nr,nf,x0_ind,ns,sfact
  character*50      :: label1,label2,label3,label4

  integer           :: unit1

  real              :: besj0

  write(0,*) '-------------------'

  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 

  ! Read from History
  call from_history ( "n1", n1 ,1  )
  call from_history ( "n2", n2 ,1  )
  call from_history ( "n3", n3 ,1  )
  call from_history ( "n4", n4 ,1  )
  call from_history ( "o1", o1 ,0. )
  call from_history ( "o2", o2 ,0. )
  call from_history ( "o3", o3 ,0. )
  call from_history ( "o4", o4 ,0. )
  call from_history ( "d1", d1 ,1. )
  call from_history ( "d2", d2 ,1. )
  call from_history ( "d3", d3 ,1. )
  call from_history ( "d4", d4 ,1. )
 ! call from_history ( "label1", label1, "" )
 ! call from_history ( "label3", label3, "" )
 ! call from_history ( "label4", label4, "" )

  call from_param ( "cnorm", cnorm, 2. )
  call from_param ( "unit1" , unit1, 1  )
  
  ! Set and get input parameters
  nf = n1  ! later reset for each data-slice
  nr = n2  ! later reset for each data-slice
  
  call from_param( "smin" , smin      ,  0. )
  call from_param( "smax" , smax      , -1. )
  call from_param(  "ns"  ,  ns       ,  5  )
  call from_param(  "ds"  , ds_target , -1. )   ! target resolution on scaling factor
  
  call from_param( "fmin" , fmin,    d1*2.*pi )
  call from_param( "fmax" , fmax, nf*d1*2.*pi )
  
  call from_param( "fmin" , rmin,    0. )
  call from_param( "fmax" , rmax, d2*nr ) 
  
  call from_param( "cmin", cmin,    0.    )
  call from_param( "cmax", cmax, 2000.    )
  call from_param(  "nc" ,  nc ,int(cmax) )
  dc   = (cmax-cmin)/nc
  
  call from_param(  "nj" ,  nj ,     nc   )
  
  call to_history ( "n1", n1 )
  call to_history ( "n2", nc )
  call to_history ( "n3", n3 )
  call to_history ( "n4", n4 )
  
  call to_history ( "d1", d1 )
  call to_history ( "d2", dc )
  call to_history ( "d3", d3 )
  call to_history ( "d4", d4 )
  
  call to_history ( "o1", o1+real(f1)*d1 )
  call to_history ( "o2", cmin )
  call to_history ( "o3", o3+real(f3)*d3 )
  call to_history ( "o4", o4+real(f4)*d4 )
  
  call to_history ( "label2", "Phase Velocity" )
  
  if ( exist_file ("scaling") ) then;
    call to_history ( "n1", nf , "scaling")
    call to_history ( "n2", n3 , "scaling")
    call to_history ( "n3", n4 , "scaling")

    call to_history ( "d1", d1 , "scaling")
    call to_history ( "d2", d3 , "scaling")
    call to_history ( "d3", d4 , "scaling")

    call to_history ( "o1", o1+real(f1)*d1 , "scaling")
    call to_history ( "o2", o3+real(f3)*d3 , "scaling")
    call to_history ( "o3", o4+real(f4)*d4 , "scaling")
  end if
  
  if ( exist_file ("mweight") ) then;
    call to_history ( "n1", n3 , "mweight")
    call to_history ( "n2", n4 , "mweight")

    call to_history ( "d1", d3 , "mweight")
    call to_history ( "d2", d4 , "mweight")

    call to_history ( "o1", o3+real(f3)*d3 , "scaling")
    call to_history ( "o2", o4+real(f4)*d4 , "scaling")
  end if
  
  if (unit1.eq.1) then; d1 = 2.*pi*d1; end if
  
  ! Close all I/O to headerfiles
  call sep_close  ( )
  
  
  ! Allocations
  allocate(  DATA (nf,nr),FOP(nj),MISFIT(nf,nc),MISFITS(nc,ns),SCALING(ns),SCALED(nf),RWEIGHT(nr) )
  allocate(  f(nf) , r(nr) , r_ind(nr) , C(nc) )

  ! Create C model-axis
  C        = 0.
  do ii=1,nc
    C(ii) = cmin + dc*(ii-1)
  end do
  
  
  ! Create or read RWEIGHT from file
  if ( exist_file ("rweight") ) then;
    call sreed("rweight",RWEIGHT,4*size(RWEIGHT))
  else;
    RWEIGHT = 1.
  end if


  ! Loop over Midpoint
  do mm=1,n4
   do ll=1,n3
    im = (mm-1)*n4+ll
    write(0,*) 'Midpoint:', im, '/', n3*n4
    
    ! Set a couple of Variables to Zero :-)
    DATA   = 0.
    MISFIT = 0.
    f      = 0.
    r      = 0.
    r_ind  = 0
    nr     = 0
    
    ! Read Data
    call sreed("in",DATA,4*size(DATA))

    ! Create r-axis for this plane
    do jj=1,n2
      if (sum(abs(DATA(:,jj))).ne.0.) then
        nr        = nr + 1
        r(nr)     = abs(o2+d2*(jj-1))
        r_ind(nr) = jj
      end if
    end do
    
    ! Create f-axis for this plane
    nf=0
    do ii=1,n1
      nf    =   nf + 1
      f(nf) = fmin + d1*(ii-1)
    end do
    
    ! Check exception
    if (C(1).eq.0.) then; C(1) = epsilon(C(1)); end if;
    
    ! Guess for this midpoint
!    ds_target_m = 
    if (ds_target.eq.-1) then
      ds_target_m = (2.*maxval(maxval(abs(DATA),1),1)-smin)/(ns*10)
    else
      ds_target_m = ds_target
    end if

    ! Loop over frequency
    do ii=1,nf

      frequency = f(ii)

      ! Reset scaling bounds
      sbnds(1)=smin
      if (smax.eq.-1.) then;  
        sbnds(2)=2.*maxval(abs(DATA(ii,:)))
      end if

      ! Calculate Initial ds
      ds = (sbnds(2)-sbnds(1))/ns

      ! Itterated Grid search
      MISFITS = 0.
      SCALING = 0.
      indices = (/1 , 1 /)
      do while (ds.ge.ds_target_m)
        
        ! Calculate new ds
        ds = (sbnds(2)-sbnds(1))/ns
        do is=1,ns
          SCALING(is) = sbnds(1)+(is-1)*ds
        end do
        
        ! Grid Search
        do is=1,ns
          snorm = SCALING(is)
          do kk=1,nc
            C_w = C(kk)
            do jj=1,nr
              MISFITS(kk,is) = MISFITS(kk,is) + RWEIGHT(r_ind(jj))*abs( DATA(ii,r_ind(jj)) - snorm*besj0(r(jj)*frequency/C_w) )**cnorm
            end do
          end do
        end do
        
        ! Get indices of maximum
        indices= minloc( MISFITS(:,:) )
        
        ! Update bounds
        sbnds(1)=SCALING(indices(2)-1)
        sbnds(2)=SCALING(indices(2)+1)
        
        ! Calculate new ds
        ds = (sbnds(2)-sbnds(1))/ns
      end do
      
      ! Store misfit scan
      MISFIT(ii,:) = MISFITS(:,indices(2))
      SCALED(ii  ) = SCALING(  indices(2))
      
      ! Reset some variables
      MISFITS = 0.
      SCALING = 0.
      
    end do
    
    ! Determine best scaling factor for misfits
    if ( exist_file ("scaling") ) then;
      call srite( "scaling" , SCALED , 4*size(SCALED) )
    end if

    ! Determine best scaling factor for misfits
    if ( exist_file ("mweight") ) then;
      call srite( "mweight" , real(nr) , 4 )
    end if

    ! Write out best Misfit Scan
    call srite("out",MISFIT,4*size(MISFIT) )

    end do
  end do


  ! Deallocations
  deallocate(  DATA  , FOP , MISFIT , SCALING , MISFITS , SCALED , RWEIGHT )
  deallocate(  f , r , r_ind , C )
  
  write(0,*) '-------------------'
  call exit(0)
  
end program
