program sdrSpectra1d
! Tape a datafile
  ! taper types: 
  ! taper types: 
  ! 1) 'zero'            --  needed: 'nothing' 
  ! 2) 'rect'            --  needed: t1,t2 
  ! 3) 'linear_up'       --  needed: t1,t2
  ! 4) 'linear_down'     --  needed: t1,t2
  ! 5) 'pyramid'         --  needed: t1,t2
  ! 6) 'pyramid_band'    --  needed: t1,t2,t3,t4
  ! 7) 'hann'            --  needed: t1,t2
  ! 8) 'hann_half'       --  needed: t1,t2
  ! 9) 'hann_band'       --  needed: t2,t2,t3,t4
  ! 10) 'cosine'         --  needed: t1,t2
  ! 11) 'gaussian'       --  needed: t1,t2,sigma,spread
  ! 12) 'gaussian_half'  --  needed: t1,t2,sigma,spread
  ! 13) 'gaussian_block' --  needed: t1,t2,t3,t4,sigma,spread
  !
  ! All tapers have the form of a band or lowcut,
  ! Notch tapers are constructed with inv=1 option.
  ! Sum result onto old data using    add=1 option.
  
  ! Multiple calls take input and overwrite if add=1
  
  use sep                      ! Main SEP module
  use tapers_mod
  use tape_mod
  use padd_mod
  use fftw_mod
  use fftshift_mod
  use spectra_mod
  use collect_mod
  implicit none
  
  ! Memory variables
  real,allocatable                :: data_in(:,:,:)
  real,allocatable                :: data_pd(:,:,:)
  real,allocatable                :: data_sf(:,:,:)
  complex,allocatable             :: data_ft(:,:,:)
  real,allocatable                :: data_sp(:,:,:)
  
  ! Axis units
  integer                         :: n(3),nt,nw,n4,n5,b,e
  real                            :: d(3),dw,o(3)
  
  ! For Tapers
  integer                         :: t1_ind(1),t2_ind(1),t3_ind(1),t4_ind(1)
  real                            :: t1    (1),t2    (1),t3    (1),t4    (1)
  real                            :: sigma(1)
  integer                         :: type(1)
  logical                         :: inv
  logical                         :: taper,padd
  
  ! for FFTdetails
  integer                         :: nfft,dir
  integer                         :: unit(1)
  logical                         :: shift,symt,true_t,true_x,true_y

  ! Spectrum type
  integer                         :: stype              ! 1: ampl    2: powr
  
  ! Misc
  integer                         :: ii,jj,kk,ll,mm
  real, parameter                 :: pi=3.14159265
  logical                         :: verb,collect_l
  
  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  ! Read acis Details
  call from_history("n1", n(1) , 1)
  call from_history("n2", n(2) , 1)
  call from_history("n3", n(3) , 1)
  call from_history("n4", n4   , 1)
  call from_history("n5", n5   , 1)
  call from_history("d1", d(1) ,1.)
  call from_history("d2", d(2) ,1.)
  call from_history("d3", d(3) ,1.)
  call from_history("o1", o(1) ,0.)
  call from_history("o2", o(2) ,0.)
  call from_history("o3", o(3) ,0.)

  call from_param ("dir",dir, 1 )

  ! Read input parameters for taper, there is lots of options :-)
  call from_param ("type" ,type , (/ 0 /) )
  call from_param ("inv"  ,inv  , .false. )
  call from_param ("sigma",sigma, (/.1 /) )

  ! Optionally get the average
  call from_param ("collect",collect_l,.false.)
  
  if (collect_l) then
    call to_history("n4", 1 )
    call to_history("n5", 1 )
  end if

  ! Taper tie points
  call from_param ("t1",t1,(/0./)                         )        ! taper in 
  call from_param ("t2",t2,(/0./) + d(dir)*real(n(dir)-2) )
  call from_param ("t3",t3,(/0./) + d(dir)*real(n(dir)-2) )
  call from_param ("t4",t4,(/0./) + d(dir)*real(n(dir)-1) )

  ! FFTW dettails
  call from_param("shift"  , shift ,.true. )
  call from_param("nfft"   , nfft  ,   n(dir)   )     ! Determine over what length to calculate the fft (can best be powers of two), does symmetric zero padding
  call from_param("true_t" , true_t,  .false.   )     ! Puts data at true indices in data-array, otherwise, it will assume all data has irrelevant absolute t,x,y coordinates
  call from_param("true_x" , true_x,  .false.   )
  call from_param("unit"   , unit  , (/ 1 /)    )     ! Kind of units to put on the axis: 1:frequency, 2:radial frequency    (also for spatial)
  call from_param("symt"   , symt  ,  .false.   )         ! To be implemented (to get t-symmetric output [-nt:nt])
  
  ! Spectrum type (1: ampl, 2: powr)
  call from_param ("stype"  ,stype, 1     )              ! Spectrum type: 1: ampl, 2: powr
  call from_param ("verbose",verb ,.false.)
  
  ! Determine b(ginning) and e(nd) indices for data_in
  ! Check if nfft is long enough for true_t and true_x
  if (true_t.or.true_x.or.true_y) then
    call taper_indice(o(dir)                  ,b,d_in=d(dir),o_in=0.)
    call taper_indice(o(dir)+(n(dir)-1)*d(dir),e,d_in=d(dir),o_in=0.)

    nt   =  2*max( abs(e),abs(b),abs(b-2),abs(e-2) )
    nfft = max(nt,nfft)   ! make sure nfft is large enough

    ! Determine final indices for data_in
    call taper_indice(o(dir)                  ,b,d_in=d(dir),o_in=-nt/2*d(dir))
    call taper_indice(o(dir)+(n(dir)-1)*d(dir),e,d_in=d(dir),o_in=-nt/2*d(dir))
  else
     b   =   1
     e   = n(dir)
    nt   = n(dir)
    nfft = max(nt,nfft)   ! make sure nfft is large enough
  end if
  
  
  !write(0,*) "=================="   
  !write(0,*) "b",b
  !write(0,*) "e",e
  !write(0,*) "n",n
  !write(0,*) "nt",nt
  !write(0,*) "nfft",nfft
  !write(0,*) "=================="   
  
  ! Padd logic
  if (nfft.gt.nt) then; padd=.true.; else; padd=.false.; end if;
  
  ! Calculate Fourier units
  call fft_r2c_units_t(.false.,nfft,d(dir),nw=nw,df=dw)
  
  if (unit(1).eq.2) then; dw = 2.*pi*dw; end if

  ! Write to output history
  if (dir.eq.1) then;  call to_history("n1", nw );  end if
  if (dir.eq.2) then;  call to_history("n2", nw );  end if
  if (dir.eq.3) then;  call to_history("n3", nw );  end if

  if (dir.eq.1) then;  call to_history("d1", dw );  end if
  if (dir.eq.2) then;  call to_history("d2", dw );  end if
  if (dir.eq.3) then;  call to_history("d3", dw );  end if

  if (dir.eq.1) then;  call to_history("o1", 0. );  end if
  if (dir.eq.2) then;  call to_history("o2", 0. );  end if
  if (dir.eq.3) then;  call to_history("o3", 0. );  end if

  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
    


  ! To taper or not to taper
  if (type(1).gt.1) then; taper = .true.; else; taper=.false.; end if

  if (taper) then

    ! Determine tie indices
    call taper_indice(t1(1),t1_ind(1),d_in=d(dir),o_in=0.)
    call taper_indice(t2(1),t2_ind(1),d_in=d(dir),o_in=0.)
    call taper_indice(t3(1),t3_ind(1),d_in=d(dir),o_in=0.)
    call taper_indice(t4(1),t4_ind(1),d_in=d(dir),o_in=0.)

    ! Initialize tape functions
    call tape_init       (n(1),n(2),n(3))

    ! Initialize taper functions in tapers_mod
    call tapers_init     (n(1),n(2),n(3))
    call tapers_interface(1,dir_in=(/dir/),t1_in=t1_ind,t2_in=t2_ind,t3_in=t3_ind,t4_in=t4_ind,type_in=type,sigma_in=sigma,scale_in=(/ 1./),inv_in=(/inv/))
    call tapers_insert_tape()
    call tapers_close      ()
  end if

  ! Initialize Padd routine
  if (dir.eq.1) then;  call padd_mod_init(center_in=.true.,p1_in=nfft,p2_in=n(2),p3_in=n(3));  end if
  if (dir.eq.2) then;  call padd_mod_init(center_in=.true.,p1_in=n(1),p2_in=nfft,p3_in=n(3));  end if
  if (dir.eq.3) then;  call padd_mod_init(center_in=.true.,p1_in=n(1),p2_in=n(2),p3_in=nfft);  end if

  ! Initialize FFTW routine
  if (dir.eq.1) then;  call fftw1d_1_r2c_lop_init(nt_in=nfft);  end if
  if (dir.eq.2) then;  call fftw1d_2_r2c_lop_init(nt_in=nfft);  end if
  if (dir.eq.3) then;  call fftw1d_3_r2c_lop_init(nt_in=nfft);  end if
  
  ! Allocate the arrays datain and dataout
  if (dir.eq.1) then;
    allocate( data_in(nt, n(2), n(3)), data_sf(nfft,n(2),n(3)) , data_ft(nw,n(2),n(3)) , data_sp(nw,n(2),n(3)) )
    if ( padd ) then;  allocate( data_pd(nfft,n(2),n(3)) ); end if
  end if
  if (dir.eq.2) then;
    allocate( data_in(n(1), nt, n(3)) , data_sf(n(1),nfft,n(3)) , data_ft(n(1),nw,n(3)) , data_sp(n(1),nw,n(3)) )
    if ( padd ) then;  allocate( data_pd(n(1),nfft,n(3)) ); end if
  end if
  if (dir.eq.3) then;
    allocate( data_in(n(1), n(2), nt) , data_sf(n(1),n(2),nfft) , data_ft(n(1),n(2),nw) , data_sp(n(1),n(2),nw) )
    if ( padd ) then;  allocate( data_pd(n(1),n(2),nfft) ); end if
  end if

  if (collect_l) then
    if (dir.eq.1) then;  call collect_init(nw,n(2),n(3));  end if
    if (dir.eq.2) then;  call collect_init(n(1),nw,n(3));  end if
    if (dir.eq.3) then;  call collect_init(n(1),n(2),nw);  end if
  end if

  ! Loup over 4th and 5th axis
  do mm=1,n5; do ll=1,n4;
    
    ! Verbose which Cube
    write(0,*) "Cube",(mm-1)*n5+ll,"of",n4*n5
    
    ! Read Data
    data_in = 0.
    if (dir.eq.1) then;  call sreed( "in" , data_in(b:e,:,:), 4*n(1)*n(2)*n(3) ); end if
    if (dir.eq.2) then;  call sreed( "in" , data_in(:,b:e,:), 4*n(1)*n(2)*n(3) ); end if
    if (dir.eq.3) then;  call sreed( "in" , data_in(:,:,b:e), 4*n(1)*n(2)*n(3) ); end if

    ! Taper data
    if (taper) then;
      if (verb) then; write(0,*) "Tapered"; end if
      if (dir.eq.1) then;  call tape_op( data_in(b:e,:,:) ); end if
      if (dir.eq.2) then;  call tape_op( data_in(:,b:e,:) ); end if
      if (dir.eq.3) then;  call tape_op( data_in(:,:,b:e) ); end if
    end if

    ! Compute (Padd) (Shift) and FFT
    if (padd) then
      if (verb) then; write(0,*) "Padded"; end if
      call padd_lop(.false.,.false.,data_in,data_pd)
      
      if (shift) then
        if (verb) then; write(0,*) "Shifted"; end if
        ! Compute (Shift) and FFT
        if (dir.eq.1) then;  call fftshift1d_1_lop(.false.,.false.,data_pd,data_sf); end if
        if (dir.eq.2) then;  call fftshift1d_2_lop(.false.,.false.,data_pd,data_sf); end if
        if (dir.eq.3) then;  call fftshift1d_3_lop(.false.,.false.,data_pd,data_sf); end if
        if (dir.eq.1) then;  call fftw1d_1_r2c_lop(.false.,.false.,data_sf,data_ft); end if
        if (dir.eq.2) then;  call fftw1d_2_r2c_lop(.false.,.false.,data_sf,data_ft); end if
        if (dir.eq.3) then;  call fftw1d_3_r2c_lop(.false.,.false.,data_sf,data_ft); end if
      else
        if (verb) then; write(0,*) "Not Shifted"; end if
        ! Compute Shift, FFT and 
        if (dir.eq.1) then;  call fftw1d_1_r2c_lop(.false.,.false.,data_pd,data_ft); end if
        if (dir.eq.2) then;  call fftw1d_2_r2c_lop(.false.,.false.,data_pd,data_ft); end if
        if (dir.eq.3) then;  call fftw1d_3_r2c_lop(.false.,.false.,data_pd,data_ft); end if
      end if
      
    else;
      
      if (shift) then
        if (verb) then; write(0,*) "Shifted"; end if
        ! Compute (Shift) and FFT
        if (dir.eq.1) then;  call fftshift1d_1_lop(.false.,.false.,data_in,data_sf); end if
        if (dir.eq.2) then;  call fftshift1d_2_lop(.false.,.false.,data_in,data_sf); end if
        if (dir.eq.3) then;  call fftshift1d_3_lop(.false.,.false.,data_in,data_sf); end if
        if (dir.eq.1) then;  call fftw1d_1_r2c_lop(.false.,.false.,data_sf,data_ft); end if
        if (dir.eq.2) then;  call fftw1d_2_r2c_lop(.false.,.false.,data_sf,data_ft); end if
        if (dir.eq.3) then;  call fftw1d_3_r2c_lop(.false.,.false.,data_sf,data_ft); end if
      else
        if (verb) then; write(0,*) "Not Shifted"; end if
        ! Compute Shift, FFT and 
        if (dir.eq.1) then;  call fftw1d_1_r2c_lop(.false.,.false.,data_in,data_ft); end if
        if (dir.eq.2) then;  call fftw1d_2_r2c_lop(.false.,.false.,data_in,data_ft); end if
        if (dir.eq.3) then;  call fftw1d_3_r2c_lop(.false.,.false.,data_in,data_ft); end if
      end if
      
    end if
    
    ! Compute Spectrum
    if (stype.eq.1) then
      if (verb) then; write(0,*) "Type 1 Spectrum"; end if
      call spectra_ampl (data_ft,data_sp)
    else
      if (verb) then; write(0,*) "Type 2 Spectrum"; end if
      call spectra_powr (data_ft,data_sp)
    end if

    ! Write Data
    if (collect_l) then
      call collect(data_sp)
    else
      call srite( "out", data_sp, 4*size(data_sp));
    end if

  end do;     end do;
  
  if (collect_l) then
    call get_collected_average(data_sp,n4*n5)
    call srite( "out", data_sp, 4*size(data_sp));
  end if
  
  ! Deallocate and close stuff
  deallocate( data_in , data_sf , data_ft , data_sp )
  if (padd)      then;  deallocate( data_pd );         end if
  if (taper)     then;  call tape_close();             end if
  if (dir.eq.1)  then;  call fftw1d_1_r2c_lop_close(); end if
  if (dir.eq.2)  then;  call fftw1d_2_r2c_lop_close(); end if
  if (dir.eq.3)  then;  call fftw1d_3_r2c_lop_close(); end if
  if (collect_l) then;  call collect_close( );         end if
  
  ! Force clean exit
  call exit (0)
  
end program

