program sdrSpectra3d
! Tape a datafile
  ! taper types: 
  ! taper types: 
  ! 1) 'zero'            --  needed: 'nothing' 
  ! 2) 'rect'            --  needed: t1,t2 
  ! 3) 'linear_up'       --  needed: t1,t2
  ! 4) 'linear_down'     --  needed: t1,t2
  ! 5) 'pyramid'         --  needed: t1,t2
  ! 6) 'pyramid_band'    --  needed: t1,t2,t3,t4
  ! 7) 'hann'            --  needed: t1,t2
  ! 8) 'hann_half'       --  needed: t1,t2
  ! 9) 'hann_band'       --  needed: t2,t2,t3,t4
  ! 10) 'cosine'         --  needed: t1,t2
  ! 11) 'gaussian'       --  needed: t1,t2,sigma,spread
  ! 12) 'gaussian_half'  --  needed: t1,t2,sigma,spread
  ! 13) 'gaussian_block' --  needed: t1,t2,t3,t4,sigma,spread
  !
  ! All tapers have the form of a band or lowcut,
  ! Notch tapers are constructed with inv=1 option.
  ! Sum result onto old data using    add=1 option.
  
  ! Multiple calls take input and overwrite if add=1
  
  use sep                      ! Main SEP module
  use tapers_mod
  use tape_mod
  use padd_mod
  use fftw_mod
  use fftshift_mod
  use spectra_mod
  use collect_mod
  implicit none
  
  ! Memory variables
  real,allocatable                :: data_in(:,:,:)
  real,allocatable                :: data_pd(:,:,:)
  real,allocatable                :: data_sf(:,:,:)
  complex,allocatable             :: data_ft(:,:,:)
  real,allocatable                :: data_sp(:,:,:)
  
  ! Axis units
  integer                         :: n(3),nt(3),nw(3),n4,n5,b(3),e(3),ob
  real                            :: d(3),dw(3),o(3)
  
  ! For Tapers
  integer                         :: t1_ind(3),t2_ind(3),t3_ind(3),t4_ind(3)
  real                            :: t1    (3),t2    (3),t3    (3),t4    (3)
  real                            :: sigma (3)
  integer                         :: type  (3),inv   (3)
  logical                         :: inv_3 (3)
  logical                         :: taper,padd
  
  ! for FFTdetails
  integer                         :: nfft(3)
  integer                         :: unit(3)
  logical                         :: shift,symt,true_t,true_x,true_y

  ! Spectrum type
  integer                         :: stype              ! 1: ampl    2: powr
  
  ! Misc
  integer                         :: ii,jj,kk,ll,mm
  real, parameter                 :: pi=3.14159265
  logical                         :: verb,collect_l
  
  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  ! Read acis Details
  call from_history("n1", n(1) )
  call from_history("n2", n(2) )
  call from_history("n3", n(3) )
  call from_history("n4", n4   )
  call from_history("n5", n5   )
  call from_history("d1", d(1) )
  call from_history("d2", d(2) )
  call from_history("d3", d(3) )
  call from_history("o1", o(1) )
  call from_history("o2", o(2) )
  call from_history("o3", o(3) )

  ! Read input parameters for taper, there is lots of options :-)
  call from_param ("type" ,type , (/ 0 , 0 , 0 /) )
  call from_param ("inv"  ,inv  , (/ 0 , 0 , 0 /) )
  call from_param ("sigma",sigma, (/.1 ,.1 ,.1 /) )

  ! Optionally get the average
  call from_param ("collect",collect_l,.false.)

  ! Taper tie points
  call from_param ("t1",t1,(/0.,0.,0./)               )        ! taper in 
  call from_param ("t2",t2,(/0.,0.,0./) + d*real(n-2) )
  call from_param ("t3",t3,(/0.,0.,0./) + d*real(n-2) )
  call from_param ("t4",t4,(/0.,0.,0./) + d*real(n-1) )

  ! FFTW dettails
  call from_param("shift"  , shift ,.true. )
  call from_param("nfft"   , nfft  , (/n(1),n(2),n(3)/) )     ! Determine over what length to calculate the fft (can best be powers of two), does symmetric zero padding
  call from_param("true_t" , true_t,   .false.     )     ! Puts data at true indices in data-array, otherwise, it will assume all data has irrelevant absolute t,x,y coordinates
  call from_param("true_x" , true_x,   .false.     )
  call from_param("unit"   , unit  , (/ 1 , 1 , 1 /) )     ! Kind of units to put on the axis: 1:frequency, 2:radial frequency    (also for spatial)
  call from_param("symt"   , symt  , .false.   )         ! To be implemented (to get t-symmetric output [-nt:nt])
  
  ! Spectrum type (1: ampl, 2: powr)
  call from_param ("stype"  ,stype, 1     )              ! Spectrum type: 1: ampl, 2: powr
  call from_param ("verbose",verb ,.false.)

  ! Convert intereger to booleans
  do ii=1,3
    if (inv(ii).eq.1) then; inv_3(ii)=.true.; else; inv_3(ii)=.false.; end if
  end do
  
  ! Determine b(ginning) and e(nd) indices for data_in
  ! Check if nfft is long enough for true_t and true_x
  if (true_t) then
    call taper_indice(o(1)              ,b(1),d_in=d(1),o_in=0.)
    call taper_indice(o(1)+(n(1)-1)*d(1),e(1),d_in=d(1),o_in=0.)

    nt  (1) =  2*max( abs(e(1)),abs(b(1)),abs(b(1)-2),abs(e(1)-2) )
    nfft(1) = max(nt(1),nfft(1))   ! make sure nfft is large enough

    ! Determine final indices for data_in
    call taper_indice(o(1)              ,b(1),d_in=d(1),o_in=-nt(1)/2*d(1))
    call taper_indice(o(1)+(n(1)-1)*d(1),e(1),d_in=d(1),o_in=-nt(1)/2*d(1))

  else
     b(1) =   1
     e(1) = n(1)
    nt(1) = n(1)
    nfft(1) = max(nt(1),nfft(1))   ! make sure nfft is large enough
  end if
  
  if (true_x) then
    call taper_indice(o(2)              ,b(2),d_in=d(2),o_in=0.)
    call taper_indice(o(2)+(n(2)-1)*d(2),e(2),d_in=d(2),o_in=0.)

    nt  (2) =  2*max( abs(e(2)),abs(b(2)),abs(b(2)-2),abs(e(2)-2) )
    nfft(2) = max(nt(2),nfft(2))   ! make sure nfft is large enough

    ! Determine final indices for data_in
    call taper_indice(o(2)              ,b(2),d_in=d(2),o_in=-nt(2)/2*d(2))
    call taper_indice(o(2)+(n(2)-1)*d(2),e(2),d_in=d(2),o_in=-nt(2)/2*d(2))

  else
    !write(0,*) "Hellowa"
     b(2) =   1
     e(2) = n(2)
    nt(2) = n(2)
    nfft(2) = max(nt(2),nfft(2))   ! make sure nfft is large enough
  end if

  if (true_y) then
    call taper_indice(o(3)              ,b(3),d_in=d(3),o_in=0.)
    call taper_indice(o(3)+(n(3)-1)*d(3),e(3),d_in=d(3),o_in=0.)

    nt  (3) =  2*max( abs(e(3)),abs(b(3)),abs(b(3)-2),abs(e(3)-2) )
    nfft(3) = max(nt(3),nfft(3))   ! make sure nfft is large enough

    ! Determine final indices for data_in
    call taper_indice(o(3)              ,b(3),d_in=d(3),o_in=-nt(3)/2*d(3))
    call taper_indice(o(3)+(n(3)-1)*d(3),e(3),d_in=d(3),o_in=-nt(3)/2*d(3))

  else
    !write(0,*) "Hellowa"
     b(3) =   1
     e(3) = n(3)
    nt(3) = n(3)
    nfft(3) = max(nt(3),nfft(3))   ! make sure nfft is large enough
  end if
  
  !write(0,*) "=================="   
  !write(0,*) "b",b
  !write(0,*) "e",e
  !write(0,*) "n",n
  !write(0,*) "nt",nt
  !write(0,*) "nfft",nfft
  !write(0,*) "=================="   
  
  ! Padd logic
  if (nfft(1).gt.nt(1) .or. nfft(2).gt.nt(2).or. nfft(3).gt.nt(3)) then; padd=.true.; else; padd=.false.; end if;
  
  ! Calculate Fourier units
  call fft_r2c_units_t(.false.,nfft(1),d(1),nw=nw(1),df=dw(1))
  call fft_c2c_units_x(.false.,nfft(2),d(2),nk=nw(2),dk=dw(2))
  call fft_c2c_units_x(.false.,nfft(3),d(3),nk=nw(3),dk=dw(3))
  
  do ii=1,3
    if (unit(ii).eq.2) then; dw(ii) = 2.*pi*dw(ii); end if
  end do
  
  ! Write to output history
  call to_history("n1", nw(1) )
  call to_history("n2", nw(2) )
  call to_history("n3", nw(3) )
  call to_history("d1", dw(1) )
  call to_history("d2", dw(2) )
  call to_history("d3", dw(3) )
  call to_history("o1",   0.  )
  call to_history("o2", -nw(2)/2*dw(2) )
  call to_history("o3", -nw(3)/2*dw(3) )
  
  if (collect_l) then
    call to_history("n4", 1 )
    call to_history("n5", 1 )
  end if

  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
    


  ! To taper or not to taper
  if (type(1).gt.1 .or. type(2).gt.1 .or. type(3).gt.1) then; taper = .true.; else; taper=.false.; end if

  if (taper) then
    ! Make sure that no taper specification gives a 1 in that direction
    do ii=1,3;
      if (type(ii)==0) then; inv_3(ii)=.true.; type(ii)=1; end if
    end do
    
    ! Determine tie indices
    do ii=1,3
      call taper_indice(t1(ii),t1_ind(ii),d_in=d(ii),o_in=0.)
      call taper_indice(t2(ii),t2_ind(ii),d_in=d(ii),o_in=0.)
      call taper_indice(t3(ii),t3_ind(ii),d_in=d(ii),o_in=0.)
      call taper_indice(t4(ii),t4_ind(ii),d_in=d(ii),o_in=0.)
    end do

    ! Initialize tape functions
    call tape_init       (n(1),n(2),n(3))

    ! Initialize taper functions in tapers_mod
    call tapers_init     (n(1),n(2),n(3))
    call tapers_interface(3,dir_in=(/1,2,3/),t1_in=t1_ind,t2_in=t2_ind,t3_in=t3_ind,t4_in=t4_ind,type_in=type,sigma_in=sigma,scale_in=(/ 1., 1., 1./),inv_in=inv_3)
    call tapers_insert_tape()
    call tapers_close      ()
  end if

  ! Initialize Padd routine
  call padd_mod_init(center_in=.true.,p1_in=nfft(1),p2_in=nfft(2),p3_in=nfft(3))

  ! Initialize FFTW routine
  call fftw3d_r2c_lop_init(n1_in=nfft(1),n2_in=nfft(2),n3_in=nfft(3))
  
  ! Initialize collector module
  if (collect_l) then;  call collect_init(nw(1),nw(2),nw(3));  end if

  ! Allocate the arrays datain and dataout
  allocate( data_in(nt(1), nt(2), nt(3)) , data_pd(nfft(1),nfft(2),nfft(3)), data_sf(nfft(1),nfft(2),nfft(3)) , data_ft(nw(1),nw(2),nw(3)) , data_sp(nw(1),nw(2),nw(3)) )
  


  ! Loup over 3rd, 4th and 5th axis
  do mm=1,n5; do ll=1,n4;
    
    ! Verbose which Cube
    write(0,*) "Cube",(mm-1)*n5+ll,"of",n4*n5
    
    ! Read Data
    data_in = 0.
    call sreed( "in" , data_in(b(1):e(1),b(2):e(2),b(3):e(3)), 4*n(1)*n(2)*n(3) )

    ! Taper data
    if (taper) then;
      if (verb) then; write(0,*) "Tapered"; end if
      call tape_op( data_in(b(1):e(1),b(2):e(2),b(3):e(3)) )
    end if

    ! Compute (Padd) (Shift) and FFT
    if (padd) then
      if (verb) then; write(0,*) "Padded"; end if
      call padd_lop(.false.,.false.,data_in,data_pd)
      
      if (shift) then
        if (verb) then; write(0,*) "Shifted"; end if
        ! Compute (Shift) and FFT
        call fftshift2d_23_lop(.false.,.false.,data_pd,data_sf)
        call fftw3d_r2c_lop   (.false.,.false.,data_sf,data_ft)
      else
        if (verb) then; write(0,*) "Not Shifted"; end if
        ! Compute Shift, FFT and 
        call fftw3d_r2c_lop(.false.,.false.,data_pd,data_ft)
      end if
      
    else;
      
      if (shift) then
        if (verb) then; write(0,*) "Shifted1"; end if
        call fftshift3d_lop(.false.,.false.,data_in,data_sf)
        if (verb) then; write(0,*) "Shifted2"; end if
        call fftw3d_r2c_lop(.false.,.false.,data_sf,data_ft)
      else
        if (verb) then; write(0,*) "Not Shifted"; end if
        call fftw3d_r2c_lop(.false.,.false.,data_in,data_ft)
      end if
      
    end if
    
    ! Compute Spectrum
    if (stype.eq.1) then
      if (verb) then; write(0,*) "Type 1 Spectrum"; end if
      call spectra_ampl    (                data_ft,data_sp)
    else
      if (verb) then; write(0,*) "Type 2 Spectrum"; end if
      call spectra_powr    (                data_ft,data_sp)
    end if

    ! Compute (Inverse Shift) and write out
    if (shift) then
      if (verb) then; write(0,*) "Inverse Shifted"; end if
      ! Compute Shift, FFT and
      call fftshift2d_23_lop(.true.,.false.,data_sf(1:nw(1),:,:),data_sp)
      if (collect_l) then
        call collect(data_sf(1:nw(1),:,:))
      else
        ! Write Data
        call srite( "out", data_sf(1:nw(1),:,:), 4*nw(1)*nw(2)*nw(3) )
      end if
    else
      if (verb) then; write(0,*) "Not Inverse Shifted"; end if
      if (collect_l) then
        call collect(data_sp)
      else
        ! Write Data
        call srite( "out", data_sp, 4*size(data_sp))
      end if
      call srite( "out", data_sp, 4*size(data_sp))
    end if
    
  end do;     end do;
  
  if (collect_l) then
    call get_collected_average(data_sp,n4*n5)
    call srite( "out", data_sp, 4*size(data_sp));
  end if
  
  ! Deallocate and close stuff
  deallocate( data_in , data_ft , data_sp )
  if (taper    ) then;  call tape_close();      end if
  if (collect_l) then;  call collect_close( );  end if
  if (shift    ) then;  deallocate (data_sf);   end if
  call fftw3d_r2c_lop_close()
  
  ! Force clean exit
  call exit (0)
  
end program
