module linrad_mod

  use sep
  implicit none
  integer         , private         :: nx, np, nt
  real            , private         :: ox, dx, op, dp, ot, dt
  real, dimension(:,:), allocatable  :: modl2, data2

contains

  integer function linrad_init(nx_in,ox_in,dx_in,np_in,op_in,dp_in, nt_in, ot_in, dt_in) result(stat)
    integer              :: nx_in, np_in, nt_in
    real                 :: ox_in, dx_in, op_in, dp_in, ot_in, dt_in
    stat=1
    nx = nx_in
    np = np_in
    ox = ox_in
    op = op_in
    dx = dx_in
    dp = dp_in
    nt = nt_in
    ot = ot_in
    dt = dt_in
    allocate(modl2(nt,nx), data2(nt,np))

    stat=0

    end function

    integer function linrad_clos()

      deallocate(modl2, data2)

    end function linrad_clos


!!--------------------------------------------------------------------------
!! forward: (t,x) to (tau,p) and backward (tau,p) to (t,x)
!! Thanks Gboyega for the code
!! data: 1D array in tau-p domain indd=itau + (ip-1)*nt
!! modl: 1D array in t-x domain; indm=it + (ix-1)*nt
!! adj: (0) from modl to data   (1) from data to modl 

 integer function linrad_oper(adj, add, modl, data) result(stat)

   logical                  :: adj, add
   real, dimension(:)     :: modl, data
   integer                 :: counter, indm, indd
   integer                 :: it, ix, ip, itau
   integer                 :: imin, imax, ishift
   real                    :: p, tau, x, oox
   real                    :: shift, samp_shift, del


 !  write(0,*) '**************************************'
 !  write(0,*) 'in linrad_oper'
    
    stat = 1

    if (.not. add) then
       if (adj) then
          modl = 0.
       else
          data = 0.
       end if
    end if


  if (adj) then
     modl=0.
  else
     data=0.
  endif

  ! convert 1D array to 2D array
  do it=1,nt

     do ip=1,np
        indd= it+(ip-1)*nt

        data2(it,ip)=data(indd)

     enddo

     do ix=1,nx
        indm= it+(ix-1)*nt

        modl2(it,ix) = modl(indm)

     enddo

  enddo





  ! loop over offsets
    do ix=1,nx 

   !    oox = ox+(nx-1)*dx/2
       x= (ox + (ix-1)*dx)

       do ip=1,np
          p = op+(ip-1)*dp

          shift= -p*x
          samp_shift= shift/dt

          ishift=floor(samp_shift)
          imin=max( 1, -ishift+1)
          imax=min(nt-1, nt-ishift)
          del=samp_shift-ishift
          itau=imin+ishift

  
             
          if(imin>nt) then 
!              write(0,*) 'case when imin > nt'
	      counter=counter+1
              cycle
          end if

          do it=imin,imax;

             ! Transforming from cartesian to linear indexes
             indm = it + (ix-1)*nt
             indd = itau + (ip-1)*np

             if (adj) then     ! adjoint --> with linear interp

!                modl(indm) =  modl(indm) + data(indd)*(1.-del) &
!                     + data(indd+1)*del  

                modl2(it,ix) =  modl2(it,ix) + data2(itau,ip)*(1.-del) &
                     + data2(itau+1,ip)*del     
             else               ! Forward -> linear interpT

!                data(indd) =  data(indd) + modl(indm)*(1.-del);
!                data(indd+1) =  data(indd+1) + modl(indm)*del; 

                data2(itau,ip) =  data2(itau,ip) + modl2(it,ix)*(1.-del);
                data2(itau+1,ip) =  data2(itau+1,ip) + modl2(it,ix)*del; 
             endif

             itau=itau+1
          
          enddo

	
	
       enddo
    enddo


    ! convert 2D array back to 1D array
    do it=1,nt
       
       do ip=1,np
          indd= it+(ip-1)*nt
          
          data(indd) = data2(it,ip)
          
       enddo
       
       do ix=1,nx
          indm= it+(ix-1)*nt
          
           modl(indm) = modl2(it,ix)
          
       enddo
       
    enddo


  !  write(0,*) counter, 'out of', np*nx, 'excluded'

    stat=0

  end function

!------------------------------------------------------------
! Mandy's version of tau-p transform


  integer function linrad2_oper(adj, add, modl, data) result(stat)

   logical                  :: adj, add
   real, dimension(:)     :: modl, data
   integer                 :: counter
   integer                 :: it, ix, ip, itau
   integer                 :: imin, imax, ishift
   integer                 :: indd, indm
   real                    :: p, tau, x, oox
   real                    :: tloc, del, t

    


  write(0,*) '**************************************'
  write(0,*) 'in Mandy linrad_oper'

  if (adj) then
     modl=0.
  else
     data=0.
  endif

  ! loop over offsets
    do ix=1,nx 

!       oox = ox+(nx-1)*dx/2
       x= (ox + (ix-1)*dx)

       do ip=1,np
          p = op+(ip-1)*dp  

             

          do itau=1,nt

	     tau= ot + (itau-1)*dt
             t = tau+p*x
	     tloc = (t-ot)/dt+1.0
	     it = floor(tloc);
	     del = tloc-it


	     if ((it .lt. 1) .or. (it .gt. nt-1) ) then
	         counter=counter+1
                 cycle
             end if

             ! Transforming from cartesian to linear indexes
             indm = it + (ix-1)*nt
             indd = itau + (ip-1)*np

             if (adj) then    ! adjoint --> with linear interp

                modl(indm) = modl(indm) + data(indd)*(1.-del)
                modl(indm+1) = modl(indm+1) + data(indd)*del


!                modl(it,ix) = modl(it,ix) + data(itau,ip)*(1.-del)
!                modl(it+1,ix) = modl(it+1,ix) + data(itau,ip)*del
              
             else    ! Forward -> linear interpT

                 data(indd) =  data(indd) + modl(indm)*(1.-del) + modl(indm+1)*del
               
!                data(itau,ip) =  data(itau,ip) + modl(it,ix)*(1.-del) + modl(it+1,ix)*del
                
                
         
             endif
          
          enddo

	
	
       enddo
    enddo

	write(0,*) counter, 'out of', np*nx*nt, 'excluded'

  end function


end module linrad_mod
