program sdrSpectraTP2d
  
  use sep
  use tp_trans_mod
  use sep_timers_mod
  use collect_mod

  implicit none

  integer           :: nt,nx,n3,n4,n5
  real              :: dt,dx,ot,ox
  
  integer           :: ntau,np
  real              :: dtau,otau,op,dp,pmin,pmax
  
  real,allocatable  :: datain (:,:)
  real,allocatable  :: dataout(:,:)
  
  integer           :: ii,jj,kk,ll,mm
  
  logical           :: true_x,collect_l,verb


  ! Spectrum type
  integer                         :: stype              ! 1: ampl    2: powr

  ! For timers
  logical           :: logic
  integer           :: tm
  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  call sep_init()

  call from_param("true_x",true_x,.false.)
  call from_param ("verbose",verb,.false.)
  call from_param("stype",stype,1)
  call from_param ("collect",collect_l,.false.)

  call from_history("n3",n3,1)
  call from_history("n4",n4,1)
  call from_history("n5",n5,1)



    ! Write (t,x) axis to History
    call from_history("n1",nt,1 )
    call from_history("d1",dt,1.)
    call from_history("o1",ot,0.)
    
    call from_history("n2",nx,1 )
    call from_history("d2",dx,1.)
    call from_history("o2",ox,0.)
    if (.not.true_x) then; ox = -nx/2*dx; end if
    
    ! Read (tau,p) axis from Parameter
    call from_param  ("ntau",ntau, nt)
    call from_param  ("dtau",dtau, dt)
    call from_param  ("otau",otau, ot)
    
    call from_param  ("pmin",pmin, -1./500.)
    call from_param  ("pmax",pmax,  1./500.)
    call from_param  (  "np",np  , nx  )
    call from_param  (  "dp",dp  , (pmax-pmin)/(np-1) )
    call from_param  (  "op",op  , pmin)
    
    ! Write (tau,p) axis to History    
    call to_history  ("n1",ntau)
    call to_history  ("d1",dtau)
    call to_history  ("o1",otau)
    
    call to_history  ("n2",  np)
    call to_history  ("d2",  dp)
    call to_history  ("o2",  op)

    call to_history("label1",'\F10 t')   ! '\F10 t\F-1  [s]'
    call to_history("label2",'p'     )


  if (collect_l) then
    call to_history("n4", 1 )
    call to_history("n5", 1 )
  end if
  
  call sep_close()
  
  
  ! Allocations and Initializations
  allocate( datain(nt,nx) , dataout(ntau,np) )
  call tp_trans_2d_init(  nt_in=nt  ,  dt_in=dt  ,  ot_in=ot  ,nx_in=nx,dx_in=dx,ox_in=ox, &
                        ntau_in=ntau,dtau_in=dtau,otau_in=otau,np_in=np,dp_in=dp,op_in=op)
  
  ! Initialize collector module
  if (collect_l) then;  call collect_init(ntau,np);  end if  

  ! Kernel
  do mm=1,n5;  do ll=1,n4;  do kk=1,n3
    write(0,*) "Plane",(mm-1)*n5*n4+(ll-1)*n4+kk,"of",n4*n5*n3
    
    call sreed("in" , datain , 4*size(datain ))
    
    call tp_trans_2d_fop(.false.,.false.,datain,dataout)
    
    ! Compute Spectrum
    if (stype.eq.1) then
      if (verb) then; write(0,*) "Type 1 Spectrum"; end if
      if (collect_l) then
        call collect(abs(dataout) )
      else
        ! Write Data
        call srite( "out", abs(dataout), 4*size(dataout))
      end if
    end if

    ! Compute Spectrum
    if (stype.eq.2) then
      if (verb) then; write(0,*) "Type 2 Spectrum"; end if
      if (collect_l) then
        call collect(dataout**2.)
      else
        ! Write Data
        call srite( "out", dataout**2. , 4*size(dataout))
      end if
    end if
    
  end do;      end do;      end do
  
  
  ! Deallocations and closures  
  deallocate(datain,dataout)
  call tp_trans_2d_close()
  call collect_close()
  call stop_timer_num (tm)
  call print_timers()

  call exit(0)

end program
