program sdrTPtrans3d
  
  use sep
  use tp_trans_mod
  use sep_timers_mod

  implicit none

  integer           :: nt,nx,ny,n4,n5
  real              :: dt,dx,dy,ot,ox,oy
  
  integer           :: ntau,npx,npy
  real              :: dtau,otau,opx,opy,dpx,dpy,pxmin,pxmax,pymin,pymax
  
  real,allocatable  :: datain (:,:,:)
  real,allocatable  :: dataout(:,:,:)
  
  integer           :: ii,jj,kk,ll,mm
  
  logical           :: inv,true_x,true_y

  ! For timers
  logical           :: logic
  integer           :: tm
  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  call sep_init()

  call from_param("inv"   ,inv   ,.false.)
  call from_param("true_x",true_x,.false.)
  call from_param("true_y",true_y,.false.)

  call from_history("n4",n4,1)
  call from_history("n5",n5,1)

  if (inv) then

    ! Read (tau,p) axis from History
    call from_history("n1",ntau, 1 )
    call from_history("d1",dtau, 1.)
    call from_history("o1",otau, 0.)

    call from_history("n2",npx , 1 )
    call from_history("d2",dpx , 1.)
    call from_history("o2",opx , 0.)
    
    call from_history("n3",npy , 1 )
    call from_history("d3",dpy , 1.)
    call from_history("o3",opy , 0.)

    ! Write (t,x) axis to Parameter
    call from_param  ("nt",nt  ,-1      )
    call from_param  ("dt",dt  ,-1.     )
    call from_param  ("ot",ot  ,-11.1111)

    call from_param  ("nx",nx  ,-1      )
    call from_param  ("dx",dx  ,-1.     )
    call from_param  ("ox",ox  ,-11.1111)

    call from_param  ("ny",ny  ,-1      )
    call from_param  ("dy",dy  ,-1.     )
    call from_param  ("oy",oy  ,-11.1111)
    
    ! Backup (t,x) from History
    if (nt.eq.-1      ) then; call from_history("nt",nt,ntau); end if
    if (dt.eq.-1.     ) then; call from_history("dt",dt,dtau); end if
    if (ot.eq.-11.1111) then; call from_history("ot",ot,otau); end if
    
    if (nx.eq.-1      ) then; call from_history("nx",nx, npx); end if
    if (nx.eq.-1      ) then; call from_history("dx",dx,  1.); end if
    if (ox.eq.-11.1111) then; call from_history("ox",ox,  0.); end if
    
    if (ny.eq.-1      ) then; call from_history("ny",ny, npy); end if
    if (ny.eq.-1      ) then; call from_history("dy",dy,  1.); end if
    if (oy.eq.-11.1111) then; call from_history("oy",oy,  0.); end if

    ! Write (t,x) axis to History
    call to_history  ("n1",nt)
    call to_history  ("d1",dt)
    call to_history  ("o1",ot)
    
    call to_history  ("n2",nx)
    call to_history  ("d2",dx)
    call to_history  ("o2",ox)  

    call to_history  ("n3",ny)
    call to_history  ("d3",dy)
    call to_history  ("o3",oy)  

    call to_history("label1",'t')
    call to_history("label2",'x')
    call to_history("label3",'y')

  else

    ! Write (t,x) axis to History
    call from_history("n1",nt,1 )
    call from_history("d1",dt,1.)
    call from_history("o1",ot,0.)
    
    call from_history("n2",nx,1 )
    call from_history("d2",dx,1.)
    call from_history("o2",ox,0.)

    call from_history("n3",ny,1 )
    call from_history("d3",dy,1.)
    call from_history("o3",oy,0.)

    if (.not.true_x) then; ox = -nx/2*dx; end if
    if (.not.true_y) then; oy = -ny/2*dy; end if
    
    ! Read (tau,p) axis from Parameter
    call from_param  ("ntau",ntau, nt)
    call from_param  ("dtau",dtau, dt)
    call from_param  ("otau",otau, ot)
    
    call from_param  ("pxmin",pxmin,-1./500.)
    call from_param  ("pxmax",pxmax, 1./500.)
    call from_param  ("pymin",pymin,-1./500.)
    call from_param  ("pymax",pymax, 1./500.)

    call from_param  ( "npx",npx , nx  )
    call from_param  ( "dpx",dpx , (pxmax-pxmin)/(npx-1) )
    call from_param  ( "opx",opx , pxmin)

    call from_param  ( "npy",npy , ny  )
    call from_param  ( "dpy",dpy , (pymax-pymin)/(npy-1) )
    call from_param  ( "opy",opy , pymin)
    
    ! Write (tau,p) axis to History    
    call to_history  ("n1",ntau)
    call to_history  ("d1",dtau)
    call to_history  ("o1",otau)
    
    call to_history  ("n2", npx)
    call to_history  ("d2", dpx)
    call to_history  ("o2", opx)

    call to_history  ("n3", npy)
    call to_history  ("d3", dpy)
    call to_history  ("o3", opy)

    call to_history("label1",'\F10 t')   ! '\F10 t\F-1  [s]'
    call to_history("label2",'px'    )
    call to_history("label3",'py'    )

  end if
  
  call sep_close()
  
  
  ! Allocations and Initializations
  allocate( datain(nt,nx,ny) , dataout(ntau,npx,npy) )
  call tp_trans_3d_init(  nt_in=nt  ,  dt_in=dt  ,  ot_in=ot  , nx_in=nx , dx_in=dx , ox_in=ox , ny_in=ny , dy_in=dy , oy_in=oy ,&
                        ntau_in=ntau,dtau_in=dtau,otau_in=otau,npx_in=npx,dpx_in=dpx,opx_in=opx,npy_in=npy,dpy_in=dpy,opy_in=opy)
 
  
  ! Kernel
  do mm=1,n5;  do ll=1,n4;
    write(0,*) "Cube",(mm-1)*n5+ll,"of",n4*n5
    
    if (inv) then
      call sreed("in" , dataout, 4*size(dataout))
    else
      call sreed("in" , datain , 4*size(datain ))
    end if
    
    call tp_trans_3d_fop(inv,.false.,datain,dataout)
    
    if (inv) then
      call srite("out", datain , 4*size(datain ))
    else
      call srite("out", dataout, 4*size(dataout))
    end if
    
  end do;      end do;
  
  
  ! Deallocations and closures  
  deallocate(datain,dataout)
  call tp_trans_3d_close()
  call stop_timer_num (tm)
  call print_timers()

  call exit(0)

end program
