program SRrotateMH2d
  ! A Quick and dirty program to rotate a regular source receiver grid, to a midpoint offset grid
  ! In future, extend to 3d ...

  ! Rotates a regular seismic dataset: n1,ng,ns to n1,nm,nh
  ! I think it needs ng=ns , optional:
  ! Stack over third axis after rotation
  ! Weight in the Stack, and also in the sparseness stack
  ! 'sparse' option: To create a rotated array of same size, binning the midpoints and offsets.
  ! adj option: SR-->MH or MH --> SR

  use sep

  implicit none

  real,allocatable           :: data_A(:,:,:),data_B(:,:,:),wght_A(:,:),wght_B(:,:)
  integer                    :: n1,ng,ns,nm,nh
  real                       :: d1,dg,ds,dm,dh
  real                       :: o1,og,os,om,oh
  integer                    :: is,ig,im,ih
  logical                    :: adj,wght,sprs,verbose,interp,adj_interp_sparse,logical
  integer                    :: stck
  integer                    :: n4,n5,n6,n7,ll,mm,nn,oo
  
  

  call sep_init()
  
  call from_param  ( "adj"  , adj,.false.)
  call from_param  ("stack" ,stck, 0     )
  call from_param  ("weight",wght, .true.)
  call from_param  ("sparse",sprs, .true.)
  call from_param  ("verbose",verbose,.false.)
  call from_param  ("interp",interp,.false.)
  call from_param  ("adj_interp_sparse",adj_interp_sparse,.true.)
  call from_param  ("adj_interp_sparse_subs",adj_interp_sparse,.false.)

  call from_history( "n1" ,n1 ,1 )
  call from_history( "d1" ,d1 ,1.)
  call from_history( "o1" ,o1 ,0.)

  call from_history( "n4" ,n4 ,1 )
  call from_history( "n5" ,n5 ,1 )
  call from_history( "n6" ,n6 ,1 )
  call from_history( "n7" ,n7 ,1 )

  
  if (adj) then
    call from_history( "n2" ,nm ,1 )
    call from_history( "n3" ,nh ,1 )
    
    call from_history( "d2" ,dm ,1.)
    call from_history( "d3" ,dh ,1.)
    
    call from_history( "o2" ,om ,0.)
    call from_history( "o3" ,oh ,0.)
    
    ns = 2*nm; ds = dm; os = om
    ng = 2*nm; dg = dm; og = om

    if (sprs) then
      if      (stck.eq.3) then
        call to_history  ( "n2", nm/2 )
        call to_history  ( "n3",  1 )
      else if (stck.eq.2) then
        call to_history  ( "n2", nh/2 )
        call to_history  ( "n3",  1 )
      else
        call to_history  ( "n2", nm/2 )
        call to_history  ( "n3", nh/2 )
      end if
      
      call to_history  ( "d2", dg )
      call to_history  ( "d3", ds )
      
      call to_history  ( "o2", og )
      call to_history  ( "o3", os )
    else
      
      if      (stck.eq.3) then
        call to_history  ( "n2", nm*2 )
        call to_history  ( "n3",  1 )
        call to_history  ( "d2", dg )
        call to_history  ( "d3", ds )
      else if (stck.eq.2) then
        call to_history  ( "n2", nh*2 )
        call to_history  ( "n3",  1 )
        call to_history  ( "d2", dg )
        call to_history  ( "d3", ds )
      else

        if (adj_interp_sparse) then
          call to_history  ( "n2", nm   )
          call to_history  ( "n3", nh   )      
          call to_history  ( "d2", dg*2 )
          call to_history  ( "d3", ds*2 )
        else
          call to_history  ( "n2", nm*2 )
          call to_history  ( "n3", nh*2 )      
          call to_history  ( "d2", dg )
          call to_history  ( "d3", ds )
        end if

      end if 
      
    
      call to_history  ( "o2", og )
      call to_history  ( "o3", os )
    end if
    
    call to_history  ("label2","geophone")
    call to_history  ("label3", "source" )
  else
    call from_history( "n2" ,ng ,1 )
    call from_history( "n3" ,ns ,1 )
    
    call from_history( "d2" ,dg ,1.)
    call from_history( "d3" ,ds ,1.)
    
    call from_history( "o2" ,og ,0.)
    call from_history( "o3" ,os ,0.)
    
    nm = 2*ns; om = os
    nh = 2*ns;
    if (sprs) then
      dm = dg
      dh = dg
      oh = -real(ns/2-1)*dg
    else
      dh = dg/2.
      dm = dg/2.
      oh = -real(nh/2-1)*dh
    end if    

    if (sprs) then
      if      (stck.eq.3) then
        call to_history  ( "n2", nm/2 )
        call to_history  ( "n3",  1 )
      else if (stck.eq.2) then
        call to_history  ( "n2", nh/2 )
        call to_history  ( "n3",  1 )
      else
        call to_history  ( "n2", nm/2 )
        call to_history  ( "n3", nh/2 )    
      end if
      
      call to_history  ( "d2", dm )
      call to_history  ( "d3", dh )
    
      call to_history  ( "o2", om )
      call to_history  ( "o3", oh )
    else
      if      (stck.eq.3) then
        call to_history  ( "n2", nm )
        call to_history  ( "n3",  1 )
      else if (stck.eq.2) then
        call to_history  ( "n2", nh )
        call to_history  ( "n3",  1 )
      else
        call to_history  ( "n2", nm )
        call to_history  ( "n3", nh )    
      end if    
    
      call to_history  ( "d2", dm )
      call to_history  ( "d3", dh )
    
      call to_history  ( "o2", om )
      call to_history  ( "o3", oh )
    end if
    
    call to_history  ("label2","midpoint")
    call to_history  ("label3", "offset" )
  end if
  
  call sep_close()
  

!  write(0,*) "======== Parameters & Options ========"
!  write(0,*) "adj: ",adj
!  write(0,*) "stck: ",stck
!  write(0,*) "wght: ",wght
!  write(0,*) "sprs: ",sprs
!  write(0,*) "verbose: ",verbose
!  write(0,*) "interp: ",interp

!  write(0,*) "ns: ",ns
!  write(0,*) "ng: ",ng
!  write(0,*) "nh: ",nh
!  write(0,*) "nm: ",nm
!  write(0,*) "ds: ",ds
!  write(0,*) "dg: ",dg
!  write(0,*) "dm: ",dm
!  write(0,*) "dh: ",dh
!  write(0,*) "os: ",os
!  write(0,*) "og: ",og
!  write(0,*) "om: ",om
!  write(0,*) "oh: ",oh
!  write(0,*) "========= ========= ========= ========"


  
  ! Allocate memory
  allocate  (data_A(n1,ng,ns),data_B(n1,nm,nh));
  allocate  (wght_A(   ng,ns),wght_B(   nm,nh));

  
  do oo=1,n7; do nn=1,n6; do mm=1,n5; do ll=1,n4


  data_A = 0.
  data_B = 0.
  wght_A = 0.
  wght_B = 0.
  
  ! Read data
  if (adj) then
    call sreed("in",data_B,4*size(data_B))
  else  
    call sreed("in",data_A,4*size(data_A))    
  end if
  
  ! Rotate
  if (adj) then
    do ih=1,nh
    do im=1,nm
      ig = im + ih
      is = im - ih + nm
     ! if (verbose) then; write(0,*) "ig,is,im,ih: ",ig,is,im,ih; end if
      data_A(:,ig,is) = data_A(:,ig,is) + data_B(:,im,ih)
      wght_A(  ig,is) = wght_A(  ig,is) + 1.
    end do
    end do
  else
    do is=1,ns
    do ig=1,ng
      im = ig + is - 1
      ih = ig - is + ng
     ! if (verbose) then; write(0,*) 'ig,is,im,ih:',ig,is,im,ih; end if
      data_B(:,im,ih) = data_B(:,im,ih) + data_A(:,ig,is)
      wght_B(  im,ih) = wght_B(  im,ih) + 1.
    end do
    end do
  end if
  
  
  ! Write out data, based on optional sparsity and weighting
  if (sprs) then
    if (verbose) then; write(0,*) "Sparsed"; end if

    if (adj) then
      if (verbose) then; write(0,*) "Adjoint"; end if
      
      ! To get rid of redundant outer limits of geophone and source positions
      nh = nh/2
      nm = nm/2
      
      ! Subsample and stack
      do ih=1,nh
      do im=1,nm
        data_A(:,im,ih) = data_A(:,nm+2*im-1,nh+2*ih-1) + data_A(:,nm+2*im-1,nh+2*ih  ) + data_A(:,nm+2*im  ,nh+2*ih-1) + data_A(:,nm+2*im  ,nh+2*ih  )
        wght_A(  im,ih) = wght_A(  nm+2*im-1,nh+2*ih-1) + wght_A(  nm+2*im-1,nh+2*ih  ) + wght_A(  nm+2*im  ,nh+2*ih-1) + wght_A(  nm+2*im  ,nh+2*ih  )
      end do
      end do
      
      if (stck.eq.3) then
        if (verbose) then; write(0,*) "Stacked"; end if
        do im=1,nm
          do ih=2,nh
            data_A(:,im,1) = data_A(:,im,1) + data_A(:,im,ih)
          end do
        end do
        wght_A(  1:nm,1) = sum(wght_A(  1:nm,1:nh),2)
        
        if (wght) then
          if (verbose) then; write(0,*) "Weighted"; end if
          do im=1,nm
            if (wght_A(im,1).gt.0.) then
              data_A(:,im,1) = data_A(:,im,1)/wght_A(im,1)
            end if
          end do
        end if
        
        call srite("out",data_A(:,1:nm,1),4*n1*nm       )
        
      else if (stck.eq.2) then
        if (verbose) then; write(0,*) "Stacked"; end if
        do ih=1,nh
          do im=2,nm
            data_A(:,1,ih) = data_A(:,1,ih) + data_A(:,im,ih)
          end do
        end do
        wght_A(  1,1:nh) = sum(wght_A(  1:nm,1:nh),1)
        
        if (wght) then
          if (verbose) then; write(0,*) "Weighted"; end if
          do ih=1,nh
            if (wght_A(im,1).gt.0.) then
              data_A(:,1,ih) = data_A(:,1,ih)/wght_A(1,ih)
            end if
          end do
        end if
        
        call srite("out",data_A(:,1,1:nh),4*n1*nh       )

      else
        if (verbose) then; write(0,*) "Not Stacked"; end if
        
        if (wght) then
        if (verbose) then; write(0,*) "Weighted"; end if
          do im=1,nm
            if (wght_A(im,1).gt.0.) then
              data_A(:,im,1) = data_A(:,im,1)/wght_A(im,1)
            end if
          end do
        end if
        
        call srite("out",data_A(:,1:nm,1:nh),4*size(data_A(:,1:nm,1:nh)))
        
      end if
      
    else
      if (verbose) then; write(0,*) "Not Adjoint"; end if
      
      do is=1,ns
      do ig=1,ng
        data_B(:,ig,is) = data_B(:,2*ig-1,2*is-1) + data_B(:,2*ig  ,2*is-1) + data_B(:,2*ig-1,2*is  ) + data_B(:,2*ig  ,2*is  )
        wght_B(  ig,is) = wght_B(  2*ig-1,2*is-1) + wght_B(  2*ig  ,2*is-1) + wght_B(  2*ig-1,2*is  ) + wght_B(  2*ig  ,2*is  )
      end do
      end do
      
      if (stck.eq.3) then
        if (verbose) then; write(0,*) "Stacked"; end if

        do ig=1,ng
          do is=2,ns
            data_B(:,ig,1) = data_B(:,ig,1) + data_B(:,ig,is)
          end do
        end do
        wght_B(  1:ng,1) = sum(wght_B(  1:ng,1:ns),2)
        
        if (wght) then
          if (verbose) then; write(0,*) "Weighted"; end if
          do ig=1,ng
            if (wght_B(ig,1).gt.0.) then
              data_B(:,ig,1) = data_B(:,ig,1)/wght_B(ig,1)
            end if
          end do
        end if
        
        call srite("out",data_B(:,1:ng,1),4*n1*ng      )
        
      else if (stck.eq.2) then
        if (verbose) then; write(0,*) "Stacked"; end if

        do is=1,ns
          do ig=2,ng
            data_B(:,1,is) = data_B(:,1,is) + data_B(:,ig,is)
          end do
        end do
        wght_B(  1,1:ns) = sum(wght_B(  1:ng,1:ns),1)
        
        if (wght) then
          if (verbose) then; write(0,*) "Weighted"; end if
          do is=1,ns
            if (wght_B(1,is).gt.0.) then
              data_B(:,1,is) = data_B(:,1,is)/wght_B(1,is)
            end if
          end do
        end if
        
        call srite("out",data_B(:,1,1:ns),4*n1*ns      )

      else
        if (verbose) then; write(0,*) "Not Stacked"; end if
        
        if (wght) then
        if (verbose) then; write(0,*) "Weighted"; end if
          do is=1,ns
          do ig=1,ng
            if (wght_B(ig,is).gt.0.) then
              data_B(:,ig,is) = data_B(:,ig,is)/wght_B(ig,is)
            end if
          end do
          end do
        end if
        
        call srite("out",data_B(:,1:ng,1:ns),4*size(data_B(:,1:ng,1:ns)))

      end if

    end if

  else
    if (verbose) then; write(0,*) "Not Sparsed"; end if

    if (adj) then
      if (verbose) then; write(0,*) "Adjoint"; end if

      if (stck.eq.3) then
        if (verbose) then; write(0,*) "Stacked"; end if

        do ig=1,ng
          do is=2,ns
            data_A(:,ig,1) = data_A(:,ig,1) + data_A(:,ig,is)
          end do
        end do
        wght_A(  1:ng,1) = sum(wght_A(  1:ng,1:ns),2)
        
        if (wght) then
          if (verbose) then; write(0,*) "Weighted"; end if
          do ig=1,ng
            if (wght_A(ig,1).gt.0.) then
              data_A(:,ig,1) = data_A(:,ig,1)/wght_A(ig,1)
            end if
          end do
        end if

        call srite("out",data_A(:,:,1),4*n1*ng       )

      else if (stck.eq.2) then
        if (verbose) then; write(0,*) "Stacked"; end if

        do is=1,ns
          do ig=2,ng
            data_A(:,1,ig) = data_A(:,1,ig) + data_A(:,ig,is)
          end do
        end do
        wght_A(  1:ng,1) = sum(wght_A(  1:ng,1:ns),1)

        if (wght) then
          if (verbose) then; write(0,*) "Weighted"; end if
          do is=1,ns
            if (wght_A(1,is).gt.0.) then
              data_A(:,1,is) = data_A(:,1,is)/wght_A(1,is)
            end if
          end do
        end if

        call srite("out",data_A(:,1,:),4*n1*ns       )

      else
        if (verbose) then; write(0,*) "Not Stacked"; end if

        if (interp) then
          if (verbose) then; write(0,*) "Interpolated"; end if

          data_A(:,1,2) = 1./3.*data_A(:,2,2) + 1./3.*data_A(:,1,1) + 1./3.*data_A(:,1,3)
          data_A(:,2,1) = 1./3.*data_A(:,1,1) + 1./3.*data_A(:,3,1) + 1./3.*data_A(:,2,2)
          do ig=4,ng-2,2
            data_A(:,1,ig  ) = 1./3.*data_A(:,2,ig  ) + 1./3.*data_A(:,1,ig-1) + 1./3.*data_A(:,1,ig+1)
            data_A(:,2,ig-1) =   .25*data_A(:,1,ig-1) +   .25*data_A(:,3,ig-1) +   .25*data_A(:,2,ig-2) + .25*data_A(:,2,ig)
          end do
          data_A(:,1,ng  ) =  .5*data_A(:,2,ng  ) +  .5*data_A(:,1,ng-1)
          data_A(:,2,ng-1) = .25*data_A(:,1,ng-1) + .25*data_A(:,1,ng-1) + .25*data_A(:,2,ng-2) + .25*data_A(:,2,ng)

          do is=4,ns-2,2
            data_A(:,is-1,2) =   .25*data_A(:,is-2,2) +   .25*data_A(:,is  ,2) +   .25*data_A(:,is-1,1) + .25*data_A(:,is-1,3)
            data_A(:,is  ,1) = 1./3.*data_A(:,is-1,1) + 1./3.*data_A(:,is+1,1) + 1./3.*data_A(:,is  ,2)
            do ig=4,ng-2,2
              data_A(:,is-1,ig  ) = .25*data_A(:,is-2,ig  ) + .25*data_A(:,is  ,ig  ) + .25*data_A(:,is-1,ig-1) + .25*data_A(:,is-1,ig+1)
              data_A(:,is  ,ig-1) = .25*data_A(:,is-1,ig-1) + .25*data_A(:,is+1,ig-1) + .25*data_A(:,is  ,ig-2) + .25*data_A(:,is  ,ig  )
            end do
            data_A(:,is-1,ng  ) = 1./3.*data_A(:,is-2,ng  ) + 1./3.*data_A(:,is  ,ng  ) + 1./3.*data_A(:,is-1,ng-1)
            data_A(:,is  ,ng-1) =   .25*data_A(:,is-1,ng-1) +   .25*data_A(:,is+1,ng-1) +   .25*data_A(:,is  ,ng-2) + .25*data_A(:,is  ,ng  )
          end do

          data_A(:,ns-1,2) = .25*data_A(:,ns-2,2) + .25*data_A(:,ns,2) + .25*data_A(:,ns-1,1) + .25*data_A(:,ns-1,3)
          data_A(:,ns  ,1) =  .5*data_A(:,ns-1,1) +  .5*data_A(:,ns,2)
          do ig=4,ng-2,2
            data_A(:,ns-1,ig  ) = 1./3.*data_A(:,ns-2,ig  ) + 1./3.*data_A(:,ns,ig  ) + 1./3.*data_A(:,ns-1,ig-1)
            data_A(:,ns  ,ig-1) = 1./3.*data_A(:,ns-1,ig-1) + 1./3.*data_A(:,ns,ig-2) + 1./3.*data_A(:,ns  ,ig  )
          end do
          data_A(:,ns-1,ng  ) = 1./3.*data_A(:,ns-2,ng  ) + 1./3.*data_A(:,ns  ,ng  ) + 1./3.*data_A(:,ns-1,ng-1)
          data_A(:,ns  ,ng-1) = 1./3.*data_A(:,ns-1,ng-1) + 1./3.*data_A(:,ns  ,ng-2) + 1./3.*data_A(:,ns  ,ng  )

        end if

        if (adj_interp_sparse) then
          if (verbose) then; write(0,*) "Not Stacked"; end if
          if (verbose) then; write(0,*) "Made adj_interp_sparse"; end if
          call srite("out",data_A(:,ns/2-ns/4+1:ns/2+ns/4,ng/2-ng/4+1:ng/2+ng/4),4*size(data_A(:,ns/2-ns/4+1:ns/2+ns/4,ng/2-ng/4+1:ng/2+ng/4)))    
        else
        if (verbose) then; write(0,*) "Holy Cow"; end if

          call srite("out",data_A       ,4*size(data_A))    
        end if
      end if

    else
      if (verbose) then; write(0,*) "Not Adjoint"; end if
      !write(0,*) data_B
      if (stck.eq.3) then
        if (verbose) then; write(0,*) "Stacked-3"; end if
        
        do im=1,nm
          do ih=2,nh
            data_B(:,im,1) = data_B(:,im,1) + data_B(:,im,ih)
          end do
        end do
        wght_B(  1:nm,1) = sum(wght_B(  1:nm,1:nh),2)
                
        if (wght) then
          if (verbose) then; write(0,*) "Weighted"; end if
          do im=1,nm
            if (wght_B(im,1).gt.0.) then
              data_B(:,im,1) = data_B(:,im,1)/wght_B(im,1)
            end if
          end do
        end if
        
        call srite("out",data_B(:,:,1),4*n1*nm       )
        
      else if (stck.eq.2) then
        if (verbose) then; write(0,*) "Stacked-2"; end if
        
        do im=2,nm
          do ih=1,nh
            data_B(:,1,ih) = data_B(:,1,ih) + data_B(:,im,ih)
          end do
        end do
        wght_B(  1:nm,1) = sum(wght_B(  1:nm,1:nh),1)

        if (wght) then
          if (verbose) then; write(0,*) "Weighted"; end if
          do ih=1,nh
            if (wght_B(1,ih).gt.0.) then
              data_B(:,1,ih) = data_B(:,1,ih)/wght_B(1,ih)
            end if
          end do
        end if
        
        call srite("out",data_B(:,1,:),4*n1*nh       )

      else
        if (verbose) then; write(0,*) "Not Stacked"; end if
        
        if (interp) then
          if (verbose) then; write(0,*) "Interpolated"; end if
          
          data_B(:,1,1) =  .5*data_B(:,2,1) +  .5*data_B(:,1,2)
          data_B(:,2,2) = .25*data_B(:,1,2) + .25*data_B(:,3,2) + .25*data_B(:,2,1) + .25*data_B(:,2,3)
          do im=4,nm-2,2
            data_B(:,im-1,1) = 1./3.*data_B(:,im-2,1) + 1./3.*data_B(:,im  ,1) + 1./3.*data_B(:,im-1,2)
            data_B(:,im  ,2) =   .25*data_B(:,im-1,2) +   .25*data_B(:,im+1,2) + .25*data_B(:,im  ,1) + .25*data_B(:,im  ,3)
          end do
          data_B(:,nm-1,1) = 1./3.*data_B(:,nm-2,1) + 1./3.*data_B(:,nm  ,1) + 1./3.*data_B(:,nm-1,2)
          data_B(:,nm  ,2) = 1./3.*data_B(:,nm-1,2) + 1./3.*data_B(:,nm  ,1) + 1./3.*data_B(:,nm  ,3)

          do ih=4,nh-2,2
            data_B(:,1,ih-1) = 1./3.*data_B(:,2,ih-1) + 1./3.*data_B(:,1,ih-2) + 1./3.*data_B(:,1,ih  )
            data_B(:,2,ih  ) =   .25*data_B(:,1,ih  ) +   .25*data_B(:,3,ih  ) + .25*data_B(:,2,ih-1) + .25*data_B(:,2,ih+1)
            do im=4,nm-2,2
              data_B(:,im-1,ih-1) = .25*data_B(:,im-2,ih-1) + .25*data_B(:,im  ,ih-1) + .25*data_B(:,im-1,ih-2) + .25*data_B(:,im-1,ih  )
              data_B(:,im  ,ih  ) = .25*data_B(:,im-1,ih  ) + .25*data_B(:,im+1,ih  ) + .25*data_B(:,im  ,ih-1) + .25*data_B(:,im  ,ih+1)
            end do
            data_B(:,nm-1,ih-1) =   .25*data_B(:,nm-2,ih-1) +   .25*data_B(:,nm  ,ih-1) +   .25*data_B(:,nm-1,ih-2) + .25*data_B(:,nm-1,ih  )
            data_B(:,nm  ,ih  ) = 1./3.*data_B(:,nm-1,ih  ) + 1./3.*data_B(:,nm  ,ih-1) + 1./3.*data_B(:,nm  ,ih+1)
          end do

          data_B(:,1,nh-1) = 1./3.*data_B(:,2,nh-1) + 1./3.*data_B(:,1,nh-2) + 1./3.*data_B(:,1,nh  )
          data_B(:,2,nh  ) = 1./3.*data_B(:,1,nh  ) + 1./3.*data_B(:,3,nh  ) + 1./3.*data_B(:,2,nh-1)
          do im=4,nm-2,2
            data_B(:,im-1,nh-1) = .25*data_B(:,im-2,nh-1) + .25*data_B(:,im  ,nh-1) + .25*data_B(:,im-1,nh-2) + .25*data_B(:,im-1,nh  )
            data_B(:,im  ,nh  ) =  .5*data_B(:,im-1,nh  ) +  .5*data_B(:,im  ,nh-1)
          end do
          data_B(:,nm-1,nh-1) =   .25*data_B(:,nm-2,nh-1) +   .25*data_B(:,nm  ,nh-1) + .25*data_B(:,nm-1,nh-2) + .25*data_B(:,nm-1,nh  )
          data_B(:,nm  ,nh  ) = 1./3.*data_B(:,nm-1,nh  ) + 1./3.*data_B(:,nm  ,nh-1)
        else
          if (verbose) then; write(0,*) "Not Interpolated"; end if
        end if
        !write(0,*) data_B
        call srite("out",data_B       ,4*size(data_B))
        
      end if
    end if
    
  end if

  end do;  end do;  end do;  end do

  
  ! Deallocate Stuff
  deallocate(data_A,data_B,wght_A,wght_B)

  call exit(0)
  
end program

