program sdrAcousticModel
  ! Program to complete and make consistent an acoustic model
  ! velocity,density,incompressibility
  ! dimensions: z,x,y

  use sep
  implicit none

  ! Grid parameters
  integer                           :: ix,iy,iz
  integer                           :: nx,ny,nz
  real                              :: dx,dy,dz
  real                              :: ox,oy,oz
  integer                           :: vnx,vny,vnz
  real                              :: vdx,vdy,vdz
  real                              :: vox,voy,voz
  integer                           :: rnx,rny,rnz
  real                              :: rdx,rdy,rdz
  real                              :: rox,roy,roz
  integer                           :: knx,kny,knz
  real                              :: kdx,kdy,kdz
  real                              :: kox,koy,koz

  real,allocatable                  :: model(:,:,:,:)
  real,allocatable                  :: vv(:,:,:),rh(:,:,:),kk(:,:,:)
  real                              :: vc,rhoc,kc
  
  
  write(0,*) '-----------------------------'
  write(0,*) 'Construct Acoustic Model file'
  
  call sep_init()
  
  nx=1 ; ny=1 ; nz=1
  dx=1.; dy=1.; dz=1.
  ox=0.; oy=0.; oz=0.
  
  ! Geometry of the experiments
  if (exist_file("v")) then   
    call from_aux("v","n1",vnz,nz)
    call from_aux("v","n2",vnx,nx)
    call from_aux("v","n3",vny,ny)
    call from_aux("v","d1",vdz,dz)
    call from_aux("v","d2",vdx,dx)
    call from_aux("v","d3",vdy,dy)
    call from_aux("v","o1",voz,oz)
    call from_aux("v","o2",vox,ox)
    call from_aux("v","o3",voy,oy)
  else
    write(0,*) 'No velocity file found'
    call from_param("vc",vc,-1.)
    if (vc.eq.-1) write(0,*) 'No velocity constant found'
    vnx=nx; vny=ny; vnz=nz
    vdx=dx; vdy=dy; vdz=dz
    vox=ox; voy=oy; voz=oz
  end if
  if (exist_file("rho")) then
    call from_aux("rho","n1",rnz,nz)
    call from_aux("rho","n2",rnx,nx)
    call from_aux("rho","n3",rny,ny)
    call from_aux("rho","d1",rdz,dz)
    call from_aux("rho","d2",rdx,dx)
    call from_aux("rho","d3",rdy,dy)
    call from_aux("rho","o1",roz,oz)
    call from_aux("rho","o2",rox,ox)
    call from_aux("rho","o3",roy,oy)
  else
    write(0,*) 'No density file found'
    call from_param("rhoc",rhoc,-1.)
    if (vc.eq.-1) write(0,*) 'No density constant found'
    rnx=nx; rny=ny; rnz=nz
    rdx=dx; rdy=dy; rdz=dz
    rox=ox; roy=oy; roz=oz
  end if
  if (exist_file("k")) then
    call from_aux("k","n1",knz,nz)
    call from_aux("k","n2",knx,nx)
    call from_aux("k","n3",kny,ny)
    call from_aux("k","d1",kdz,dz)
    call from_aux("k","d2",kdx,dx)
    call from_aux("k","d3",kdy,dy)
    call from_aux("k","o1",koz,oz)
    call from_aux("k","o2",kox,ox)
    call from_aux("k","o3",koy,oy)
  else
    write(0,*) 'No incompressibility file found'
    call from_param("kc",kc,-1.)
    if (vc.eq.-1) write(0,*) 'No incompressibility constant found'
    knx=nx; kny=ny; knz=nz
    kdx=dx; kdy=dy; kdz=dz
    kox=ox; koy=oy; koz=oz
  end if

  write(0,*) "vnz,rnz,knz",vnz,rnz,knz
  write(0,*) "vnx,rnx,knx",vnx,rnx,knx
  write(0,*) "vny,rny,kny",vny,rny,kny
  
  
  nz = maxval( (/ vnz,rnz,knz /) )
  nx = maxval( (/ vnx,rnx,knx /) )
  ny = maxval( (/ vny,rny,kny /) )
  dz = maxval( (/ vdz,rdz,kdz /) )
  dx = maxval( (/ vdx,rdx,kdx /) )
  dy = maxval( (/ vdy,rdy,kdy /) )
  oz = maxval( (/ voz,roz,koz /) )
  ox = maxval( (/ vox,rox,kox /) )
  oy = maxval( (/ voy,roy,koy /) )
  
  ! Error checks
  if (vnz.ne.1) then;  if (vnz.ne.nz) then;  write(0,*) "vnz.ne.nz"; call exit(0);  end if;  end if
  if (vnx.ne.1) then;  if (vnx.ne.nx) then;  write(0,*) "vnx.ne.nx"; call exit(0);  end if;  end if
  if (vny.ne.1) then;  if (vny.ne.ny) then;  write(0,*) "vny.ne.ny"; call exit(0);  end if;  end if
  if (rnz.ne.1) then;  if (rnz.ne.nz) then;  write(0,*) "rnz.ne.nz"; call exit(0);  end if;  end if
  if (rnx.ne.1) then;  if (rnx.ne.nx) then;  write(0,*) "rnx.ne.nx"; call exit(0);  end if;  end if
  if (rny.ne.1) then;  if (rny.ne.ny) then;  write(0,*) "rny.ne.ny"; call exit(0);  end if;  end if
  if (knz.ne.1) then;  if (knz.ne.nz) then;  write(0,*) "knz.ne.nz"; call exit(0);  end if;  end if
  if (knx.ne.1) then;  if (knx.ne.nx) then;  write(0,*) "knx.ne.nx"; call exit(0);  end if;  end if
  if (kny.ne.1) then;  if (kny.ne.ny) then;  write(0,*) "kny.ne.ny"; call exit(0);  end if;  end if

  write(0,*) '-----------------------------'
  write(0,*) 'Geometry:'
  write(0,*) 'nz,nx,ny',nz,nx,ny
  write(0,*) 'dz,dx,dy',dz,dx,dy
  write(0,*) 'oz,ox,oy',oz,ox,oy

  ! Create output header
  if (nz.eq.1) then  ! 0d   (when only constants are given)
    call to_history("n1",3)
    call to_history("n2",1)
    call to_history("n3",1)
    call to_history("n4",1)
    call to_history("d1",1.)
    call to_history("d2",1.)
    call to_history("d3",1.)
    call to_history("d4",1.)
    call to_history("o1",1.)
    call to_history("o2",0.)
    call to_history("o3",0.)
    call to_history("o4",0.)
    call to_history("label1","v,rho,k")
    call to_history("label2"," ")
    call to_history("label3"," ")
    call to_history("label4"," ")
  else if (nx.eq.1) then  ! 1d   (when only constants are given)
    call to_history("n1",nz)
    call to_history("n2",3)
    call to_history("n3",1)
    call to_history("n4",1)
    call to_history("d1",dz)
    call to_history("d2",1.)
    call to_history("d3",1.)
    call to_history("d4",1.)
    call to_history("o1",oz)
    call to_history("o2",1.)
    call to_history("o3",0.)
    call to_history("o4",0.)
    call to_history("label1","z position")
    call to_history("label2","v,rho,k")
    call to_history("label3"," ")
    call to_history("label4"," ")
  else if (ny.eq.1) then  ! 2d   (when only constants are given)
    call to_history("n1",nz)
    call to_history("n2",nx)
    call to_history("n3",3)
    call to_history("n4",1)
    call to_history("d1",dz)
    call to_history("d2",dx)
    call to_history("d3",1.)
    call to_history("d4",1.)
    call to_history("o1",oz)
    call to_history("o2",ox)
    call to_history("o3",1.)
    call to_history("o4",0.)
    call to_history("label1","z position")
    call to_history("label2","x position")
    call to_history("label3","v,rho,k")
    call to_history("label4"," ")
  else  ! 3d   (when only constants are given)
    call to_history("n1",nz)
    call to_history("n2",nx)
    call to_history("n3",ny)
    call to_history("n4", 3)
    call to_history("d1",dz)
    call to_history("d2",dx)
    call to_history("d3",dy)
    call to_history("d4",1.)
    call to_history("o1",oz)
    call to_history("o2",ox)
    call to_history("o3",oy)
    call to_history("o4",1.)
    call to_history("label1","z position")
    call to_history("label2","x position")
    call to_history("label3","y position")
    call to_history("label4","v,rho,k")
  end if
  
  call sep_close()
  
  
  !!! Medium Parameters
  allocate(vv(nz,nx,ny)); vv = -1
  allocate(rh(nz,nx,ny)); rh = -1
  allocate(kk(nz,nx,ny)); kk = -1
  allocate(model(nz,nx,ny,3)); model = 0.
  
  if (exist_file("v"  )) then;   call sreed("v"  ,vv,4*size(vv));   else if (  vc.ne.-1) then; vv=vc;   end if
  if (exist_file("rho")) then;   call sreed("rho",rh,4*size(rh));   else if (rhoc.ne.-1) then; rh=rhoc; end if
  if (exist_file("k"  )) then;   call sreed("k"  ,kk,4*size(kk));   else if (  kc.ne.-1) then; kk=kc;   end if
  
  
  write(0,*) '-----------------------------'
  if ( maxval(vv).eq.-1 .and. maxval(rh).ne.-1 .and. maxval(kk).ne.-1 ) then
    write(0,*) "Calculate V from rho and k"
    do ix=1,nx
     do iy=1,ny
      do iz=1,nz
        model(iz,ix,iy,1) = 1./sqrt(dble(rh(iz,ix,iy))*dble(kk(iz,ix,iy)))
      end do
     end do
    end do
    model(:,:,:,2) = rh
    model(:,:,:,3) = kk
  else
    model(:,:,:,1) = vv
  end if
  if ( maxval(rh).ne.-1) then
    write(0,*) "Calculate k from v and rho"
    do ix=1,nx
     do iy=1,ny
      do iz=1,nz
        model(iz,ix,iy,3) =  1./(dble(vv(iz,ix,iy)))**2 *1./dble(rh(iz,ix,iy))
      end do
     end do
    end do
    model(:,:,:,1) = vv
    model(:,:,:,2) = rh
  else if ( maxval(kk).ne.-1) then
    write(0,*) "Calculate rho from v and k"
    do ix=1,nx
     do iy=1,ny
      do iz=1,nz
        model(iz,ix,iy,2) = 1./(dble(vv(iz,ix,iy)))**2 *1./dble(kk(iz,ix,iy))
      end do
     end do
    end do
    model(:,:,:,1) = vv
    model(:,:,:,3) = rh
  else
    model(:,:,:,2) = rh
    model(:,:,:,3) = kk
  end if
  
  write(0,*) '-----------------------------'
  write(0,*) "Model output"
  ! Write out model
  call srite("out",model,4*size(model))

  write(0,*) '-----------------------------'
  deallocate(vv,rh,kk,model)
  call exit(0)

end program

