program circlestack
  ! Program to perform a circular stack on a data cube.
  !
  ! Sjoerd de Ridder
  ! Stanford, January 20th, 2009
  use sep
  use regurize1d_splines_mod
  implicit none

  integer                                :: n1, n2, n3, nr,n4,n5,n6,n7
  real                                   :: d2, d3, dr, o2, o3, or,d4,d5,d6,d7,o4,o5,o6,o7
  character*50                           :: label2,label3,label4,label5,label6,label7
  integer                                :: ii, jj, kk
  real                                   :: x2, x3, center2, center3, radius, ir,rf,minv,p
  real,             allocatable          :: data(:,:), fold(:)
  double precision, allocatable          :: stack(:,:)
  integer                                :: interp
  integer                                :: ll,mm,nn,oo
  integer                                :: nin,nout,minl
  real,allocatable                       :: rout(:), rin(:), rdata(:,:), datar(:,:)


  call sep_init()
  


  ! Read from input History
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('n4',n4,1 )
  call from_history('n5',n5,1 )
  call from_history('n6',n6,1 )
  call from_history('n7',n7,1 )
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)
  call from_history('d4',d4,1.)
  call from_history('d5',d5,1.)
  call from_history('d6',d6,1.)
  call from_history('d7',d7,1.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('o4',o4,0.)
  call from_history('o5',o5,0.)
  call from_history('o6',o6,0.)
  call from_history('o7',o7,0.)
  call from_history('label2',label2," ")
  call from_history('label3',label3," ")
  call from_history('label4',label4," ")
  call from_history('label5',label5," ")
  call from_history('label6',label6," ")
  call from_history('label7',label7," ")

  ! Read from input Parameters
  call from_par('nr',nr, -1)              ! Maximum nr of offsets along radius
  if (nr.eq.-1) then
    nr = sqrt(n2**2.+n3**2.)+.5  
  end if
  call from_par('dr',dr, d2)              ! Offset step along radius
  call from_par('or',or, 0.)              ! Smallest offset of radius

  ! Read from input Parameters
  call from_param("interp",interp,1)


  ! Adapt center of Cube axes
  call from_param('center2',center2,0.)    ! If different from origin of cube axes 2
  call from_param('center3',center3,0.)    ! If different from origin of cube axes 3
  call from_param('rf',rf,1.)              ! Rounding value for gridding before regularization with splines
  call from_param("p",p,.1)                ! Tense value for splines
  
  ! Write to output History
  call to_history('n2',nr)
  call to_history('n3',n4)
  call to_history('n4',n5)
  call to_history('n5',n6)
  call to_history('n6',n7)
  call to_history('n7',1 )

  call to_history('d2',dr)
  call to_history('d3',d4)
  call to_history('d4',d5)
  call to_history('d5',d6)
  call to_history('d6',d7)
  call to_history('d7',1.)

  call to_history('o2',or)
  call to_history('o3',o4)
  call to_history('o4',o5)
  call to_history('o5',o6)
  call to_history('o6',o7)
  call to_history('o7',0.)

  call to_history('label2',"radial offset")
  call to_history('label3',label4)
  call to_history('label4',label5)
  call to_history('label5',label6)
  call to_history('label6',label7)
  call to_history('label7',  " " )  

  ! Write to output History of optional fold file
  if ( exist_file ("fold") ) then
    call to_history('n1',nr,'fold')
    call to_history('d1',dr,'fold')
    call to_history('o1',or,'fold')
    call to_history('n2',1,'fold')
    call to_history('n3',1,'fold')
  end if

  call sep_close()
  
  ! Memory allocation
  allocate( data(n1,n2), stack(n1,nr), fold(nr) )

  do oo=1,n7; do nn=1,n6; do mm=1,n5; do ll=1,n4

  stack = 0.
  fold  = 0.
  

  select case (interp)


    case(0)              ! Binning by pushing
      do kk = 1, n3
         !data  = 0.
         call sreed("in",data,4*size(data))             ! Read in each plane
         x3 = o3 - center3 + (kk-1)*d3                  ! Determine x3 coordinate

         do jj = 1, n2
            x2 = o2 - center2 + (jj-1)*d2               ! Determine x2 coordinate

            radius = sqrt( x2**2+x3**2 )                ! Radial coordinate
            ir     = 1. + radius/dr                     ! determine radial index (unrounded)     
            if (abs(sum(data(:,jj))).gt.0.) then        ! Only if not empty
 
              if (    nint(ir).le.nr) then
                 stack(:,nint(ir)) = stack(:,nint(ir)) + data(:,jj)   ! Stack
                 fold (  nint(ir)) = fold (  nint(ir)) + 1.           ! Fold count
              end if
            end if
        
         end do
      end do
      ! Normalize with fold
      do ii=1,nr
        if ( fold(ii).ne.0. ) then
          stack(:,ii) = stack(:,ii)/dble(fold(ii))
        end if
      end do


    case(1)              ! Lineair Interpolation by pushing
      do kk = 1, n3
         !data  = 0.
         call sreed("in",data,4*size(data))             ! Read in each plane
         x3 = o3 - center3 + (kk-1)*d3                  ! Determine x3 coordinate

         do jj = 1, n2
            x2 = o2 - center2 + (jj-1)*d2               ! Determine x2 coordinate

            radius = sqrt( x2**2+x3**2 )                ! Radial coordinate
            ir     = 1. + radius/dr                     ! determine radial index (unrounded)
            if (abs(sum(data(:,jj))).gt.0.) then        ! Only if not empty
 
              if (floor(ir)+1.le.nr) then
                 stack(:,  floor(ir)) = stack(:,  floor(ir)) + (floor(ir)+1-ir)*data(:,jj)   ! Stack
                 stack(:,floor(ir)+1) = stack(:,floor(ir)+1) + (ir-  floor(ir))*data(:,jj)   ! Stack
                 fold (    floor(ir)) = fold (    floor(ir)) + (floor(ir)+1-ir)*1.           ! Fold count
                 fold (  floor(ir)+1) = fold (  floor(ir)+1) + (ir-  floor(ir))*1.           ! Fold count
              end if
            end if
        
         end do
      end do
      ! Normalize with fold
      do ii=1,nr
        if ( fold(ii).ne.0. ) then
          stack(:,ii) = stack(:,ii)/dble(fold(ii))
        end if
      end do


    case(2)              ! Spline Regulaization
      do kk = 1, n3
         !data  = 0.
         call sreed("in",data,4*size(data))             ! Read in each plane
         x3 = o3 - center3 + (kk-1)*d3                  ! Determine x3 coordinate

         do jj = 1, n2
            x2 = o2 - center2 + (jj-1)*d2               ! Determine x2 coordinate

            radius = sqrt( x2**2+x3**2 )                ! Radial coordinate
            ir     = 1. + radius/dr                     ! determine radial index (unrounded)
            if (abs(sum(data(:,jj))).gt.0.) then        ! Only if not empty
 
              if (floor(ir)+1.le.nr) then
                 stack(:,  floor(ir)) = stack(:,  floor(ir)) + (floor(ir)+1-ir)*data(:,jj)   ! Stack
                 stack(:,floor(ir)+1) = stack(:,floor(ir)+1) + (ir-  floor(ir))*data(:,jj)   ! Stack
                 fold (    floor(ir)) = fold (    floor(ir)) + (floor(ir)+1-ir)*1.           ! Fold count
                 fold (  floor(ir)+1) = fold (  floor(ir)+1) + (ir-  floor(ir))*1.           ! Fold count
              end if
            end if
        
         end do
      end do
      ! Normalize with fold
      do ii=1,nr
        if ( fold(ii).ne.0. ) then
          stack(:,ii) = stack(:,ii)/dble(fold(ii))
        end if
      end do

      
      nin =nr   ! Maximum input size
      nout=nr
      
      ! Determine output grid
      allocate(rout(nr)); do ii=1,nr; rout(ii)=(ii-1)*dr; end do;
      ! Allocate input grid
      allocate(rin(nin)); rin = 0.
      ! Allocate input data space
      allocate(rdata(n1,nin)); rdata=0.
      ! Allocate output data space
      allocate(datar(n1,nr));  datar=0.
      
      nin=0
      do ii=1,nr
        if (sum(real(stack(:,ii))).ne.0.) then
          nin = nin+1
          rdata(:,nin) = real(stack(:,ii))
          rin  (  nin) = (ii-1)*dr
        end if
      end do
      nout = maxval(minloc(sum(real(stack(:,:)),1)))


      if (nin.ge.2) then;
      ! Actual regularization
      if (p.eq.0.) then
        call regurize1d_2_isplines_init(rin(1:nin),rout (  1:nout))
        call regurize1d_2_isplines( rdata(:,1:nin),datar(:,1:nout))
        call regurize1d_splines_close()
      else
        call regurize1d_2_isplinest_init(rin(1:nin),rout (  1:nout), p)
        call regurize1d_2_isplinest( rdata(:,1:nin),datar(:,1:nout))
        call regurize1d_splines_close()
      end if
      stack = dble(datar)
      end if

      deallocate(rin,rout,rdata,datar)
  
  
  end select
   
   ! Write out
   call srite("out",real(stack),4*size(stack))

  end do; end do; end do; end do;

  ! Optionally write out fold
  if ( exist_file ("fold") ) then
     call sep_write(fold,'fold')
  end if

  ! Deallocate
  deallocate( data, stack, fold )

  call exit(0);
  
end program circlestack




