program sdrCylinTrans
  ! Program to transform three component data into cylindrical coordinates,
  ! With origin at source location
  
  ! Input files: x, y and z(optional)
  
  ! Parameter: regular (logical) determines if the grid is regular, and if
  ! the 4th and 5 dimentions are taken as the source locations, else, need
  ! header file: head with as many traces as n2 (for x,y and z). and
  ! paremeters rx_key,ry_key,sx_key and sy_key

  use sep
  
  integer            :: n1,nx1,ny1,nz1,nxy1,nyx1
  integer            :: n2,nx2,ny2,nz2,nxy2,nyx2
  integer            :: n3,nx3,ny3,nz3,nxy3,nyx3
  integer            :: n4,nx4,ny4,nz4,nxy4,nyx4
  integer            :: n5,nx5,ny5,nz5,nxy5,nyx5

  real               :: o1,ox1,oy1,oz1,oxy1,oyx1
  real               :: o2,ox2,oy2,oz2,oxy2,oyx2
  real               :: o3,ox3,oy3,oz3,oxy3,oyx3
  real               :: o4,ox4,oy4,oz4,oxy4,oyx4
  real               :: o5,ox5,oy5,oz5,oxy5,oyx5

  real               :: d1,dx1,dy1,dz1,dxy1,dyx1
  real               :: d2,dx2,dy2,dz2,dxy2,dyx2
  real               :: d3,dx3,dy3,dz3,dxy3,dyx3
  real               :: d4,dx4,dy4,dz4,dxy4,dyx4
  real               :: d5,dx5,dy5,dz5,dxy5,dyx5

  character*50       :: label1  ,label2  ,label3  ,label4  ,label5
  
  logical            :: regular
  integer            :: n1hdr,n2hdr

  integer            :: rx,ry,sx,sy,ind   ! Counters

  real               :: angle,delx,dely
  
  real, allocatable  :: dataxx(:),datayy(:),dataxy(:),datayx(:),dataz(:)
  real, allocatable  :: datarr(:),datatt(:),datart(:),datatr(:),datav(:)
  real, allocatable  :: head(:,:)


  call sep_init()

  ! *********************************************************************
  !  Read Grid
  ! *********************************************************************
  write(0,*) '------------------------'
  write(0,*) 'Retrieve geometry'

  ! Geometry of the experiments
  if (exist_file("x")) then; 
   call from_aux("x","n1",nx1,1 )
   call from_aux("x","n2",nx2,1 )
   call from_aux("x","n3",nx3,1 )
   call from_aux("x","n4",nx4,1 )
   call from_aux("x","n5",nx5,1 )
   call from_aux("x","d1",dx1,1.)
   call from_aux("x","d2",dx2,1.)
   call from_aux("x","d3",dx3,1.)
   call from_aux("x","d4",dx4,1.)
   call from_aux("x","d5",dx5,1.)
   call from_aux("x","o1",ox1,0.)
   call from_aux("x","o2",ox2,0.)
   call from_aux("x","o3",ox3,0.)
   call from_aux("x","o4",ox4,0.)
   call from_aux("x","o5",ox5,0.)
   call from_aux("x","label1",label1," ")
   call from_aux("x","label2",label2," ")
   call from_aux("x","label3",label3," ")
   call from_aux("x","label4",label4," ")
   call from_aux("x","label5",label5," ")
  end if
  
  if (exist_file("y")) then; 
   call from_aux("y","n1",ny1,1 )
   call from_aux("y","n2",ny2,1 )
   call from_aux("y","n3",ny3,1 )
   call from_aux("y","n4",ny4,1 )
   call from_aux("y","n5",ny5,1 )
   call from_aux("y","d1",dy1,1.)
   call from_aux("y","d2",dy2,1.)
   call from_aux("y","d3",dy3,1.)
   call from_aux("y","d4",dy4,1.)
   call from_aux("y","d5",dy5,1.)
   call from_aux("y","o1",oy1,0.)
   call from_aux("y","o2",oy2,0.)
   call from_aux("y","o3",oy3,0.)
   call from_aux("y","o4",oy4,0.)
   call from_aux("y","o5",oy5,0.)
   call from_aux("y","label1",label1,label1)
   call from_aux("y","label2",label2,label2)
   call from_aux("y","label3",label3,label3)
   call from_aux("y","label4",label4,label4)
   call from_aux("y","label5",label5,label5)
  else;
   ny1 = nx1; dy1 = dx1; oy1 = ox1; ! needed for error check only
   ny2 = nx1; dy2 = dx2; oy2 = ox2; ! needed for error check only
   ny3 = nx1; dy3 = dx3; oy3 = ox3; ! needed for error check only
   ny4 = nx1; dy4 = dx4; oy4 = ox4; ! needed for error check only
   ny5 = nx1; dy5 = dx5; oy5 = ox5; ! needed for error check only
  end if

  if (exist_file("z")) then   
   call from_aux("z","n1",nz1,1 )
   call from_aux("z","n2",nz2,1 )
   call from_aux("z","n3",nz3,1 )
   call from_aux("z","n4",nz4,1 )
   call from_aux("z","n5",nz5,1 )
   call from_aux("z","d1",dz1,1.)
   call from_aux("z","d2",dz2,1.)
   call from_aux("z","d3",dz3,1.)
   call from_aux("z","d4",dz4,1.)
   call from_aux("z","d5",dz5,1.)
   call from_aux("z","o1",oz1,0.)
   call from_aux("z","o2",oz2,0.)
   call from_aux("z","o3",oz3,0.)
   call from_aux("z","o4",oz4,0.)
   call from_aux("z","o5",oz5,0.)
   call from_aux("z","label1",label1,label1)
   call from_aux("z","label2",label2,label2)
   call from_aux("z","label3",label3,label3)
   call from_aux("z","label4",label4,label4)
   call from_aux("z","label5",label5,label5)
  else
   nz1 = nx1; dz1 = dx1; oz1 = ox1; ! needed for error check only
   nz2 = nx2; dz2 = dx2; oz2 = ox2; ! needed for error check only
   nz3 = nx3; dz3 = dx3; oz3 = ox3; ! needed for error check only
   nz4 = nx4; dz4 = dx4; oz4 = ox4; ! needed for error check only
   nz5 = nx5; dz5 = dx5; oz5 = ox5; ! needed for error check only
  end if

  if (exist_file("xy")) then   
   call from_aux("xy","n1",nxy1,1 )
   call from_aux("xy","n2",nxy2,1 )
   call from_aux("xy","n3",nxy3,1 )
   call from_aux("xy","n4",nxy4,1 )
   call from_aux("xy","n5",nxy5,1 )
   call from_aux("xy","d1",dxy1,1.)
   call from_aux("xy","d2",dxy2,1.)
   call from_aux("xy","d3",dxy3,1.)
   call from_aux("xy","d4",dxy4,1.)
   call from_aux("xy","d5",dxy5,1.)
   call from_aux("xy","o1",oxy1,0.)
   call from_aux("xy","o2",oxy2,0.)
   call from_aux("xy","o3",oxy3,0.)
   call from_aux("xy","o4",oxy4,0.)
   call from_aux("xy","o5",oxy5,0.)
   call from_aux("xy","label1",label1,label1)
   call from_aux("xy","label2",label2,label2)
   call from_aux("xy","label3",label3,label3)
   call from_aux("xy","label4",label4,label4)
   call from_aux("xy","label5",label5,label5)
  else
   nxy1 = nx1; dxy1 = dx1; oxy1 = ox1; ! needed for error check only
   nxy2 = nx2; dxy2 = dx2; oxy2 = ox2; ! needed for error check only
   nxy3 = nx3; dxy3 = dx3; oxy3 = ox3; ! needed for error check only
   nxy4 = nx4; dxy4 = dx4; oxy4 = ox4; ! needed for error check only
   nxy5 = nx5; dxy5 = dx5; oxy5 = ox5; ! needed for error check only
  end if

  if (exist_file("yx")) then   
   call from_aux("yx","n1",nyx1,1 )
   call from_aux("yx","n2",nyx2,1 )
   call from_aux("yx","n3",nyx3,1 )
   call from_aux("yx","n4",nyx4,1 )
   call from_aux("yx","n5",nyx5,1 )
   call from_aux("yx","d1",dyx1,1.)
   call from_aux("yx","d2",dyx2,1.)
   call from_aux("yx","d3",dyx3,1.)
   call from_aux("yx","d4",dyx4,1.)
   call from_aux("yx","d5",dyx5,1.)
   call from_aux("yx","o1",oyx1,0.)
   call from_aux("yx","o2",oyx2,0.)
   call from_aux("yx","o3",oyx3,0.)
   call from_aux("yx","o4",oyx4,0.)
   call from_aux("yx","o5",oyx5,0.)
   call from_aux("yx","label1",label1,label1)
   call from_aux("yx","label2",label2,label2)
   call from_aux("yx","label3",label3,label3)
   call from_aux("yx","label4",label4,label4)
   call from_aux("yx","label5",label5,label5)
  else
   nyx1 = nx1; dyx1 = dx1; oyx1 = ox1; ! needed for error check only
   nyx2 = nx2; dyx2 = dx2; oyx2 = ox2; ! needed for error check only
   nyx3 = nx3; dyx3 = dx3; oyx3 = ox3; ! needed for error check only
   nyx4 = nx4; dyx4 = dx4; oyx4 = ox4; ! needed for error check only
   nyx5 = nx5; dyx5 = dx5; oyx5 = ox5; ! needed for error check only
  end if
  
  
  
  ! Check if receiver grid of these receivers is complete and consistent
  if (.not. exist_file("x" )) then; write(0,*) 'no x-component file' ; end if   
  if (.not. exist_file("y" )) then; write(0,*) 'no y-component file' ; end if   
  if (.not. exist_file("xy")) then; write(0,*) 'no xy-component file'; end if   
  if (.not. exist_file("yx")) then; write(0,*) 'no yx-component file'; end if   
  if (nx1.ne.ny1 .or. nx1.ne.nz1 .or. nx1.ne.nxy1 .or. nx1.ne.nyx1) then;   write(0,*) 'n1 not consistent';  call exit(0);   end if
  if (nx2.ne.ny2 .or. nx2.ne.nz2 .or. nx2.ne.nxy2 .or. nx2.ne.nyx2) then;   write(0,*) 'n2 not consistent';  call exit(0);   end if
  if (nx3.ne.ny3 .or. nx3.ne.nz3 .or. nx3.ne.nxy3 .or. nx3.ne.nyx3) then;   write(0,*) 'n3 not consistent';  call exit(0);   end if
  if (nx4.ne.ny4 .or. nx4.ne.nz4 .or. nx4.ne.nxy4 .or. nx4.ne.nyx4) then;   write(0,*) 'n4 not consistent';  call exit(0);   end if
  if (nx5.ne.ny5 .or. nx5.ne.nz5 .or. nx5.ne.nxy5 .or. nx5.ne.nyx5) then;   write(0,*) 'n5 not consistent';  call exit(0);   end if
  n1=nx1; n2=nx2;  n3=nx3;  n4=nx4;  n5=nx5;
  d1=dx1; d2=dx2;  d3=dx3;  d4=dx4;  d5=dx5;
  o1=ox1; o2=ox2;  o3=ox3;  o4=ox4;  o5=ox5;
  
  call from_param('regular',regular,.true.)
  if (.not.exist_file("head")) then; regular=.true.; write(0,*) "No header-file, assuming regular format"; end if
  
  rx_key = 1;   ry_key = 2
  sx_key = 3;   sy_key = 4
  if (.not.regular) then
    ! Read header of irregular grid
    if (exist_file("head")) then
      call from_aux("head","n1",n1hdr)
      call from_aux("head","n2",n2hdr)
    else
      write(0,*) "Need header for irrelugar geometry"; call exit(0)
    end if
    if (n2hdr.ne.n2*n3*n4*n5) then; write(0,*) "n2hdr not equal to nr of traces"; end if
    allocate(head(n1hdr,n2hdr))
    call srite("head",head,4*size(head))
    call from_param("rx_key",rx_key,rx_key)
    call from_param("ry_key",ry_key,ry_key)
    call from_param("sx_key",sx_key,sx_key)
    call from_param("sy_key",sy_key,sy_key)
  else
    ! Create header for regular grid
    n2hdr = n2*n3*n4*n5
    allocate(head(4,n2hdr))
    do sy=1,n5
    do sx=1,n4
    do ry=1,n3
    do rx=1,n2
      ind = (sy-1)*n4*n3*n2+(sx-1)*n3*n2+(ry-1)*n2+rx
      head(1,ind) = o2+(rx-1)*d2
      head(2,ind) = o3+(ry-1)*d3
      head(3,ind) = o4+(sx-1)*d4
      head(4,ind) = o5+(sy-1)*d5
    end do
    end do
    end do
    end do
  end if
  
  ! Create output headers
  if (.not.exist_file("r").and..not.exist_file("t")) then;
  write(0,*) 'No radial or tangential component output filed'; end if 

  if (exist_file("r")) then;
    call to_history("n1",n1,"r")
    call to_history("n2",n2,"r")
    call to_history("n3",n3,"r")
    call to_history("n4",n4,"r")
    call to_history("n5",n5,"r")

    call to_history("d1",d1,"r")
    call to_history("d2",d2,"r")
    call to_history("d3",d3,"r")
    call to_history("d4",d4,"r")
    call to_history("d5",d5,"r")

    call to_history("o1",o1,"r")
    call to_history("o2",o2,"r")
    call to_history("o3",o3,"r")
    call to_history("o4",o4,"r")
    call to_history("o5",o5,"r")

    call to_history("label1",label1,"r")
    call to_history("label2",label2,"r")
    call to_history("label3",label3,"r")
    call to_history("label4",label4,"r")
    call to_history("label5",label5,"r")
  end if
  if (exist_file("t")) then;
    call to_history("n1",n1,"t")
    call to_history("n2",n2,"t")
    call to_history("n3",n3,"t")
    call to_history("n4",n4,"t")
    call to_history("n5",n5,"t")

    call to_history("d1",d1,"t")
    call to_history("d2",d2,"t")
    call to_history("d3",d3,"t")
    call to_history("d4",d4,"t")
    call to_history("d5",d5,"t")

    call to_history("o1",o1,"t")
    call to_history("o2",o2,"t")
    call to_history("o3",o3,"t")
    call to_history("o4",o4,"t")
    call to_history("o5",o5,"t")

    call to_history("label1",label1,"t")
    call to_history("label2",label2,"t")
    call to_history("label3",label3,"t")
    call to_history("label4",label4,"t")
    call to_history("label5",label5,"t")
  end if
  if (exist_file("rt")) then;
    call to_history("n1",n1,"rt")
    call to_history("n2",n2,"rt")
    call to_history("n3",n3,"rt")
    call to_history("n4",n4,"rt")
    call to_history("n5",n5,"rt")

    call to_history("d1",d1,"rt")
    call to_history("d2",d2,"rt")
    call to_history("d3",d3,"rt")
    call to_history("d4",d4,"rt")
    call to_history("d5",d5,"rt")

    call to_history("o1",o1,"rt")
    call to_history("o2",o2,"rt")
    call to_history("o3",o3,"rt")
    call to_history("o4",o4,"rt")
    call to_history("o5",o5,"rt")

    call to_history("label1",label1,"rt")
    call to_history("label2",label2,"rt")
    call to_history("label3",label3,"rt")
    call to_history("label4",label4,"rt")
    call to_history("label5",label5,"rt")
  end if
  if (exist_file("tr")) then;
    call to_history("n1",n1,"tr")
    call to_history("n2",n2,"tr")
    call to_history("n3",n3,"tr")
    call to_history("n4",n4,"tr")
    call to_history("n5",n5,"tr")

    call to_history("d1",d1,"tr")
    call to_history("d2",d2,"tr")
    call to_history("d3",d3,"tr")
    call to_history("d4",d4,"tr")
    call to_history("d5",d5,"tr")

    call to_history("o1",o1,"tr")
    call to_history("o2",o2,"tr")
    call to_history("o3",o3,"tr")
    call to_history("o4",o4,"tr")
    call to_history("o5",o5,"tr")

    call to_history("label1",label1,"tr")
    call to_history("label2",label2,"tr")
    call to_history("label3",label3,"tr")
    call to_history("label4",label4,"tr")
    call to_history("label5",label5,"tr")
  end if
  if (exist_file("v")) then;
    call to_history("n1",n1,"v")
    call to_history("n2",n2,"v")
    call to_history("n3",n3,"v")
    call to_history("n4",n4,"v")
    call to_history("n5",n5,"v")

    call to_history("d1",d1,"v")
    call to_history("d2",d2,"v")
    call to_history("d3",d3,"v")
    call to_history("d4",d4,"v")
    call to_history("d5",d5,"v")

    call to_history("o1",o1,"v")
    call to_history("o2",o2,"v")
    call to_history("o3",o3,"v")
    call to_history("o4",o4,"v")
    call to_history("o5",o5,"v")

    call to_history("label1",label1,"v")
    call to_history("label2",label2,"v")
    call to_history("label3",label3,"v")
    call to_history("label4",label4,"v")
    call to_history("label5",label5,"v")
  end if
  
  call sep_close()
  
  allocate(dataxx(n1))
  allocate(dataxy(n1))
  allocate(datayx(n1))
  allocate(datayy(n1))
  allocate(dataz(n1))
  allocate(datarr(n1))
  allocate(datatr(n1))
  allocate(datart(n1))
  allocate(datatt(n1))
  allocate(datav(n1))
  
  
  
  ! *********************************************************************
  !  Rotations
  ! *********************************************************************
  write(0,*) '------------------------'
  write(0,*) 'Actual rotations'

  do sy=1,n5
  do sx=1,n4

  do ry=1,n3
  do rx=1,n2
  ind = (sy-1)*n4*n3*n2+(sx-1)*n3*n2+(ry-1)*n2+rx

  ! Zero out traces
  datax = 0.
  datay = 0.
  dataz = 0.
  datar = 0.
  datat = 0.
  datav = 0.

  ! Read data traces
  call sreed("x" ,dataxx,4*n1)
  call sreed("xy",dataxy,4*n1)
  call sreed("y" ,datayy,4*n1)
  call sreed("yx",datayx,4*n1)
  if (exist_file("z")) then;  call sreed("z",dataz,4*n1);  else;  dataz = 0.;  end if
  
  ! Rotation Angle
  delx  = head(rx_key,ind) - head(sx_key,ind)
  dely  = head(ry_key,ind) - head(sy_key,ind)
  angle = atan2(dely,delx)
  
  ! Rotation formula
  datarr = cos(angle)*cos(angle)*dataxx - cos(angle)*sin(angle)*dataxy - sin(angle)*cos(angle)*datayx + sin(angle)*sin(angle)*datayy
  datart = cos(angle)*sin(angle)*dataxx + cos(angle)*cos(angle)*dataxy - sin(angle)*sin(angle)*datayx - sin(angle)*cos(angle)*datayy
  
  datatr = sin(angle)*cos(angle)*dataxx - sin(angle)*sin(angle)*dataxy + cos(angle)*cos(angle)*datayx - cos(angle)*sin(angle)*datayy
  datatt = sin(angle)*sin(angle)*dataxx + sin(angle)*cos(angle)*dataxy + cos(angle)*sin(angle)*datayx + cos(angle)*cos(angle)*datayy
  
  datav  = dataz
  
  ! Write out traces
  if (exist_file("r" )) then;  call srite("r" ,datarr,4*n1);  end if
  if (exist_file("t" )) then;  call srite("t" ,datatt,4*n1);  end if
  if (exist_file("rt")) then;  call srite("rt",datart,4*n1);  end if
  if (exist_file("tr")) then;  call srite("tr",datatr,4*n1);  end if
  if (exist_file("v" )) then;  call srite("v" ,datav ,4*n1);  end if
  
  end do
  end do
  
  end do
  end do
  
  
  
  ! *********************************************************************
  !  Clean up and exit
  ! *********************************************************************
  write(0,*) '------------------------'
  deallocate(dataxx)
  deallocate(datayx)
  deallocate(dataxy)
  deallocate(datayy)
  deallocate(datarr)
  deallocate(datart)
  deallocate(datatt)
  deallocate(datatr)
  deallocate(dataz)
  deallocate(datav)
  deallocate(head)
  call exit(0)
  
end program
