program Ldifint
  ! Leaky differention and integration
  !
  ! Sjoerd de Ridder
  ! Stanford, January 22th 2009.
  ! 
  !  
  ! Differantiation and integration implemented with helix (1, -rho Z)
  ! 
  ! rho  = 1.   : damping
  !
  ! inv  = 0    : convolve[0]     or deconvolve[1] by (1-rho Z)
  !               differentiation or integration
  !
  ! adj  = 0    : forward[0] or adjoint[1]
  !               causal     or anti-causal
  
  use sep
  !use helix
  !use helicon
  !use polydiv
  use leaky_helix_mod
  implicit none
  
  integer                       :: n1,n2,n3
  integer                       :: n23
  real, target, allocatable     :: datain(:,:,:),dataout(:,:,:)
  real, pointer                 :: xx(:),yy(:)
  logical                       :: inv,adj,add
  integer                       :: flag,stat
  real                          :: rho
  type (filter)                 :: filt
  
  call sep_init(SOURCE)
  
  ! Read from history
  call from_history(n1,n2,n3)
  
  ! read from parameters
  call from_par('inv',inv,.false.);
  call from_par('adj',adj,.false.);
  call from_par('add',add,.false.);
  call from_par('rho',rho,1.)
  
  call sep_close()

  ! Allocate memory
  allocate( datain(n1,n2,n3), dataout(n1,n2,n3) )

  ! Initialize Leaky module
  call leaky_init(rho,n1,adj)

  ! Read input data
  call sreed(  "in", datain , 4*size(datain)  )

  ! Call Operator
  call leaky_fop(inv,add,datain,dataout)  

  ! Write output data
  call sreed( "out", dataout, 4*size(dataout) )
  
  ! Close Leaky module
  call leaky_close( )
  
  ! Deallocate
  deallocate(datain,dataout)
  
end program Ldifint



