program sdrMDTape
! Tape a datafile
  ! taper types: 
  ! 1) 'zero'        --  needed: 'nothing' 
  ! 2) 'rect'        --  needed: t1,t2 
  ! 3) 'linear'      --  needed: t1,t2
  ! 4) 'pyramid'     --  needed: t1,t2
  ! 5) 'pyramid_band'--  needed: t1,t2,t3,t4
  ! 6) 'hann'        --  needed: t1,t2
  ! 7) 'hann_half'   --  needed: t1,t2
  ! 8) 'hann_band'   --  needed: t2,t2,t3,t4
  ! 9) 'cosine'      --  needed: t1,t2
  !! 10) 'sine'       --  needed: t1,t2    (this is cosine with inv=1)
  ! 11) 'gaussian'   --  needed: t1,t2,sigma,spread
  ! 12) 'gaussian_half'  --  needed: t1,t2,sigma,spread
  ! 13) 'gaussian_block' --  needed: t1,t2,t3,t4,sigma,spread
  ! 14) 'slepian'    --  needed: t1,t2,nt
  !
  ! All tapers have the form of a band or lowcut,
  ! Notch tapers are constructed with inv=1 option.
  ! Sum result onto old data using    add=1 option.
  
  ! Multiple calls take input and overwrite if add=1
  
  use sep                      ! Main SEP module
  use mdtapers_mod
  use mdtape_mod
  implicit none
  
  real, allocatable               :: mdtaper(:,:,:)
  integer                         :: n(3)
  integer                         :: type,dim
  real                            :: d(3),o(3)
  real                            :: p1,p2,p3,p4
  real                            :: t1_in(3),t2_in(3),c(3)
  integer                         :: t1_ind(3),t2_ind(3),c_ind(3)
  real                            :: p1_ang,p2_ang,p3_ang,p4_ang
  real                            :: t1(3),t2(3)
  logical                         :: add,inv
  real                            :: sigma,scale
  integer                         :: ii

  real, parameter                 :: pi=3.14159265
  
  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  call from_param("n1", n(1) ,256 )
  call from_param("n2", n(2) ,256 )
  call from_param("n3", n(3) , 1  )
  call from_param("d1", d(1) , 1. )
  call from_param("d2", d(2) , 1. )
  call from_param("d3", d(3) , 1. )
  call from_param("o1", o(1) , 0. )
  call from_param("o2", o(2) , 0. )
  call from_param("o3", o(3) , 0. )
  
  ! Read input parameters for taper, there is lots of options :-)
  call from_param ("type" ,type, 0 )
  call from_param ("inv"  ,inv,.false. )
  call from_param ("add"  ,add,.true.  )
  call from_param ("sigma",sigma,.1)
  call from_param ("scale",scale,1.)
  
  call from_param ("t1",t1, (/ 0., 0., 0. /) )
  call from_param ("t2",t2, (/ o(1) + d(1)*real(n(1)-1), o(2) + d(2)*real(n(2)-1), o(3) + d(3)*real(n(3)-1)/) )
  call from_param ("p1",p1_ang,  0.   )
  call from_param ("p2",p2_ang, .5 )
  call from_param ("p3",p3_ang, .5 )
  call from_param ("p4",p4_ang, .5 )
  call from_param ("c",c, (/ 0.,0.,0./) )
  
  ! Write to output .H file
  call to_history ( "n1", n(1) )   
  call to_history ( "n2", n(2) )
  call to_history ( "n3", n(3) )
  call to_history ( "d1", d(1) )
  call to_history ( "d2", d(2) )
  call to_history ( "d3", d(3) )
  call to_history ( "o1", o(1) )
  call to_history ( "o2", o(2) )
  call to_history ( "o3", o(3) )
  
  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
  
  ! Convert angle in pi to radial angle
  p1_ang = p1_ang*pi; p2_ang = p2_ang*pi
  p3_ang = p3_ang*pi; p4_ang = p4_ang*pi
  do ii=1,3
    call mdtaper_indice(t1(ii),t1_ind(ii),d_in=d(ii),o_in=o(ii))
    call mdtaper_indice(t2(ii),t2_ind(ii),d_in=d(ii),o_in=o(ii))
    call mdtaper_indice( c(ii), c_ind(ii),d_in=d(ii),o_in=o(ii))
  end do

  !write(0,*) "p1_ang,p2_ang,t1,t1_ind,t2,t2_ind:",p1_ang,p2_ang,t1,t1_ind,t2,t2_ind
  !write(0,*) "n1,n2,n3",n(1),n(2),n(3)
  !write(0,*) "d1,d2,d3",d(1),d(2),d(3)
  !write(0,*) "o1,o2,o3",o(1),o(2),o(3)

  !write(0,*) "c,c_ind",c,c_ind

  !write(0,*) '1,p1_in=(/p1_ang/),p2_in=(/p2_ang/),p3_in=(/p3_ang/),p4_in=(/p4_ang/), &
  !                        t1_in=(/1.,1.,1/),t2_in=(/nw(1),nw(2),nw(3)/),type_in=(/p_type/),sigma_in=(/sigma/),scale_in=(/1./),inv_in=(/inv_ang/)'
  !write(0,*) "=============================="
  !write(0,*) "=============================="
  !write(0,*)  "n_in", n
  !write(0,*) "c_in", c_ind
  !write(0,*) "d_in", d
  !write(0,*) "o_in", o
  !write(0,*) 1
  !write(0,*) "p1_in=", (/p1_ang/)
  !write(0,*) "p2_in=", (/p2_ang/)
  !write(0,*) "p3_in=", (/p3_ang/)
  !write(0,*) "p4_in=", (/p4_ang/)
  !write(0,*) "t1_in=", (/1.,1.,1./)
  !write(0,*) "t2_in=", (/n(1),n(2),n(3)/)
  !write(0,*) "type=", (/type/)
  !write(0,*) "sigma_in=", (/sigma/)
  !write(0,*) "scale_in=", (/1./)
  !write(0,*) "inv_in=", (/inv/)
  !write(0,*) "=============================="
  !write(0,*) "=============================="


  ! Initialize taper functions in tapers_mod
  call mdtapers_init     (n_in=n,c_in=c_ind,d_in=d,o_in=o)
  call mdtapers_interface(1,p1_in=(/p1_ang/),p2_in=(/p2_ang/),p3_in=(/p3_ang/),p4_in=(/p4_ang/), &
                          t1_in=t1_ind,t2_in=t2_ind,type_in=(/type/),sigma_in=(/sigma/),scale_in=(/scale/),inv_in=(/inv/))
  
  ! Allocate the arrays datain and dataout
  allocate ( mdtaper(n(1),n(2),n(3)) )
  
  ! Execute Taper
  call mdtapers_gettaper(mdtaper,inv)
  
  ! Write data to output file
  call sep_write (mdtaper)
  
  ! Deallocate memory  
  deallocate     (mdtaper)
  call mdtape_close()
  
  ! Force clean exit
  call exit (0)
  
end program

