program sdrMTape
! Tape a datafile
  ! taper types: 
  ! 1) 'zero'        --  needed: 'nothing' 
  ! 2) 'rect'        --  needed: t1,t2 
  ! 3) 'linear'      --  needed: t1,t2
  ! 4) 'pyramid'     --  needed: t1,t2
  ! 5) 'pyramid_band'--  needed: t1,t2,t3,t4
  ! 6) 'hann'        --  needed: t1,t2
  ! 7) 'hann_half'   --  needed: t1,t2
  ! 8) 'hann_band'   --  needed: t2,t2,t3,t4
  ! 9) 'cosine'      --  needed: t1,t2
  !! 10) 'sine'       --  needed: t1,t2    (this is cosine with inv=1)
  ! 11) 'gaussian'   --  needed: t1,t2,sigma,spread
  ! 12) 'gaussian_half'  --  needed: t1,t2,sigma,spread
  ! 13) 'gaussian_block' --  needed: t1,t2,t3,t4,sigma,spread
  ! 14) 'slepian'    --  needed: t1,t2,nt
  !
  ! All tapers have the form of a band or lowcut,
  ! Notch tapers are constructed with inv=1 option.
  ! Sum result onto old data using    add=1 option.
  
  ! Multiple calls take input and overwrite if add=1
  
  use sep                      ! Main SEP module
  use mtapers_mod
  use mtape_mod
  implicit none
  
  real, allocatable               :: mtaper(:,:),data_in(:,:,:),data_out(:,:,:,:)
  integer                         :: n(3), ii, nt
  real                            :: d(3),o(3)
  real                            :: t1,t2
  integer                         :: t1_ind,t2_ind
  logical                         :: inv
  real                            :: scale,tbp
  
  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  call from_history("n1", n(1) )
  call from_history("n2", n(2) )
  call from_history("n3", n(3) )
  call from_history("d1", d(1) )
  call from_history("d2", d(2) )
  call from_history("d3", d(3) )
  call from_history("o1", o(1) )
  call from_history("o2", o(2) )
  call from_history("o3", o(3) )
  
  ! Read input parameters for taper, there is lots of options :-)
  call from_param ("nt"   ,nt ,    4   )
  call from_param ("inv"  ,inv,.false. )
  call from_param ("tbp"  ,tbp,   10.  )
  call from_param ("scale",scale,  1.  )
  
  call from_param ("t1",t1, o(1)                     )
  call from_param ("t2",t2, o(1) + d(1)*real(n(1)-1) )
  
  ! Write to auxiliary .H file
  call to_history ( "n1", n(1) )   
  call to_history ( "n2", nt   )
  call to_history ( "n3", 1    )
  call to_history ( "d1", d(1) )
  call to_history ( "d2", 1    )
  call to_history ( "d3", 1    )
  call to_history ( "o1", o(1) )
  call to_history ( "o2", 0.   )
  call to_history ( "o3", 0.   )

  ! Write to auxiliary .H file
  if (  exist_file("mtaper_out") ) then
    call to_history ( "n1", n(1) , "mtaper_out" )  
    call to_history ( "n2",  nt  , "mtaper_out" )
    call to_history ( "n3",   1  , "mtaper_out" )
    call to_history ( "d1", d(1) , "mtaper_out" )
    call to_history ( "d2",   1. , "mtaper_out" )
    call to_history ( "d3",   1. , "mtaper_out" )
    call to_history ( "o1", o(1) , "mtaper_out" )
    call to_history ( "o2",   0. , "mtaper_out" )
  end if
  
  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
  
  ! Determine tie indices
  do ii=1,3
    call mtapers_indice( t1,t1_ind,d_in=d(1),o_in=o(1) )
    call mtapers_indice( t2,t2_ind,d_in=d(1),o_in=o(1) )
  end do
  
  ! Initialize tape functions
  call mtape_init  (n1_in=n(1),n2_in=n(2),n3_in=n(3),nt_in=nt)
  
  ! Initialize taper functions in tapers_mod
  ! Allocate Taper Space
  call mtapers_init(n1_in=n(1) ,nt_in=nt )
  call mtapers_interface(.false.,t1_in=t1_ind,t2_in=t2_ind,scale_in=scale,inv_in=inv,tbp_in=tbp)
  call mtapers_insert_mtape()
  call mtapers_close()
  
  ! Allocate the arrays datain and dataout
  allocate ( data_in(n(1),n(2),n(3)) , data_out(n(1),nt,n(2),n(3)))
  
  ! Read in data from input file
  call sep_read  (data_in)
  
  ! Execute Taper
  call mtape_lop(.false.,.false.,data_in,data_out)
  
  ! Write data to output file
  call sep_write (data_out)
  
  ! Retrieve and put out taper, if required.
  if ( exist_file("mtaper_out") ) then
     allocate  (mtaper(n(1),nt))
     call mtape_gettaper(mtaper)
     call srite("mtaper_out", mtaper, 4*size(mtaper))
     deallocate(mtaper         )
  end if
  
  ! Deallocate memory  
  deallocate     (data_out)
  call mtape_close()

  ! Force clean exit
  call exit (0)
  
end program

