program circlestack
  ! Program to perform a circular stack on a data cube.
  !
  ! Sjoerd de Ridder
  ! Stanford, January 20th, 2009
  use sep
  implicit none

  integer                                :: n1, n2, n3, nr
  real                                   :: d2, d3, dr, o2, o3, or
  integer                                :: ii, jj, kk, ir
  real                                   :: x2, x3, center2, center3
  real,             allocatable          :: data(:,:), fold(:), stackr(:,:)
  double precision, allocatable          :: stack(:,:)
  
  call sep_init()
  
  ! Read from input History
  call from_history('n1',n1)
  call from_history('n2',n2)
  call from_history('n3',n3)
  call from_history('d2',d2)
  call from_history('d3',d3)
  call from_history('o2',o2)
  call from_history('o3',o3)

  ! Read from input Parameters
  call from_par('nr',nr,n2/2)              ! Maximum nr of offsets along radius
  call from_par('dr',dr,d2)              ! Offset step along radius
  call from_par('or',or,0.)              ! Smallest offset of radius

  ! Adapt center of Cube axes
  call from_par('center2',center2,0.)    ! If different from origin of cube axes 2
  call from_par('center3',center3,0.)    ! If different from origin of cube axes 3

  ! Write to output History
  call to_history('n2',nr )
  call to_history('d2',dr )
  call to_history('o2',or )
  call to_history('n3',1  )
  
  ! Write to output History of optional fold file
  if ( exist_file ("fold") ) then
    call to_history('n1',nr,'fold')
    call to_history('d1',dr,'fold')
    call to_history('o1',or,'fold')
    call to_history('n2',1,'fold')
    call to_history('n3',1,'fold')
  end if
  
  call sep_close()
  
  ! Memory allocation
  allocate( data(n1,n2), stack(n1,nr), stackr(n1,nr), fold(nr) )
  fold  = 0.

  ! Perform stack by pushing
  do kk = 1, n3
     call sep_read(data)                            ! Read in each plane
     x3 = o3 - center3 + (kk-1)*d3                  ! Determine x3 coordinate
     do jj = 1, n2
        x2 = o2 - center2 + (jj-1)*d2               ! Determine x2 coordinate
        ir = 1.5 + sqrt( x2**2+x3**2 )/dr           ! determine radial index
        if ( ir.le.nr ) then
           stack(:,ir) = stack(:,ir) + data(:,jj)   ! Stack
           fold (  ir) = fold (  ir) + 1.           ! Fold count
        end if
     end do
  end do
  
  ! Normalize with fold
  do ir=1,nr
    if ( fold(ir).ne.0. ) then
      stackr(:,ir) = stack(:,ir)/fold(ir)
    else
      stackr(:,ir) = stack(:,ir)
    end if
  end do
  
  ! Write out
  call sep_write(stackr)
  
  ! Optionally write out fold
  if ( exist_file ("fold") ) then
     call sep_write(fold,'fold')
  end if
  
end program circlestack




