program sdrPatch
  ! Patch program that works!
  ! Up to 6 dimensions
  ! All patched dimensions are added as additional dimensions

  ! Need to add extensive error checks
  
  use sep
  implicit none
  
  integer            :: n(6),nout(12),nwind(6),npatch(6),fwind(6),jwind(6),ncut(6)
  character*50       :: labels(12)
  
  real               :: d(12)
  real               :: o(12)
  
  integer            :: ii,jj,kk,ll,mm,nn,ij,ndim
  
  real,allocatable   :: data(:,:,:,:,:,:)
  

  ! Hearder IO
  call sep_init()
  n=1
  nout=1
  d=1.
  o=0.
  labels=' '

  ! read original header
  call from_history("n1",n(1),n(1))
  call from_history("n2",n(2),n(2))
  call from_history("n3",n(3),n(3))
  
  call from_history("n4",n(4),n(4))
  call from_history("n5",n(5),n(5))
  call from_history("n6",n(6),n(6))

  call from_history("d1",d(1),d(1))
  call from_history("d2",d(2),d(2))
  call from_history("d3",d(3),d(3))
  
  call from_history("d4",d(4),d(4))
  call from_history("d5",d(5),d(5))
  call from_history("d6",d(6),d(6))

  call from_history("o1",o(1),o(1))
  call from_history("o2",o(2),o(2))
  call from_history("o3",o(3),o(3))
  
  call from_history("o4",o(4),o(4))
  call from_history("o5",o(5),o(5))
  call from_history("o6",o(6),o(6))

  call from_history("label1",labels(1),labels(1))
  call from_history("label2",labels(2),labels(2))
  call from_history("label3",labels(3),labels(3))

  call from_history("label4",labels(4),labels(4))
  call from_history("label5",labels(5),labels(5))
  call from_history("label6",labels(6),labels(6))

  ! read patching parameters
  call from_param("npatch1",npatch(1),1)
  call from_param("npatch2",npatch(2),1)
  call from_param("npatch3",npatch(3),1)
  
  call from_param("npatch4",npatch(4),1)
  call from_param("npatch5",npatch(5),1)
  call from_param("npatch6",npatch(6),1)
  
  call from_param("nwind1",nwind(1),n(1))
  call from_param("nwind2",nwind(2),n(2))
  call from_param("nwind3",nwind(3),n(3))
  
  call from_param("nwind4",nwind(4),n(4))
  call from_param("nwind5",nwind(5),n(5))
  call from_param("nwind6",nwind(6),n(6))
  
  call from_param("fwind1",fwind(1),0)
  call from_param("fwind2",fwind(2),0)
  call from_param("fwind3",fwind(3),0)
  
  call from_param("fwind4",fwind(4),0)
  call from_param("fwind5",fwind(5),0)
  call from_param("fwind6",fwind(6),0)
  
  call from_param("jwind1",jwind(1),nwind(1))
  call from_param("jwind2",jwind(2),nwind(2))
  call from_param("jwind3",jwind(3),nwind(4))
  
  call from_param("jwind4",jwind(4),nwind(4))
  call from_param("jwind5",jwind(5),nwind(5))
  call from_param("jwind6",jwind(6),nwind(6))
  
  
  ! Creating a decent output header is a little hairy:
  do ii=1,6     ! adapt to patch size
    nout(ii) = nwind(ii)
  end do

  do ii=1,6     ! original maximum non-zero dimension
    if (n(ii).ne.1) ndim=ii
  end do
  
  write(0,*) 'npatch', npatch
  write(0,*) 'nwind', nwind

  do ii=1,6     ! add dimensions for the patches
    if (npatch(ii).gt.1) then;
      ndim=ndim+1
      nout(ndim)=npatch(ii)
      d(ndim)=d(ii)*nwind(ii)
      o(ndim)=o(ii)+jwind(ii)*d(ii)
      if (ii.eq.1) labels(ndim)="patches for axis 1"
      if (ii.eq.2) labels(ndim)="patches for axis 2"
      if (ii.eq.3) labels(ndim)="patches for axis 3"
      if (ii.eq.4) labels(ndim)="patches for axis 4"
      if (ii.eq.5) labels(ndim)="patches for axis 5"
      if (ii.eq.6) labels(ndim)="patches for axis 6"
    end if
  end do

  
  ! Write new header
  if (ndim.ge.1) then
    call to_history("n1"    ,  nout(1) )
    call to_history("d1"    ,     d(1) )
    call to_history("o1"    ,     o(1) )
    call to_history("label1",labels(1) )
  end if
  if (ndim.ge.2) then
    call to_history("n2"    ,  nout(2) )
    call to_history("d2"    ,     d(2) )
    call to_history("o2"    ,     o(2) )
    call to_history("label2",labels(2) )
  end if
  if (ndim.ge.3) then
    call to_history("n3"    ,  nout(3) )
    call to_history("d3"    ,     d(3) )
    call to_history("o3"    ,     o(3) )
    call to_history("label3",labels(3) )
  end if
  if (ndim.ge.4) then
    call to_history("n4"    ,  nout(4) )
    call to_history("d4"    ,     d(4) )
    call to_history("o4"    ,     o(4) )
    call to_history("label4",labels(4) )
  end if
  if (ndim.ge.5) then
    call to_history("n5"    ,  nout(5) )
    call to_history("d5"    ,     d(5) )
    call to_history("o5"    ,     o(5) )
    call to_history("label5",labels(5) )
  end if
  if (ndim.ge.6) then
    call to_history("n6"    ,  nout(6) )
    call to_history("d6"    ,     d(6) )
    call to_history("o6"    ,     o(6) )
    call to_history("label6",labels(6) )
  end if
  if (ndim.ge.7) then
    call to_history("n7"    ,  nout(7) )
    call to_history("d7"    ,     d(7) )
    call to_history("o7"    ,     o(7) )
    call to_history("label7",labels(7) )
  end if
  if (ndim.ge.8) then
    call to_history("n8"    ,  nout(8) )
    call to_history("d8"    ,     d(8) )
    call to_history("o8"    ,     o(8) )
    call to_history("label8",labels(8) )
  end if
  if (ndim.ge.9) then
    call to_history("n9"    ,  nout(9) )
    call to_history("d9"    ,     d(9) )
    call to_history("o9"    ,     o(9) )
    call to_history("label9",labels(9) )
  end if
  if (ndim.ge.10) then
    call to_history("n10"    ,  nout(10) )
    call to_history("d10"    ,     d(10) )
    call to_history("o10"    ,     o(10) )
    call to_history("label10",labels(10) )
  end if
  if (ndim.ge.11) then
    call to_history("n11"    ,  nout(11) )
    call to_history("d11"    ,     d(11) )
    call to_history("o11"    ,     o(11) )
    call to_history("label11",labels(11) )
  end if
  if (ndim.ge.12) then
    call to_history("n12"    ,  nout(12) )
    call to_history("d12"    ,     d(12) )
    call to_history("o12"    ,     o(12) )
    call to_history("label12",labels(12) )
  end if
  
  call sep_close()
  

  ! Allocate patch size
  allocate(data(nwind(1),nwind(2),nwind(3),nwind(4),nwind(5),nwind(6)))
  
  ! Loop over patches
  do nn=1,npatch(6)
  do mm=1,npatch(5)
  do ll=1,npatch(4)
  do kk=1,npatch(3)
  do jj=1,npatch(2)
  do ii=1,npatch(1)
   
  ! For padding with zeros
  data=0.

  ! Determine size of readable patch
  ncut(1) = minval( (/ nwind(1) , n(1)-(fwind(1)+(ii-1)*jwind(1)) /) )
  ncut(2) = minval( (/ nwind(2) , n(2)-(fwind(2)+(jj-1)*jwind(2)) /) )
  ncut(3) = minval( (/ nwind(3) , n(3)-(fwind(3)+(kk-1)*jwind(3)) /) )
  ncut(4) = minval( (/ nwind(4) , n(4)-(fwind(4)+(ll-1)*jwind(4)) /) )
  ncut(5) = minval( (/ nwind(5) , n(5)-(fwind(5)+(mm-1)*jwind(5)) /) )
  ncut(6) = minval( (/ nwind(6) , n(6)-(fwind(6)+(nn-1)*jwind(6)) /) )

  ! Patch-cube n and f
  write(0,*) "ncut",ncut
  write(0,*) "fwin",(/fwind(1)+(ii-1)*jwind(1),fwind(2)+(jj-1)*jwind(2), &
                                      fwind(3)+(kk-1)*jwind(3),fwind(4)+(ll-1)*jwind(4), &
                                      fwind(5)+(mm-1)*jwind(5),fwind(6)+(nn-1)*jwind(6)/)

  ! Read patch
  call sreed_window("in",6,n,ncut,(/fwind(1)+(ii-1)*jwind(1),fwind(2)+(jj-1)*jwind(2), &
                                    fwind(3)+(kk-1)*jwind(3),fwind(4)+(ll-1)*jwind(4), &
                                    fwind(5)+(mm-1)*jwind(5),fwind(6)+(nn-1)*jwind(6)/),(/1,1,1,1,1,1/),4, &
                                    data(1:ncut(1),1:ncut(2),1:ncut(3),1:ncut(4),1:ncut(5),1:ncut(6)))

  ! Write patch
  call srite("out",data,4*size(data))

  end do    
  end do    
  end do
  end do    
  end do
  end do
  
  ! Close this mess
  deallocate(data)
  call exit(0)

end program
