program Reg_splines_1d
  
  use sep
  use regurize1d_splines_mod
  
  implicit none
  
  ! Input dimensions
  Integer          :: n1in
  
  ! Output dimensions
  integer          :: n1,fact
  real             :: d1
  real             :: o1
  
  integer          :: ix
  
  real             :: p
  integer          :: mode  ! 1: Irregular spaced splines (conventional),
                            ! 2: Regular spaced splines   (good with tension for noisy data)

  real,allocatable :: datain(:),datout(:)
  real,allocatable :: headerin(:),headeout(:),splngrid(:)
  

  call sep_init()
  
  ! From History
  call from_history('n1',n1in,1)
  
  ! Regular Axis
  call from_param('n1',n1,1 )
  call from_param('d1',d1,1.)
  call from_param('o1',o1,0.)

  call from_param('fact',fact,1)
  call from_param('p',p,0.); p = abs(p)
  call from_param('mode',mode,1);

  call to_history('n1',n1*fact)
  call to_history('d1',d1/fact)
  call to_history('o1',o1)
  
  call sep_close()
  

  ! Allocate
  allocate( datain(n1in   )); datain   = 0.
  allocate( datout(n1*fact)); datout   = 0.
  allocate( headerin(n1in )); headerin = 0.
  allocate( headeout(n1*fact)); headeout = 0.
  
  ! Read header file
  call sreed('header',headerin,4*size(headerin))
  ! Create ouput header
  do ix=1,n1*fact; headeout(ix) = o1+(ix-1)*d1/fact; end do
  ! Initialize interpolator
  if (mode.ne.2) then
  if (p.eq.0.) then
    call regurize1d_1_isplines_init (headerin,headeout)
  else;
    call regurize1d_1_isplinest_init(headerin,headeout,p)
  end if
  else
  allocate( splngrid(n1) )
  do ix=1,n1;      splngrid(ix) = o1+(ix-1)*d1;      end do
  if (p.eq.0.) then
    call regurize1d_1_rsplines_init (headerin,headeout,splngrid)
  else;
    call regurize1d_1_rsplinest_init(headerin,headeout,splngrid,p)
  end if
  end if
  
  
  ! Read, Regularize and Write
  call sreed('in' ,datain,4*size(datain))
  
  if (mode.ne.2) then
  if (p.eq.0.) then;
    call regurize1d_1_isplines (datain,datout)
  else;
    call regurize1d_1_isplinest(datain,datout)
  end if
  else
  if (p.eq.0.) then;
    call regurize1d_1_rsplines (datain,datout)
  else;
    call regurize1d_1_rsplinest(datain,datout)
  end if
  end if
  
  call srite('out',datout,4*size(datout))
  
  ! Deallocate and exit
  call regurize1d_1_splines_close()  
  deallocate(datain)
  deallocate(datout)
  deallocate(headerin)
  deallocate(headeout)
  if (allocated(splngrid)) deallocate(splngrid)
  call exit()
  
end program
