program Reg_splines_2d
  
  use sep
  use regurize2d_splines_mod
  
  implicit none
  
  ! Input dimensions
  Integer          :: n1in
  
  ! Output dimensions
  integer          :: n1,fact1,n2,fact2
  real             :: d1,d2
  real             :: o1,o2
  
  integer          :: ix,iy
  
  real             :: p
  integer          :: mode  ! 1: Irregular spaced splines (conventional),
                            ! 2: Regular spaced splines   (good with tension for noisy data)

  real,allocatable :: datain(:),datout(:)
  real,allocatable :: headerin(:,:),headeout(:,:),splngrid(:,:)
  
  
  call sep_init()
  
  ! From History
  call from_history('n1',n1in,1)
  
  ! Regular Axis
  call from_param('n1',n1,1 )
  call from_param('d1',d1,1.)
  call from_param('o1',o1,0.)
  
  call from_param('n2',n2,1 )
  call from_param('d2',d2,1.)
  call from_param('o2',o2,0.)
  
  call from_param('fact1',fact1,1)
  call from_param('fact2',fact2,1)
  call from_param('p',p,0.); p = abs(p)
  call from_param('mode',mode,1);
  
  call to_history('n1',n1*fact1)
  call to_history('d1',d1/fact1)
  call to_history('o1',o1)
  
  call to_history('n2',n2*fact2)
  call to_history('d2',d2/fact2)
  call to_history('o2',o2)
  
  call sep_close()
  
  
  ! Allocate
  allocate( datain  (n1in    )); datain   = 0.
  allocate( datout  (n1*fact1*n2*fact2)); datout   = 0.
  allocate( headerin(n1in,2));   headerin = 0.
  allocate( headeout(n1*fact1*n2*fact2,2)); headeout = 0.
  
  ! Read header file
  call sreed('header',headerin,4*size(headerin))
  ! Create ouput header
  do iy=1,n2*fact2; 
  do ix=1,n1*fact1;
    headeout((iy-1)*n1+ix,:) = (/ (o1+(ix-1)*d1/fact1) , (o2+(iy-1)*d2/fact2) /);
  end do
  end do
  
  ! Initialize interpolator
  if (mode.ne.2) then
  if (p.eq.0.) then
    call regurize2d_isplines_init (headerin,headeout)
  else;
    call regurize2d_isplinest_init(headerin,headeout,p)
  end if
  else
  allocate( splngrid(n1*n2,2) )
  do iy=1,n2; 
  do ix=1,n1;
    splngrid((iy-1)*n1+ix,:) = (/ (o1+(ix-1)*d1) , (o2+(iy-1)*d2) /);
  end do
  end do
  if (p.eq.0.) then
    call regurize2d_rsplines_init (headerin,headeout,splngrid)
  else;
    call regurize2d_rsplinest_init(headerin,headeout,splngrid,p)
  end if
  end if
  
  
  ! Read, Regularize and Write
  call sreed('in' ,datain,4*size(datain))
  
  if (mode.ne.2) then
  if (p.eq.0.) then;
    call regurize2d_isplines (datain,datout)
  else;
    call regurize2d_isplinest(datain,datout)
  end if
  else
  if (p.eq.0.) then;
    call regurize2d_rsplines (datain,datout)
  else;
    call regurize2d_rsplinest(datain,datout)
  end if
  end if
  
  call srite('out',datout,4*size(datout))
  
  ! Deallocate and exit
  call regurize2d_splines_close()  
  deallocate(datain)
  deallocate(datout)
  deallocate(headerin)
  deallocate(headeout)
  if (allocated(splngrid)) deallocate(splngrid)
  call exit()
  
end program
