program sdrTapers
! Tape a datafile
  ! taper types: 
  ! 1) 'zero'            --  needed: 'nothing' 
  ! 2) 'rect'            --  needed: t1,t2 
  ! 3) 'linear_up'       --  needed: t1,t2
  ! 4) 'linear_down'     --  needed: t1,t2
  ! 5) 'pyramid'         --  needed: t1,t2
  ! 6) 'pyramid_band'    --  needed: t1,t2,t3,t4
  ! 7) 'hann'            --  needed: t1,t2
  ! 8) 'hann_half'       --  needed: t1,t2
  ! 9) 'hann_band'       --  needed: t2,t2,t3,t4
  ! 10) 'cosine'         --  needed: t1,t2
  ! 11) 'gaussian'       --  needed: t1,t2,sigma,spread
  ! 12) 'gaussian_half'  --  needed: t1,t2,sigma,spread
  ! 13) 'gaussian_block' --  needed: t1,t2,t3,t4,sigma,spread
  !
  ! All tapers have the form of a band or lowcut,
  ! Notch tapers are constructed with inv=1 option.
  ! Sum result onto old data using    add=1 option.
  
  ! Multiple calls take input and overwrite if add=1

  use sep                      ! Main SEP module
  use tapers_mod
!  use mdtapers_mod
  implicit none
  
  real, allocatable               :: taper(:,:,:),taper_old(:,:,:)
  integer                         :: n(3), ii
  integer                         :: type(3),dir(3)
  real                            :: d(3),o(3)
  real                            :: t1(3),t2(3),t3(3),t4(3)
  integer                         :: t1_ind(3),t2_ind(3),t3_ind(3),t4_ind(3)
  logical                         :: add,inv,inv_3(3)
  real                            :: sigma(3),scale(3)
  
  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  ! read from input .H file or parameter
  if (exist_file ("old")) then
    call from_aux  ( "old","n1", n(1) )
    call from_aux  ( "old","n2", n(2) )
    call from_aux  ( "old","n3", n(3) )
    call from_aux  ( "old","d1", d(1) )
    call from_aux  ( "old","d2", d(2) )
    call from_aux  ( "old","d3", d(3) )
    call from_aux  ( "old","o1", o(1) )
    call from_aux  ( "old","o2", o(2) )
    call from_aux  ( "old","o3", o(3) )
  else
    call from_param ("n1",n(1),1024)
    call from_param ("n2",n(2),1   )
    call from_param ("n3",n(3),1   )
    call from_param ("d1",d(1),1.  )
    call from_param ("d2",d(2),1.  )
    call from_param ("d3",d(3),1.  )
    call from_param ("o1",o(1),0.  )
    call from_param ("o2",o(2),0.  )
    call from_param ("o3",o(3),0.  )
  end if
  
  ! Read input parameters for taper, there is lots of options :-)
 ! call from_param ("dir"  ,dir, (/ 1,2,3 /) )
  call from_param ("type" ,type,(/ 0,0,0 /) )
  call from_param ("inv"  ,inv,.false. )
  call from_param ("add"  ,add,.true.  )
  call from_param ("sigma",sigma,(/ .1, .1, .1 /))
  call from_param ("scale",scale,(/ 1., 1., 1. /))
  call from_param ("dir"  ,dir  ,(/ 1 , 2 , 3  /))
  
  call from_param ("t1",t1, o               )
  call from_param ("t2",t2, o + d*real(n-2) )
  call from_param ("t3",t3, o + d*real(n-2) )
  call from_param ("t4",t4, o + d*real(n-1) )
  
  ! Write to auxiliary .H file
  call to_history ("n1",n(1))  
  call to_history ("n2",n(2))
  call to_history ("n3",n(3))
  call to_history ("d1",d(1))
  call to_history ("d2",d(2))
  call to_history ("d3",d(3))
  call to_history ("o1",o(1))
  call to_history ("o2",o(2))
  call to_history ("o3",o(3))
  
  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles

  ! Make sure that no taper specification gives a 1 in that direction
  do ii=1,3;
    if (type(ii)==0) then; inv_3(ii)=.true.; type(ii)=1
    else; inv_3(ii)=inv; end if
  end do

  ! Determine tie indices
  do ii=1,3
    call taper_indice(t1(ii),t1_ind(ii),d_in=d(ii),o_in=o(ii))
    call taper_indice(t2(ii),t2_ind(ii),d_in=d(ii),o_in=o(ii))
    call taper_indice(t3(ii),t3_ind(ii),d_in=d(ii),o_in=o(ii))
    call taper_indice(t4(ii),t4_ind(ii),d_in=d(ii),o_in=o(ii))
  end do

  ! Allocate Taper Space
  allocate ( taper(n(1),n(2),n(3)) )
  call tapers_init(n1_in=n(1),n2_in=n(2),n3_in=n(3))
  call tapers_interface(3, dir_in=dir,t1_in=t1_ind,t2_in=t2_ind,t3_in=t3_ind,t4_in=t4_ind,type_in=type,sigma_in=sigma,scale_in=scale,inv_in=inv_3)

  ! Retreave constructed taper from taper_mod
  call tapers_gettaper(taper)

  ! Optional summing into old taper
  if (exist_file ("old")) then
    allocate( taper_old(n(1),n(2),n(3)) )
    call sep_read(taper_old,"old")
    if (add) then; taper = taper + taper_old; end if
  end if

  call sep_write(   taper   )

  ! Deallocate memory  
  deallocate(     taper     )
  if (exist_file ("in")) then
    deallocate( taper_old )
  end if

  call tapers_close()
  
  ! Force clean exit
  call exit (0)
  
end program


