% Script to prepare the inputs to Matt's package from SEPlib files
clear all; close all; clc;
addpath ./Src/sepmatlab

est=sread('estimates2.matlab');

% Comes as Log10(velocity)
dest = est.data; %-log10(est.data);

data1 = 10.^(-dest);  [dave1,dstd1] = normfit(data1);  %       slowness
data2 = 10.^( dest);  [dave2,dstd2] = normfit(data2);  %       velocity
data3 =      -dest;   [dave3,dstd3] = normfit(data3);  % Log10 slowness
data4 =       dest;   [dave4,dstd4] = normfit(data4);  % Log10 velocity

% Convert to velocity space
disp('Analysis in 4 different domains')
disp(['Normal distribution fit in slowness space       - mean: ' num2str(dave1) ',  std-dev: ' num2str(dstd1)]);
disp(['Normal distribution fit in velocity space       - mean: ' num2str(dave2) ',  std-dev: ' num2str(dstd2)]);
disp(['Normal distribution fit in Log10 slowness space - mean: ' num2str(dave3) ',  std-dev: ' num2str(dstd3)]);
disp(['Normal distribution fit in Log10 velocity space - mean: ' num2str(dave4) ',  std-dev: ' num2str(dstd4)]);
disp('')
disp('Equivalent numbers in velocity units:')
disp(['Normal distribution fit in slowness space       - mean: ' num2str(1/dave1)   ',  std-dev: ' num2str((dave1)^(-2)*dstd1)]);
disp(['Normal distribution fit in velocity space       - mean: ' num2str(dave2)     ',  std-dev: ' num2str(dstd2)]);
disp(['Normal distribution fit in Log10 slowness space - mean: ' num2str(10^-dave3) ',  std-dev: ' num2str(.5*10^-dave3*(10^(dstd3)-10^(-dstd3)))]);
disp(['Normal distribution fit in Log10 velocity space - mean: ' num2str(10^dave4)  ',  std-dev: ' num2str(.5*10^dave4 *(10^dstd4  -10^(-dstd4)))]);
disp('')

fsize=12;

% plot data histograms and Normal distribution fits
figure; hold on;
a = histc(data1,0.002+[0:30]*.0001); bar(0.002+.0001/2+[0:30]*.0001,a);
vec = 0.002+[0:120]*.000025;
mod = normpdf(vec,dave1,dstd1);
mod = mod/sum(mod)*sum(a)*4;
plot(vec,mod,'LineWidth',4); axis([min(vec) max(vec) 0 350]);
xlabel('SNR bin  [s/m]','Fontsize',fsize,'FontWeight','bold'); ylabel('log(bin count)','Fontsize',fsize,'FontWeight','bold');
set(gca,'Fontsize',fsize,'FontWeight','bold');% title('a)');
fillpage(gcf, 'margins', [0 0 0 0], 'papersize', [9 6]);
print(gcf, '-depsc2', 'estimates2a.eps');

figure; hold on;
a = histc(data2,150+[0:30]*10); bar(150+10/2+[0:30]*10,a); %axis([.06 .1 0 500]);
vec = 150+[0:120]*2.5;
mod = normpdf(vec,dave2,dstd2);
mod = mod/sum(mod)*sum(a)*4;
plot(vec,mod,'LineWidth',4); axis([min(vec) max(vec) 0 400]);
xlabel('SNR bin  [m/s]','Fontsize',fsize,'FontWeight','bold'); ylabel('log(bin count)','Fontsize',fsize,'FontWeight','bold');
set(gca,'Fontsize',fsize,'FontWeight','bold');% title('a)');
fillpage(gcf, 'margins', [0 0 0 0], 'papersize', [9 6]);
print(gcf, '-depsc2', 'estimates2b.eps');

figure; hold on;
a = histc(data3,-2.75+[0:25]*.02); bar(-2.75+.02/2+[0:25]*.02,a); %axis([-2.75 -2.25 0 600]);
vec = -2.7+[0:100]*.005;
mod = normpdf(vec,dave3,dstd3);
mod = mod/sum(mod)*sum(a)*4;
plot(vec,mod,'LineWidth',4); axis([-2.7001 -2.3 0 500]);
xlabel('SNR bin  [-]','Fontsize',fsize,'FontWeight','bold'); ylabel('log(bin count)','Fontsize',fsize,'FontWeight','bold');
set(gca,'Fontsize',fsize,'FontWeight','bold'); %title('c)');
fillpage(gcf, 'margins', [0 0 0 0], 'papersize', [9 6]);
print(gcf, '-depsc2', 'estimates2c.eps');

figure; hold on;
a = histc(data4,2.3+[0:25]*.02); bar(2.3+.02/2+[0:25]*.02,a); %axis([2.3 2.75 0 600]);
vec = 2.2+[0:100]*.005;
mod = normpdf(vec,dave4,dstd4);
mod = mod/sum(mod)*sum(a)*4;
plot(vec,mod,'LineWidth',4); axis([min(vec) max(vec) 0 500]);
xlabel('SNR bin  [-]','Fontsize',fsize,'FontWeight','bold'); ylabel('log(bin count)','Fontsize',fsize,'FontWeight','bold');
set(gca,'Fontsize',fsize,'FontWeight','bold'); %title('d)');
fillpage(gcf, 'margins', [0 0 0 0], 'papersize', [9 6]);
print(gcf, '-depsc2', 'estimates2d.eps');


figure; hold on;
qqplot(data1); axis tight;
set(gca,'Fontsize',fsize,'FontWeight','bold');
print(gcf, '-depsc2', 'qq1b.eps');

figure; hold on;
qqplot(data2); axis tight;
set(gca,'Fontsize',fsize,'FontWeight','bold');
print(gcf, '-depsc2', 'qq2b.eps');

figure; hold on;
qqplot(data4); axis tight;
set(gca,'Fontsize',fsize,'FontWeight','bold');
print(gcf, '-depsc2', 'qq2c.eps');
